## @file
# Compute GTI the standard way

## @method void compute_gti()
# Compute GTI the standard way
# @return Void

#========================================================================
sub compute_gti(){
    print "\n----> Working out GTI .....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Working out GTI ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for GTI ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for GTI ..... ";}
    
    use vars qw(
		$epic_GTI_lightcurvefile 
		$epic_GTI_file 
		$PG_flagCall
		 );

#.. Parameters for GTI 
    
    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    
    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {&error_code(3);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {&error_code(3);}
    &set_odfingest($odfingestfile);
    
#.. Print sas_setup    
    &print_sas_setup();

#.. Flag to check if compute_gti has been called by PG optimization (PG_flag = 1 (true))
#.. or it is the standard gti call


#.. Get event files
   my @files = &get_EPIC_eventfiles();
	
 	my %epicFileMap = get_EPIC_Exposures(@files);
	my $inst;
	my $exposure;
	my $mode;
   if ( $#files == 0 ) {&error_code(4);} 
	
	foreach my $file (@files) {

		($inst,$exposure,$mode) = &get_File_Info($file);
 		if ( $gv->$inst eq "no" ) {
			print
		"INSTRUMENT $inst [exposure=$exposure]: Not selected for calculating GTI\n";
			print "Skipping to the next exposure...\n";
		}
		else {
	 		print "\n----> Calculating GTI: $file [$inst,$exposure,$mode] .....\n";
	  	  	if ($Log_Flag == 0) {print MYOUTFILE "\n----> Calculating GTI: $file [$inst,$exposure,$mode] ....\n";}
	 		
			
			my $instrument = "";
			if ($inst eq "EPN") {		 
				$instrument = "pn";
			} elsif ($inst eq "EMOS1") {
				$instrument = "m1";
			} elsif ($inst eq "EMOS2") {
				$instrument = "m2";
			}
	 		
	 		#.. Get event files
	  		@test_epicevt=&test_epiceventfiles($inst,$exposure,$mode);
	    	$file=$test_epicevt[1];
			$epic_event_list = $file;
			
			#.. Get Information from event files 
	    	if ($Log_Evt_Flag == 0) {
				my $event_log=$results_directory."event_".$source_name."\.log";
				if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}    
				open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";    #open for write, overwrite
	    	}
	
		    if ($file) {&event_file_inf($epic_event_list,$instrument);}
		
	    	if ($Log_Evt_Flag == 0) {
				close(EVTLOG);
				$Log_Evt_Flag = 1;	
	    	}
		    
		    my $optimize_SN_flag = $gv->GTI_optimize_SN_flag;
			#.. This routine will work out a new PN_Rate, MOS1_Rate and MOS2_Rate
		
			if ($PG_flagCall == 0)
			{
				#.. If the interactive mode is enabled, open ds9 with the eventlist and let the 
				#.. user to play with the src and bkg regions.		
			
				if($gv->epic_instrument eq $inst) 
				{
					&calculate_Signal2Noise($epic_event_list,$instrument,$exposure,$mode);
				}
				else 
				{
					if (($optimize_SN_flag eq "yes")  && $file)
					{
						&PG_gti_filter($instrument,$exposure,$mode);			
					}
				}      
			}
			
		    chdir($gti_directory);
		    my $currentEnerCut  = "GTI_".$instrument."energy_cut";
		    my $epic_energy_cut = $gv->$currentEnerCut; 
			my $currentRate  = "GTI_".$instrument."_Rate";
	    	my $epic_Rate          = $gv->$currentRate;
	    	my $timebin          = $gv->GTI_timebin;        
	
		    print "\n   \#> Parameters for GTI\n";
		    if ($optimize_SN_flag eq "F"){
			print "           Background Light Curve Parameters: \n";
			print "             $instrument Energy Cut  : $epic_energy_cut eV\n";
		    }	
		    print "           Definition of GTI: \n";
		    print "             $instrument Rate   <    : $epic_Rate cts/sec\n";
		    print "             Time Bin       : $timebin sec\n\n";
	
	   		if ($Log_Flag == 0) {
				print MYOUTFILE "\n   \#> Parameters for GTI\n";
				if ($optimize_SN_flag eq "F"){
			    	print MYOUTFILE "           Background Light Curve Parameters: \n";
		    		print MYOUTFILE "             $instrument Energy Cut  : $epic_energy_cut eV\n";
				}	
				print MYOUTFILE "           Definition of GTI: \n";
				print MYOUTFILE "             $instrument Rate   <    : $epic_Rate cts/sec\n";
				print MYOUTFILE "             Time Bin       : $timebin sec\n\n";
	    	}
	#.. File definition
	
		
			$epic_GTI_lightcurvefile = $gti_directory.$instrument."_".$exposure."_".$mode."_back_lightc.fits";
			$epic_GTI_file           = &set_gtifilename($instrument,$exposure,$mode);
			
	    	
	    	#.. Create EPIC GTI File
	    	if ($Log_GTIEvt_Flag == 0) {
				my $gtievent_log=$results_directory."gtievent_".$source_name."\.log";
				if (-e $gtievent_log) {unlink("$gtievent_log") or die "Cannot rm file: $!";}    
					open(GTILOG, "> $gtievent_log") or die "Cant open output file: $! \n";    #open for write, overwrite
			}
	
		    if ($file){
		    	my $expression ="";
		    	
				if ($instrument eq "pn")
				{
					$expression = "\#XMMEA_EP&&(PI in [$epic_energy_cut:12000])&&(PATTERN==0)";				
				}
				else
				{
					$expression = "\#XMMEA_EM&&(PI>$epic_energy_cut)&&(PATTERN==0)"
				}
	
				@args = ("evselect","table=$epic_event_list:EVENTS","expression=$expression"
					 ,"rateset=$epic_GTI_lightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
					 ,"makeratecolumn=yes");
				system(@args)== 0 or &error_code(4);
	
				@args = ("tabgtigen","table=$epic_GTI_lightcurvefile","expression=RATE<$epic_Rate","gtiset=$epic_GTI_file");
				system(@args)== 0 or &error_code(4);
				&gti_file_inf($epic_GTI_file,$instrument);	
	    	}
	    
	    	if ($Log_GTIEvt_Flag == 0) {
				close(GTILOG);
				$Log_GTIEvt_Flag = 1;	
	    	}
		}
	} # end of EPIC file loop

    &error_code(0);
    return(0);
}

## @method set_gtifilename($instr,$exposure,$mode)
# Set the name of the GTI file for EPIC
# @param instr Instrument (pn,mos1,mos2)
# @param exposure
# @param mode Observing Mode (imaging,timing)
# @return GTI File name for a EPIC instr/mode

#========================================================================
sub set_gtifilename(){

    my $instr = $_[0];
	my $exposure = $_[1];
    my $mode  = $_[2];

    my $gtifile = $instr."_gti_".$exposure."_".$mode.".fits";
    
    return($gtifile)
}

## @method test_epic_gtifile()
# Test for existance of EPIC-pn GTI file and get the name of the GTI file if it exist
# @param $instrument
# @param $exposure
# @param $mode
# @return EPIC-pn GTI File name

#========================================================================
sub test_epic_gtifile(){
	
	my $instrument = $_[0];
	my $exposure = $_[1];
	my $mode = $_[2];
	
    chdir ($gti_directory);

    my $flag = 0;
    my $file = "No file";

    $epic_GTI_file=&set_gtifilename($instrument,$exposure,$mode);
    if (-e $epic_GTI_file) {
		print "   \#> $mode $instrument GTI File found: $epic_GTI_file \n";
		if ($Log_Flag == 0) {print MYOUTFILE "   \#> $mode $instrument GTI File found: $epic_GTI_file \n";}
		$flag=1;
    } else {
		print "   \#> $mode $instrument GTI File not found \n";
		if ($Log_Flag == 0) {print MYOUTFILE "   \#> $mode $instrument GTI File not found \n";}	
    }

    if ($flag) {$file=$gti_directory.$epic_GTI_file};
    return($flag,$file);     

}


## @method void produce_gticleanfile($even_file,$even_gtifile,$gti_file,$subMode,$instr)
# Given a gti file and event file, produce a gti clean event file.
# @param even_file Event File Name
# @param even_gtifile Event GTI Clean File Name
# @param gti_file GTI File Name
# @param subMode instrument mode
# @param instr Instrument
# @return Void

#========================================================================
sub produce_gticleanfile(){

    my $even_file    = $_[0];
    my $even_gtifile = $_[1]; 
    my $gti_file     = $_[2];
    my $subMode      = $_[3];
    my $instr        = $_[4];
    

    print "   \#> Producing GTI clean event file from $even_file .... \n";
  	
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Producing GTI clean event file from $even_file .... \n";}
   
    if ($instr eq "pn") {$expr="\#XMMEA_EP && gti($gti_file,TIME) && (FLAG==0) && (PATTERN<=4) && (PI>100)";}
    if ($instr =~ m/m/) {
    	if (uc($subMode) eq "TIMING")
    		{$expr="\#XMMEA_EM && gti($gti_file,TIME) && (FLAG==0) && (PATTERN==0) && (PI>100)";}
    	else
    		{$expr="\#XMMEA_EM && gti($gti_file,TIME) && (FLAG==0) && (PATTERN<=12) && (PI>100)";}
    }	

    @args = ("evselect table=\"$even_file:EVENTS\" withfilteredset=Y filteredset=$even_gtifile destruct=Y keepfilteroutput=T expression=\"$expr\"");
    system(@args) == 0 or &error_code(6);

    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

    return(1);
}
1;
