
sub prepare_spectra() {
	my $instrument          = $_[0];
	my $exposure            = $_[1];
	my $mode                = $_[2];
	my $epic_obsmode        = $_[3];
	my $epic_obssubmode     = $_[4];
	my $type                = $_[5];
	my $ori_instrument_name = $instrument;

	#.. Retrieve source OBSID
	my $source_obsID = $gv->SRC_OBSID;

	#.. Initial value set to 1 (true)
	$epic_spectral_flag = 1;

	my $src_region = $type . "_" . $instrument . "_srcregion";
	my $bkg_region = $type . "_" . $instrument . "_bkgregion";

	#.. Source Regions...
	my $src_region_X       = $type . "_" . $instrument . "_srcregion_X";
	my $src_region_Y       = $type . "_" . $instrument . "_srcregion_Y";
	my $src_region_R1      = $type . "_" . $instrument . "_srcregion_R1";
	my $src_region_R2      = $type . "_" . $instrument . "_srcregion_R2";
	my $src_region_XLength = $type . "_" . $instrument . "_srcregion_XLength";
	my $src_region_YLength = $type . "_" . $instrument . "_srcregion_YLength";

	#.. Bkg Regions...
	my $bkg_region_X       = $type . "_" . $instrument . "_bkgregion_X";
	my $bkg_region_Y       = $type . "_" . $instrument . "_bkgregion_Y";
	my $bkg_region_R1      = $type . "_" . $instrument . "_bkgregion_R1";
	my $bkg_region_R2      = $type . "_" . $instrument . "_bkgregion_R2";
	my $bkg_region_XLength = $type . "_" . $instrument . "_bkgregion_XLength";
	my $bkg_region_YLength = $type . "_" . $instrument . "_bkgregion_YLength";

	#.. Area Factor
	my $epic_areafactor = $type . "_" . $instrument . "_areafactor";

	if ( $epic_obsmode eq 'IMAGING' || $epic_obsmode eq 'imaging' ) {

		#.. Get shape of src and bkg regions:
		$src_region_shape = $gv->{$src_region};
		$bkg_region_shape = $gv->{$bkg_region};

		#.. Run eregionanalyze at the source ra and dec position.
		my $epic_bore_img_x         = $instrument . "_bore_img_x";
		my $epic_bore_img_y         = $instrument . "_bore_img_y";
		my $epic_bore_optrad_asec   = $instrument . "_bore_optrad_asec";
		my $epic_bore_optradius     = $instrument . "_bore_optradius";
		my $epic_bore_img_xbkg      = $instrument . "_bore_img_xbkg";
		my $epic_bore_img_ybkg      = $instrument . "_bore_img_ybkg";
		my $epic_bkg_inner_rad_asec = $instrument . "_bkg_inner_rad_asec";
		my $epic_bkg_inner_rad      = $instrument . "_bkg_inner_rad";
		my $epic_bkg_outer_rad_asec = $instrument . "_bkg_outer_rad_asec";
		my $epic_bkg_outer_rad      = $instrument . "_bkg_outer_rad";

		(
			$epic_bore_img_x,         $epic_bore_img_y,
			$epic_bore_optrad_asec,   $epic_bore_optradius,
			$epic_bore_img_xbkg,      $epic_bore_img_ybkg,
			$epic_bkg_inner_rad_asec, $epic_bkg_inner_rad,
			$epic_bkg_outer_rad_asec, $epic_bkg_outer_rad
		) = ( 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
		(
			$epic_bore_img_x,         $epic_bore_img_y,
			$epic_bore_optrad_asec,   $epic_bore_optradius,
			$epic_bore_img_xbkg,      $epic_bore_img_ybkg,
			$epic_bkg_inner_rad_asec, $epic_bkg_inner_rad,
			$epic_bkg_outer_rad_asec, $epic_bkg_outer_rad
		  )
		  = &run_eregionanalyse( $epic_gti_image_file, $ra_anal, $dec_anal,
			$instrument, $epic_obssubmode, $epic_eregionanal_file, $type )
		  ;    # EPIC

		if ( $epic_bore_optrad_asec == 300 ) {
			$epic_spectral_flag = 0;
		} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

		#.. Check values of parameters defining source and background regions

		$src_XC = $gv->$src_region_X;
		$src_YC = $gv->$src_region_Y;
		$src_R1 = $gv->$src_region_R1;
		$src_R2 = $gv->$src_region_R2;
		$src_XL = $gv->$src_region_XLength;
		$src_YL = $gv->$src_region_YLength;
		$bkg_XC = $gv->$bkg_region_X;
		$bkg_YC = $gv->$bkg_region_Y;
		$bkg_R1 = $gv->$bkg_region_R1;
		$bkg_R2 = $gv->$bkg_region_R2;
		$bkg_XL = $gv->$bkg_region_XLength;
		$bkg_YL = $gv->$bkg_region_YLength;

		if ( $src_XC == 0 && $src_YC == 0 )
		{    # if values are set to cero take the default; from eregionanalyse
			$gv->$src_region_X($epic_bore_img_x);
			$gv->$src_region_Y($epic_bore_img_y);
		}

		if ( $bkg_XC == 0 && $bkg_YC == 0 )
		{    # if values are set to cero take the default; from eregionanalyse
			$gv->$bkg_region_X($epic_bore_img_xbkg);
			$gv->$bkg_region_Y($epic_bore_img_ybkg);
		}

		#.. SRC Region
		if ( $src_region_shape eq "circle" ) {
			$gv->$src_region_R1(0);
			if ( $src_R2 == 0 )
			{  # if values are set to cero take the default; from eregionanalyse
				$gv->$src_region_R2($epic_bore_optradius);
			}

			$src_area =
			  $pi * $gv->$src_region_R2 * $gv->$src_region_R2 -
			  $pi * $gv->$src_region_R1 * $gv->$src_region_R1;
		}
		elsif ( $src_region_shape eq "annulus" ) {
			if ( $src_R1 == 0 || $src_R2 == 0 ) {
				print "       $instrument SRC Region should be defined \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
					  "       $instrument SRC Region should be defined \n";
				}
				&error_code(6);
			}
			$src_area =
			  $pi * $gv->$src_region_R2 * $gv->$src_region_R2 -
			  $pi * $gv->$src_region_R1 * $gv->$src_region_R1;
		}
		elsif ( $src_region_shape eq "box" ) {
			if ( $src_XL == 0 || $src_YL == 0 ) {
				print "       $instrument SRC Region should be defined \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "       PN SRC Region should be defined \n";
				}
				&error_code(6);
			}
			$src_area = $gv->$src_region_XLength * $gv->$src_region_YLength;
		}
		else {
			print "       $instrument SRC Region not recognized \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "       instrument SRC Region not recognized \n";
			}
			&error_code(6);
		}

		#.. BKG Region
		if ( $bkg_region_shape eq "circle" ) {
			$gv->$bkg_region_R1(0);
			if ( $bkg_R2 == 0 ) {
				print "       $instrument BKG Region should be defined \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
					  "       $instrument BKG Region should be defined \n";
				}
				&error_code(6);
			}

			$bkg_area =
			  $pi * $gv->$bkg_region_R2 * $gv->$bkg_region_R2 -
			  $pi * $gv->$bkg_region_R1 * $gv->$bkg_region_R1;
		}
		elsif ( $bkg_region_shape eq "annulus" ) {
			if ( $bkg_R1 == 0 || $bkg_R2 == 0 )
			{  # if values are set to cero take the default; from eregionanalyse
				$gv->$bkg_region_R1($epic_bkg_inner_rad);
				$gv->$bkg_region_R2($epic_bkg_outer_rad);
			}
			$bkg_area =
			  $pi * $gv->$bkg_region_R2 * $gv->$bkg_region_R2 -
			  $pi * $gv->$bkg_region_R1 * $gv->$bkg_region_R1;
		}
		elsif ( $bkg_region_shape eq "box" ) {
			if ( $bkg_XL == 0 || $bkg_YL == 0 ) {
				print "       $instrument BKG Region should be defined \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
					  "       $instrument BKG Region should be defined \n";
				}
				&error_code(6);
			}
			$bkg_area = $gv->$bkg_region_XLength * $gv->$bkg_region_YLength;
		}
		else {
			print "       $instrument BKG Region not recognized \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "       $instrument BKG Region not recognized \n";
			}
			&error_code(6);
		}

		$gv->$epic_areafactor( $src_area / $bkg_area );

		if ( $gv->$src_region_X == 0 && $gv->$src_region_Y == 0 ) {
			$epic_spectral_flag = 0;
		}    # Something has gone wrong so wont produce spectra.

	}
	elsif ( $epic_obsmode eq 'TIMING' || $epic_obsmode eq 'timing' ) {

		if ( $ori_instrument_name eq 'SPC' ) {
			if ( $instrument eq 'm1' || $instrument eq 'm2' ) {
				my $low_ener = $type . "_" . $instrument . "_e1_low";
				if ( $gv->$low_ener <= 0.5 ) { $gv->$low_ener(0.5) }
				; # If Timing mode, use 0.5. Kev for low energy limit for the fit,
			}    # or whatever the user specifies as long as is above 0.5 keV
		}

		$gv->$src_region("box");
		$src_XC = $gv->$src_region_X;
		$src_YC = $gv->$src_region_Y;
		$src_XL = $gv->$src_region_XLength;
		$src_YL = $gv->$src_region_YLength;
		
		if (   ( $src_XC == 0 && $src_YC == 0 )
			&& ( $src_XL == 0 && $src_XL == 0 ) )
		{
			if ( $instrument eq 'm1' || $instrument eq 'm2' ) {
				$t_start =
				  $obsid_instruments{$source_obsID}{$instrument}
				  {'starting_time'};
				$t_end =
				  $obsid_instruments{$source_obsID}{$instrument}{'ending_time'};
				$interval     = $t_end - $t_start;
				$mid_interval = $t_start + $interval / 2.;
				$gv->$src_region_X(299.5);
				$gv->$src_region_Y($mid_interval);
				$gv->$src_region_XLength(20);
				$gv->$src_region_YLength($interval);
			}
			else {
				$gv->$src_region_X(37.);
				$gv->$src_region_Y(101.);
				$gv->$src_region_XLength(16);
				$gv->$src_region_YLength(199);
			}
		}

		$gv->$bkg_region("box");
		$bkg_XC = $gv->$bkg_region_X;
		$bkg_YC = $gv->$bkg_region_Y;
		$bkg_XL = $gv->$bkg_region_XLength;
		$bkg_YL = $gv->$bkg_region_YLength;
		if (   ( $bkg_XC == 0 && $bkg_YC == 0 )
			&& ( $bkg_XL == 0 && $bkg_XL == 0 ) )
		{
			if ( $instrument eq 'm1' || $instrument eq 'm2' ) {
				$gv->$bkg_region_X(259.0)
				  ;    # lowest column value in MOS Timing 256
				$gv->$bkg_region_Y( $gv->$src_region_Y );
				$gv->$bkg_region_XLength( $gv->$src_region_XLength );
				$gv->$bkg_region_YLength( $gv->$src_region_YLength );
			}
			else {
				$gv->$bkg_region_X( $gv->$src_region_XLength / 2. + 0.5 )
				  ;    # +0.5 to keep it away from the edge of the detector
				$gv->$bkg_region_Y( $gv->$src_region_Y );
				$gv->$bkg_region_XLength( $gv->$src_region_XLength );
				$gv->$bkg_region_YLength( $gv->$src_region_YLength );
			}
		}

		$src_area = $gv->$src_region_XLength * $gv->$src_region_YLength;
		$bkg_area = $gv->$bkg_region_XLength * $gv->$bkg_region_YLength;
		$gv->$epic_areafactor( $src_area / $bkg_area );

	}
	else {
		print "       $instrument Mode ($epic_obsmode) not recognized \n";
		exit 1;
		if ( $Log_Flag == 0 ) {
			print MYOUTFILE "       $instrument Mode not recognized \n";
		}
		&error_code(6);
	}

	print "       $instrument \n";
	print "         $instrument Region SRC : ", $gv->{$src_region},   "\n";
	print "         $instrument SRC XC     : ", $gv->{$src_region_X}, "\n";
	print "         $instrument SRC YC     : ", $gv->{$src_region_Y}, "\n";
	if ( $gv->{$src_region} eq 'circle' || $gv->{$src_region} eq 'annulus' ) {
		print "         $instrument SRC RC     : ", $gv->{$src_region_R1},
		  " - ", $gv->{$src_region_R2}, "\n";
	}
	if ( $gv->{$src_region} eq 'box' ) {
		print "        $instrument SRC BOX    : ", $gv->{$src_region_XLength},
		  " - ", $gv->{$src_region_YLength}, "\n";
	}
	print "         $instrument Region BKG : ", $gv->{$bkg_region},   "\n";
	print "         $instrument BKG XC     : ", $gv->{$bkg_region_X}, "\n";
	print "         $instrument BKG YC     : ", $gv->{$src_region_Y}, "\n";
	if ( $gv->{$bkg_region} eq 'circle' || $gv->{$bkg_region} eq 'annulus' ) {
		print "         $instrument BKG RC     : ", $gv->{$bkg_region_R1},
		  " - ", $gv->{$bkg_region_R2}, "\n";
		if ( $gv->{$src_region_R2} > $gv->{$bkg_region_R1} ) {
			print
"         WARNING: $instrument regions : The inner background region radii is smaller than the source region radii \n";
		}
	}
	if ( $gv->{$bkg_region} eq 'box' ) {
		print "         $instrument BKG BOX    : ", $gv->{$bkg_region_XLength},
		  " - ", $gv->{$bkg_region_YLength}, "\n";
	}
	print "         $instrument BKG AREA SCALE FACTOR : ",
	  $gv->{$epic_areafactor}, " (Only area taken into account)\n";
	if ( $gv->{$src_region_R2} == 6000 ) {
		print
"         WARNING : $instrument regions : Maximum allowed radius reached";
	}
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "       $instrument \n";
		print MYOUTFILE "         $instrument Region SRC  : ",
		  $gv->{$src_region}, "\n";
		print MYOUTFILE "         $instrument SRC XC      : ",
		  $gv->{$src_region_X}, "\n";
		print MYOUTFILE "         $instrument SRC YC      : ",
		  $gv->{$src_region_Y}, "\n";
		if ( $gv->{$src_region} eq 'circle' || $gv->{$src_region} eq 'annulus' )
		{
			print MYOUTFILE "         $instrument SRC RC      : ",
			  $gv->{$src_region_R1}, " - ", $gv->{$src_region_R2}, "\n";
		}
		if ( $gv->{$src_region} eq 'box' ) {
			print MYOUTFILE "         $instrument SRC BOX     : ",
			  $gv->{$src_region_XLength}, " - ", $gv->{$src_region_YLength},
			  "\n";
		}
		print MYOUTFILE "         $instrument Region BKG  : ",
		  $gv->{$bkg_region}, "\n";
		print MYOUTFILE "         $instrument BKG XC      : ",
		  $gv->{$bkg_region_X}, "\n";
		print MYOUTFILE "         $instrument BKG YC      : ",
		  $gv->{$src_region_Y}, "\n";
		if ( $gv->{$bkg_region} eq 'circle' || $gv->{$bkg_region} eq 'annulus' )
		{
			print MYOUTFILE "         $instrument BKG RC      : ",
			  $gv->{$bkg_region_R1}, " - ", $gv->{$bkg_region_R2}, "\n";
			if ( $gv->{$src_region_R2} > $gv->{$bkg_region_R1} ) {
				print MYOUTFILE
"         WARNING: $instrument regions : The inner background region radii is smaller than the source region radii \n";
			}
		}
		if ( $gv->{$bkg_region} eq 'box' ) {
			print MYOUTFILE "         $instrument BKG BOX     : ",
			  $gv->{$bkg_region_XLength}, " - ", $gv->{$bkg_region_YLength},
			  "\n";
		}
		print MYOUTFILE "         $instrument BKG AREA SCALE FACTOR : ",
		  $gv->{$epic_areafactor}, " (Only area taken into account)\n";
		if ( $gv->{$src_region_R2} == 6000 ) {
			print MYOUTFILE
"         WARNING : $instrument regions : Maximum allowed radius reached";
		}
	}

	return ($epic_spectral_flag);
}

sub create_spec_plots_and_regions() {
	my $instrument = $_[0];
	my $exposure   = $_[1];
	my $mode       = $_[2];
	my $type       = $_[3];

	#.. SRC and BKG regions
	my $src_region = $type . "_" . $instrument . "_srcregion";
	my $bkg_region = $type . "_" . $instrument . "_bkgregion";

	#.. Source Regions...
	my $src_region_X       = $type . "_" . $instrument . "_srcregion_X";
	my $src_region_Y       = $type . "_" . $instrument . "_srcregion_Y";
	my $src_region_R1      = $type . "_" . $instrument . "_srcregion_R1";
	my $src_region_R2      = $type . "_" . $instrument . "_srcregion_R2";
	my $src_region_XLength = $type . "_" . $instrument . "_srcregion_XLength";
	my $src_region_YLength = $type . "_" . $instrument . "_srcregion_YLength";

	#.. Bkg Regions...
	my $bkg_region_X       = $type . "_" . $instrument . "_bkgregion_X";
	my $bkg_region_Y       = $type . "_" . $instrument . "_bkgregion_Y";
	my $bkg_region_R1      = $type . "_" . $instrument . "_bkgregion_R1";
	my $bkg_region_R2      = $type . "_" . $instrument . "_bkgregion_R2";
	my $bkg_region_XLength = $type . "_" . $instrument . "_bkgregion_XLength";
	my $bkg_region_YLength = $type . "_" . $instrument . "_bkgregion_YLength";

	print "\n   \#> Producing plots and region files ..... \n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "\n   \#> Producing plots and region files ..... \n";
	}

	if ( -e $epic_spectral_region_file ) {
		unlink("$epic_spectral_region_file")
		  or die "Cannot rm file $temp_file: $!";
	}

	if ( -e $epic_spectral_bkgregion_file ) {
		unlink("$epic_spectral_bkgregion_file")
		  or die "Cannot rm file $temp_file: $!";
	}

	if ( -e $epic_spectral_combinedregion_file ) {
		unlink("$epic_spectral_combinedregion_file")
		  or die "Cannot rm file $temp_file: $!";
	}

	#.. SRC
	if ( $gv->{$src_region} eq "circle" || $gv->{$src_region} eq "annulus" ) {
		$x  = $gv->{$src_region_X};
		$y  = $gv->{$src_region_Y};
		$L1 = $gv->{$src_region_R1};
		$L2 = $gv->{$src_region_R2};
	}
	if ( $gv->{$src_region} eq "box" ) {
		$x  = $gv->{$src_region_X};
		$y  = $gv->{$src_region_Y};
		$L1 = $gv->{$src_region_XLength};
		$L2 = $gv->{$src_region_YLength};
	}
	if ($epic_spectral_flag) {
		&addsourcetoregionfile( $epic_spectral_region_file, $x, $y, $L1, $L2,
			"yellow", "", $gv->{$src_region}, "physical" );    # EPIC
	}
	if ($epic_spectral_flag) {
		&addsourcetoregionfile( $epic_spectral_combinedregion_file,
			$x, $y, $L1, $L2, "yellow", "", $gv->{$src_region}, "physical" )
		  ;                                                    #EPIC
	}

	#.. BKG

	if ( $gv->{$bkg_region} eq "circle" || $gv->{$bkg_region} eq "annulus" ) {
		$x  = $gv->{$bkg_region_X};
		$y  = $gv->{$bkg_region_Y};
		$L1 = $gv->{$bkg_region_R1};
		$L2 = $gv->{$bkg_region_R2};
	}
	if ( $gv->{$bkg_region} eq "box" ) {
		$x  = $gv->{$bkg_region_X};
		$y  = $gv->{$bkg_region_Y};
		$L1 = $gv->{$bkg_region_XLength};
		$L2 = $gv->{$bkg_region_YLength};
	}
	if ($epic_spectral_flag) {
		&addsourcetoregionfile( $epic_spectral_bkgregion_file, $x, $y, $L1, $L2,
			"red", "", $gv->{$bkg_region}, "physical" );
	}
	if ($epic_spectral_flag) {
		&addsourcetoregionfile( $epic_spectral_combinedregion_file,
			$x, $y, $L1, $L2, "red", "", $gv->{$bkg_region}, "physical" );
	}

	#.. PS Plots

	if ($epic_spectral_flag) {
		if (-e $epic_gti_smoothimage_file) {
			&produce_ps_file( $epic_gti_smoothimage_file,
				$epic_spectral_combinedregion_file,
				$instrument, $epic_spectrum_psfile );
		}
	}    # PN

}

1;
