## @file
# Produce EPIC event lists

## @method produce_epiceventlist()
# Starts and controls the production of EPIC Event List.
# @return Flags of error/success for EPIC-pn, EPIC-MOS1 and EPIC-MOS2.

#========================================================================
sub produce_epiceventlist() {

	#.. Run epproc and emproc
	print "\n----> Start EPIC Analysis ....\n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "\n----> Start EPIC Analysis ....\n";
	}

	my $source_obsID = $gv->SRC_OBSID;
	my $source_name  = $gv->SRC_Name;
	my $source_ra    = $gv->SRC_RA;
	my $source_dec   = $gv->SRC_DEC;

	#.. Test for ccf file and set ENV variable
	@test_ccf = &test_ccf();
	my $ccffile = $test_ccf[1];
	if ( !$test_ccf[0] ) {
		&run_cifbuild();
		@test_ccf = &test_ccf();
		$ccffile = $test_ccf[1];
	}
	&set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {	
	&run_odfingest();
    }
    @test_odfingest=&test_odfingest();
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(255);}
    &set_odfingest($odfingestfile);
    

	#.. Print sas_setup
	&print_sas_setup();

	#.. Run epproc

	if ( $gv->EPN eq "yes" ) {
		print "\n----> Running epproc ....\n";
		if ( $Log_Flag == 0 ) {
			print MYOUTFILE "\n----> Running epproc ....\n";
		}
		chdir($pn_directory);

		$pnevt_flag          = 0;
		@test_pnevt          = &test_existance_pneventfiles();
		$pn_event_list       = $test_pnevt[1];
		$pntiming_flag       = 0;
		@test_pntimingevt    = &test_existance_pntimingeventfiles();
		$pn_timingevent_list = $test_pntimingevt[1];


		if ( $test_pnevt[0] ) {    # If event file already exist ...
			print "   \#> PN Image Event List already exist: $pn_event_list \n";
			print "   \#> Not running epproc \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "   \#> PN Image Event List alreatestdy exist: $pn_event_list \n";
				print MYOUTFILE "   \#> Not running epproc \n";
			}
			$pnevt_flag = 1;
		}

		if ( $test_pntimingevt[0] ) {   # If timing event file already exist ...
			print
"   \#> PN Timing Event List already exist: $pn_timingevent_list \n";
			print "   \#> Not running epproc \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
"   \#> PN Timing Event List already exist: $pn_timingevent_list \n";
				print MYOUTFILE "   \#> Not running epproc \n";
			}
			$pntiming_flag = 1;
		}

		if ( !$pnevt_flag && !$pntiming_flag )
		{    # If event or timing files do not exist ...

			@args = ("epproc");
			system(@args) == 0
			  or return (
				$pnevt_flag,       $m1evt_flag,       $m2evt_flag,
				$pntimingevt_flag, $m1timingevt_flag, $m2timingevt_flag
			  );
			$pnevt_flag          = 0;
			@test_pnevt          = &test_existance_pneventfiles();
			$pn_event_list       = $test_pnevt[1];
			$pntiming_flag       = 0;
			@test_pntimingevt    = &test_existance_pntimingeventfiles();
			$pn_timingevent_list = $test_pntimingevt[1];

			if ( $test_pnevt[0] )
			{    # Once epproc has run, do I have event files ?
				print
"   \#> After running epproc: PN Image Event List found: $pn_event_list \n";
				print "   \#> epproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
"   \#> After running epproc: PN Image Event List found: $pn_event_list \n";
					print MYOUTFILE "   \#> epproc OK \n";
				}
				$pnevt_flag = 1;
			}
			if ( $test_pntimingevt[0] )
			{    # Once epproc has run, do I have timing event files ?
				print
"   \#> After running epproc: PN Timing Event List found: $pn_timingevent_list \n";
				print "   \#> epproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
"   \#> After running epproc: PN Timing Event List found: $pn_timingevent_list \n";
					print MYOUTFILE "   \#> epproc OK \n";
				}
				$pntiming_flag = 1;
			}

			if ( !$pnevt_flag && !$pntiming_flag ) {
				print
"   \#> After running epproc: No PN Image or Timing Event List were found \n";
				print "   \#> epproc FAILED (no Events file found) \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
"   \#> After running epproc: No PN Image or Timing Event List were found \n";
					print MYOUTFILE
					  "   \#> epproc FAILED (no Events file found) \n";
				}

				#	    &error_code(3) && return;
			}
		}

	&setcoordinates();				
	}



	#.. Run emproc
	if ( ($gv->EMOS1 eq "yes") || ($gv->EMOS2 eq "yes") ) {
		print "\n----> Running emproc ....\n";
		if ( $Log_Flag == 0 ) {
			print MYOUTFILE "\n----> Running emproc ....\n";
		}

		$m1evt_flag          = 0;
		@test_m1evt          = &test_existance_mos1eventfiles();
		$m1_event_list       = $test_m1evt[1];
		$m1timing_flag       = 0;
		@test_m1timingevt    = &test_existance_mos1timingeventfiles();
		$m1_timingevent_list = $test_m1timingevt[1];

		$m2evt_flag          = 0;
		@test_m2evt          = &test_existance_mos2eventfiles();
		$m2_event_list       = $test_m2evt[1];
		$m2timing_flag       = 0;
		@test_m2timingevt    = &test_existance_mos2timingeventfiles();
		$m2_timingevent_list = $test_m2timingevt[1];

		if ( $test_m1evt[0] && $test_m2evt[0] ) {

			print "   \#> Image Event file already exist: \n";
			print "       MOS1: $m1_event_list \n";
			print "       MOS2: $m2_event_list \n";
			print "   \#> Not running emproc \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE "   \#> Image Event file already exist: \n";
				print MYOUTFILE "       MOS1: $m1_event_list \n";
				print MYOUTFILE "       MOS2: $m2_event_list \n";
				print MYOUTFILE "   \#> Not running emproc \n";
			}
			$m1evt_flag = 1;
			$m2evt_flag = 1;
		}
		if ( $test_m1timingevt[0] && $test_m2timingevt[0] ) {

			print "   \#> Timing Event file already exist: \n";
			print "       MOS1: $m1_timingevent_list \n";
			print "       MOS2: $m2_timingevent_list \n";
			print "   \#> Not running emproc \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE "   \#> Timing Event file already exist: \n";
				print MYOUTFILE "       MOS1: $m1_timingevent_list \n";
				print MYOUTFILE "       MOS2: $m2_timingevent_list \n";
				print MYOUTFILE "   \#> Not running emproc \n";
			}
			$m1timingevt_flag = 1;
			$m2timingevt_flag = 1;
		}

		if (   !$m1evt_flag
			&& !$m2evt_flag
			&& !$m1timingevt_flag
			&& !$m2timingevt_flag )
		{

			@args = ("emproc");
			system(@args) == 0
			  or return (
				$pnevt_flag,       $m1evt_flag,       $m2evt_flag,
				$pntimingevt_flag, $m1timingevt_flag, $m2timingevt_flag
			  );

			$m1evt_flag          = 0;
			@test_m1evt          = &test_existance_mos1eventfiles();
			$m1_event_list       = $test_m1evt[1];
			$m1timing_flag       = 0;
			@test_m1timingevt    = &test_existance_mos1timingeventfiles();
			$m1_timingevent_list = $test_m1timingevt[1];

			$m2evt_flag          = 0;
			@test_m2evt          = &test_existance_mos2eventfiles();
			$m2_event_list       = $test_m2evt[1];
			$m2timing_flag       = 0;
			@test_m2timingevt    = &test_existance_mos2timingeventfiles();
			$m2_timingevent_list = $test_m2timingevt[1];

			if ( $test_m1evt[0] && $test_m2evt[0] )
			{    # Once emproc has run, do I have event files ?

				print
				  "   \#> After running emproc: MOS Image Event List found: \n";
				print "       MOS1: $m1_event_list \n";
				print "       MOS2: $m2_event_list \n";
				print "   \#> emproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: MOS Image Event List found: \n";
					print MYOUTFILE "       MOS1: $m1_event_list \n";
					print MYOUTFILE "       MOS2: $m2_event_list \n";
					print MYOUTFILE "   \#> emproc OK \n";
				}
				$m1evt_flag = 1;
				$m2evt_flag = 1;

			}
			elsif ( $test_m1evt[0] && !$test_m2evt[0] ) {

				print "   \#> After running emproc: MOS1 Image Event List found: $m1_event_list \n";
				print "   \#> emproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: MOS1 Image Event List found: $m1_event_list \n";
					print MYOUTFILE "   \#> emproc OK \n";
				}
				$m1evt_flag = 1;

			}
			elsif ( !$test_m1evt[0] && $test_m2evt[0] ) {

				print "   \#> After running emproc: MOS2 Image Event List found: $m2_event_list \n";
				print "   \#> emproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: MOS2 Image Event List found: $m2_event_list \n";
					print MYOUTFILE "   \#> emproc OK \n";
				}
				$m2evt_flag = 1;

			}

			if ( $test_m1timingevt[0] && $test_m2timingevt[0] )
			{    # Once emproc has run, do I have timing event files ?

				print "   \#> After running emproc: MOS Timing Event List found: \n";
				print "       MOS1: $m1_timingevent_list \n";
				print "       MOS2: $m2_timingevent_list \n";
				print "   \#> emproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: MOS Timing Event List found: \n";
					print MYOUTFILE "       MOS1: $m1_timingevent_list \n";
					print MYOUTFILE "       MOS2: $m2_timingevent_list \n";
					print MYOUTFILE "   \#> emproc OK \n";
				}
				$m1timing_flag = 1;
				$m2timing_flag = 1;

			}
			elsif ( $test_m1timingevt[0] && !$test_m2timingevt[0] ) {

				print "   \#> After running emproc: MOS1 Timing Event List found: $m1_timingevent_list \n";
				print "   \#> emproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: MOS1 Timing Event List found: $m1_timingevent_list \n";
					print MYOUTFILE "   \#> emproc OK \n";
				}
				$m1timing_flag = 1;

			}
			elsif ( !$test_m1timingevt[0] && $test_m2timingevt[0] ) {

				print "   \#> After running emproc: MOS2 Timing Event List found: $m2_timingevent_list \n";
				print "   \#> emproc OK \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: MOS2 Timing Event List found: $m2_timingevent_list \n";
					print MYOUTFILE "   \#> emproc OK \n";
				}
				$m2timing_flag = 1;

			}

			if (   !$m1evt_flag
				&& !$m2evt_flag
				&& !$m1timingevt_flag
				&& !$m2timingevt_flag )
			{

				print "   \#> After running emproc: No MOS Image or Timing Event List were found  \n";
				print "   \#> emproc FAILED (no Events file found) \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE "   \#> After running emproc: No MOS Image or Timing Event List were found \n";
					print MYOUTFILE "   \#> emproc FAILED (no Events file found) \n";
				}

				#	    &error_code(3) && return;
			}

		}
	}    

	if ( $Log_Evt_Flag == 0 ) {
		my $event_log = $results_directory . "event_" . $source_name . "\.log";
		if ( -e $event_log ) {
			unlink("$event_log") or die "Cannot rm file: $!";
		}    # Delete if exists
		open( EVTLOG, "> $event_log" )
		  or die "Cant open output file: $! \n";    # Open for write, overwrite
		if ( $test_pnevt[0] ) { &event_file_inf( $pn_event_list, "pn" ); }
		if ( $test_pntimingevt[0] ) {
			&event_file_inf( $pn_timingevent_list, "pn" );
		}
		if ( $test_m1evt[0] ) { &event_file_inf( $m1_event_list, "m1" ); }
		if ( $test_m1timingevt[0] ) {
			&event_file_inf( $m1_timingevent_list, "m1" );
		}
		if ( $test_m2evt[0] ) { &event_file_inf( $m2_event_list, "m2" ); }
		if ( $test_m2timingevt[0] ) {
			&event_file_inf( $m2_timingevent_list, "m2" );
		}
		close(EVTLOG);

		#	$Log_Evt_Flag = 1;
	}

	&error_code(0);
	return (
		$pnevt_flag,       $m1evt_flag,       $m2evt_flag,
		$pntimingevt_flag, $m1timingevt_flag, $m2timingevt_flag
	);
}

## @method test_epiceventfiles($inst,$exposure,$mode)
# Test the existance of EPIC Imaging Event List
# @param instrument
# @param exposure
# @param mode
# @return File Name

#========================================================================
sub test_epiceventfiles() {
	my $instrument = $_[0];
	my $exposure = $_[1];
	my $mode = $_[2];
	
	my $dir= "";
	if ($instrument eq "EPN") { 
		$dir = $pn_directory;
	}
	else { 
		$dir = $mos_directory;
	}
	chdir($dir);
	
	my $match = "*".$instrument."_".$exposure."_".$mode.".ds";
	@EPIC_Evtlist =  glob($match);

	if ( $#EPIC_Evtlist >= 0 ) {
		print "   \#> $instrument $mode Event List found: \n";
		foreach (@EPIC_Evtlist) { print "\t" . $_ . "\n"; }
		if ( $Log_Flag == 0 ) {
			print MYOUTFILE "   \#> $instrument $mode Event List found: \n";
			foreach (@EPIC_Evtlist) { print MYOUTFILE "\t" . $_ . "\n"; }
		}
		$index = 0;
		if ( $#EPIC_Evtlist > 0 ) {
			print "      Using event file with longest Observation Duration: $EPIC_Evtlist[0] \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE"      Using event file with longest Observation Duration: $EPIC_Evtlist[0] \n";
			}
		}
		my $epic_event_list = $EPIC_Evtlist[0];
		return ( 1, $dir . $epic_event_list );
	}
	else {
		print "   \#> $instrument $mode Event List not found \n";
		if ( $Log_Flag == 0 ) {
			print MYOUTFILE "   \#> $instrument $mode Event List not found \n";
		}
		return ( 0, "No File" );
	}
}

## @method test_existance_pneventfiles()
# Test the existance of EPIC-pn Imaging Event List
# @return File Name

#========================================================================
sub test_existance_pneventfiles(){
    chdir ($pn_directory);
    @PN_Evtlist = glob('*EPN*ImagingEvts.ds');
    if ($#PN_Evtlist >= 0) {
	print "   \#> PN Image Event List found: \n"; foreach (@PN_Evtlist) {print "\t".$_ ."\n";}
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> PN Image Event List found: \n"; foreach (@PN_Evtlist) {print MYOUTFILE "\t".$_."\n";}}
	$index=0;
	if ($#PN_Evtlist > 0) {
	    foreach my $event_list (@PN_Evtlist) {$obs_duration=&getduration($event_list,1); push(@pnitime,$obs_duration);}
	    $max_time=0.;
	    $i=0;
	    foreach (@pnitime){if ($_ > $max_time){$max_time=$_;$index=$i;};$i++}
	    print "      Using event file with longest Observation Duration: $PN_Evtlist[$index] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using event file with longest Observation Duration: $PN_Evtlist[$index] \n";}
	}
	my $pn_event_list = $PN_Evtlist[$index];
	return(1,$pn_directory.$pn_event_list);
    } else {
	print "   \#> PN Imaging Event List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> PN Imaging Event List not found \n";}
	return (0,"No File"); 
    }
}

## @method test_pntimingeventfiles()
# Test the existance of EPIC-pn Timing Event List
# @return File Name

#========================================================================
sub test_existance_pntimingeventfiles(){
    chdir ($pn_directory);
    @PN_TimEvtlist = glob('*EPN*TimingEvts.ds');
    if ($#PN_TimEvtlist >= 0) {
	print "   \#> PN Timing Event List found: \n"; foreach (@PN_TimEvtlist) {print "\t".$_ ."\n";}
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> PN Timing Event List found: \n"; foreach (@PN_TimEvtlist) {print MYOUTFILE "\t".$_."\n";}}
	$index=0;
	if ($#PN_TimEvtlist > 0) {
	    foreach my $event_list (@PN_TimEvtlist) {$obs_duration=&getduration($event_list,1); push(@pnttime,$obs_duration);}
	    $max_time=0.;
	    $i=0;
	    foreach (@pnttime){if ($_ > $max_time){$max_time=$_;$index=$i;};$i++}
	    print "      Using event file with longest Observation Duration: $PN_TimEvtlist[$index] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using event file with longest Observation Duration: $PN_TimEvtlist[$index] \n";}
	}
	my $pn_timingevent_list = $PN_TimEvtlist[$index];
	return(1,$pn_directory.$pn_timingevent_list);
    } else {
	print "   \#> PN Timing List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> PN Timing List not found \n";}
	return (0,"No File"); 
    }
}

## @method test_existance_mos1eventfiles()
# Test the existance of EPIC-MOS1 Imaging Event List
# @return File Name

#========================================================================
sub test_existance_mos1eventfiles(){
    chdir ($mos_directory);
    my @MOS1_Evtlist = glob('*EMOS1*ImagingEvts.ds');
    if ($#MOS1_Evtlist >= 0){
	print "   \#> MOS1 Image Event List found: \n"; foreach (@MOS1_Evtlist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS1 Image Event List found: \n"; foreach (@MOS1_Evtlist) {print MYOUTFILE "\t".$_ ."\n";}}
	$index=0;
	if ($#MOS1_Evtlist > 0){	    
	    foreach my $event_list (@MOS1_Evtlist) {$obs_duration=&getduration($event_list,1); push(@m1itime,$obs_duration);}
	    $max_time=0.;
	    $i=0;
	    foreach (@m1itime){if ($_ > $max_time){$max_time=$_;$index=$i;};$i++}
	    print "      Using event file with longest Observation Duration: $MOS1_Evtlist[$index] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using event file with longest Observation Duration: $MOS1_Evtlist[$index] \n";}
	}
	my $m1_event_list = $MOS1_Evtlist[$index];
	return(1,$mos_directory.$m1_event_list);
    } else {
	print "   \#> MOS1 Image Event List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS1 Image Event List not found \n";}
	return(0,"No File");
    }
}

## @method test_mos1timingeventfiles()
# Test the existance of EPIC-MOS1 Timining Event List
# @return File Name

#========================================================================
sub test_existance_mos1timingeventfiles(){
    chdir ($mos_directory);
    my @MOS1_TimEvtlist = glob('*EMOS1*TimingEvts.ds');
    if ($#MOS1_TimEvtlist >= 0){
	print "   \#> MOS1 Timing List found: \n"; foreach (@MOS1_TimEvtlist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS1 Timing List found: \n"; foreach (@MOS1_TimEvtlist) {print MYOUTFILE "\t".$_ ."\n";}}
	$index=0;
       	if ($#MOS1_TimEvtlist > 0){
	    foreach my $event_list (@MOS1_TimEvtlist) {$obs_duration=&getduration($event_list,1); push(@m1ttime,$obs_duration);}
	    $max_time=0.;
	    $i=0;
	    foreach (@m1ttime){if ($_ > $max_time){$max_time=$_;$index=$i;};$i++}
	    print "      Using event file with longest Observation Duration: $MOS1_TimEvtlist[$index] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using event file with longest Observation Duration: $MOS1_TimEvtlist[$index] \n";}
	}
	my $m1_timingevent_list = $MOS1_TimEvtlist[$index];
	return(1,$mos_directory.$m1_timingevent_list);
    } else {
	print "   \#> MOS1 Timing List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS1 Timing List not found \n";}
	return(0,"No File");
    }
}

## @method test_existance_mos2eventfiles()
# Test the existance of EPIC-MOS2 Imaging Event List
# @return File Name

#========================================================================
sub test_existance_mos2eventfiles(){
    chdir ($mos_directory);
    my @MOS2_Evtlist = glob('*EMOS2*ImagingEvts.ds');
    if ($#MOS2_Evtlist >= 0){
	print "   \#> MOS2 Image Event List found: \n"; foreach (@MOS2_Evtlist) {print "\t".$_ ."\n";}  
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS2 Image Event List found: \n"; foreach (@MOS2_Evtlist) {print MYOUTFILE "\t".$_."\n";}}
	$index=0;
	if ($#MOS2_Evtlist > 0){
	    foreach my $event_list (@MOS2_Evtlist) {$obs_duration=&getduration($event_list,1); push(@m2itime,$obs_duration);}
	    $max_time=0.;
	    $i=0;
	    foreach (@m2itime){if ($_ > $max_time){$max_time=$_;$index=$i;};$i++}
	    print "      Using event file with longest Observation Duration: $MOS2_Evtlist[$index] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using event file with longest Observation Duration: $MOS2_Evtlist[$index] \n";}
	}
	my $m2_event_list = $MOS2_Evtlist[$index];
	return(1,$mos_directory.$m2_event_list);
    } else {
	print "   \#> MOS2 Image Event List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS2 Image Event List not found \n";}
	return(0,"No File");
    }
}

## @method test_mos2timingeventfiles()
# Test the existance of EPIC-MOS2 Timining Event List
# @return File Name

#========================================================================
sub test_existance_mos2timingeventfiles(){
    chdir ($mos_directory);
    my @MOS2_TimEvtlist = glob('*EMOS2*TimingEvts.ds');
    if ($#MOS2_TimEvtlist >= 0){
	print "   \#> MOS2 Timing List found: \n"; foreach (@MOS2_TimEvtlist) {print "\t".$_ ."\n";}  
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS2 Timing List found: \n"; foreach (@MOS2_TimEvtlist) {print MYOUTFILE "\t".$_."\n";}}
	$index=0;
	if ($#MOS2_TimEvtlist > 0){
	    foreach my $event_list (@MOS2_TimEvtlist) {$obs_duration=&getduration($event_list,1); push(@m2ttime,$obs_duration);}
	    $max_time=0.;
	    $i=0;
	    foreach (@m2ttime){if ($_ > $max_time){$max_time=$_;$index=$i;};$i++}
	    print "      Using event file with longest Observation Duration: $MOS2_TimEvtlist[$index] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using event file with longest Observation Duration: $MOS2_TimEvtlist[$index] \n";}
	}
	my $m2_timingevent_list = $MOS2_TimEvtlist[$index];
	return(1,$mos_directory.$m2_timingevent_list);
    } else {
	print "   \#> MOS2 Timing List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS2 Timing List not found \n";}
	return(0,"No File");
    }
}

1;
