## @file
# Produce EPIC point source lightcurve

## @method void produce_lightcurve()
# Produce EPIC lightcurve products
# @return Void

#========================================================================
sub produce_lightcurve() {
	print "\n----> Working out Light Curve .....\n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "\n----> Working out Light Curve ....\n";
	}

	#.. Define Global Variables
	print "   \#> Defining File Names for Light Curve .....\n ";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "   \#> Defining File Names for Light Curve .....\n ";
	}

	use vars qw($src_scale
	  $bkg_scale
	  $epic_obsmode
	  $epic_event_list
	  $epic_GTI_file
	  $epic_bkglc_file
	  $epic_cleanevents_file
	  $epic_gti_image_file
	  $epic_gti_smoothimage_file
	  $epic_eregionanal_file
	  $epic_lcurve_region_file
	  $epic_lcurve_bkgregion_file
	  $epic_lcurve_combinedregion_file
	  $epic_emllist_file
	  $source_epic_region_file

	  $epic_sourcelc_e0_file
	  $epic_bkglc_e0_file
	  $epic_srcbkgsubtractlc_e0_file

	  $epic_lcurve_flag
	);
	#.. Test for ccf file and set ENV variable
	@test_ccf = &test_ccf();
	my $ccffile = $test_ccf[1];
	if ( !$test_ccf[0] ) {
		print "      Run cifbuild first \n";
		&error_code(7);
	}
	&set_ccf($ccffile);

	#.. Test for odfingest file and set ENV variable
	@test_odfingest = &test_odfingest();
	my $odfingestfile = $test_odfingest[1];
	if ( !$test_odfingest[0] ) {
		&run_odfingest();
	}
	@test_odfingest = &test_odfingest();
	if ( !$test_odfingest[0] ) {
		print "      Run odfingest first \n";
		&error_code(7);
	}
	&set_odfingest($odfingestfile);

	#.. Print sas_setup
	&print_sas_setup();

	#.. Default Names
	my $source_obsID = $gv->SRC_OBSID;
	my $source_name  = $gv->SRC_Name;

	#.. Get event files
	my @files = &get_EPIC_eventfiles();

	#	my %epicFileMap = get_EPIC_Exposures(@files);
	my $inst;
	my $exposure;
	my $mode;
	if ( $#files == 0 ) { &error_code(4); }

	foreach my $file (@files) {
		( $inst, $exposure, $mode ) = &get_File_Info($file);
		my $isExposureFlag = isExposureSelected($exposure,$inst,"main");
		my $isExposureLCFlag = isExposureSelected($exposure,$inst,"LC");	
		
		$epic_lcurve_flag = 0
		  ; # Flag that determines whether the light curve should be worked out (1: True)

		# Check if the user select EMOS1, EMOS2, PN for analysing data
		if ( $gv->$inst eq "no" ) {
			print
"INSTRUMENT $inst [exposure=$exposure]: Not selected for calculating light curve\n";
			print "Skipping to the next exposure...\n";
		}
	  	#.. Check if the current exposure has been selected to be processed
	  	elsif ( ($isExposureFlag eq "no") ||  ($isExposureLCFlag eq "no") )
	  	{
	  		print
			"EXPOSURE $exposure [intrument=$inst]: Not selected for producing lightcurve\n";
			print "Skipping to the next exposure...\n";
		}	
		else {
		    $epic_lcurve_flag = 1;
		    
		    print "Working out light curve for: $inst, $exposure, $mode\n";
		    
		    my $instrument = "";
		    if ( $inst eq "EPN" ) {
			$instrument = "pn";
		    }
		    elsif ( $inst eq "EMOS1" ) {
			$instrument = "m1";
		    }
		    elsif ( $inst eq "EMOS2" ) {
			$instrument = "m2";
		    }
		    
		    #.. Definition of file names
		    create_pnProductFileNames( $instrument, $exposure, $mode );
		    
		    $epic_cleanevents_file =
			$lcurve_directory
			. $instrument . "_"
			. $exposure . "_"
			. $mode
			. "_events_filtered\.fit";
			$epic_gti_image_file =
			    $gti_directory
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_img_gtifiltered\.img";
			$epic_gti_smoothimage_file =
			    $gti_directory
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_img_gtifiltered_smooth\.img";
			$epic_eregionanal_file =
			    $lcurve_directory
			  . $gv->SRC_Name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_eregionanalyse\.txt";
			$epic_emllist_file =
			    $images_directory
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_emllist\.fits";

			$source_epic_region_file =
			    $images_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_region\.reg";
			$epic_lcurve_region_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_region\.reg";
			$epic_lcurve_bkgregion_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_bkgregion\.reg";
			$epic_lcurve_combinedregion_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_combinedregion\.reg";

			$epic_sourcelc_e0_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_source.lc";
			$epic_bkglc_e0_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_bkg.lc";
			$epic_srcbkgsubtractlc_e0_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_sourcebkgsubtracted.lc";

			$epic_bkglc_file =
			    $lcurve_directory
			  . $source_name . "_"
			  . $instrument . "_"
			  . $exposure . "_"
			  . $mode
			  . "_bkg.lc";

			# Light curve parameters
			#.. Check if the user has defined its own coordinates
			&setcoordinates();


			my $source_ra    = $gv->SRC_RA;
			my $source_dec   = $gv->SRC_DEC;

			print " OK \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE " OK \n";
			}

			if ( $Log_Spec_Flag == 0 ) {
				$temp_Log_Spec_Flag = $Log_Spec_Flag;
				$Log_Spec_Flag      = 1;
			}    # Deactivate Spectral Log flag if active

			chdir($lcurve_directory);


			#.. RA & DEC Coordinates of position to analyze

#.. If run_edetect chain has been ran, use the closest source to the RA DEC given by the user for analysis
			my ( $ra_epic_closesource, $dec_epic_closesource,
				$epic_mindist_deg ) =
			  &find_closest_source($source_epic_region_file);

			if (   $epic_mindist_deg != 1000.
				&& $epic_mindist_deg <= $gv->EDC_sourcematch )
			{
				print
"   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to $instrument).\n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
"   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";
				}
				$ra_anal  = $ra_epic_closesource;
				$dec_anal = $dec_epic_closesource;
			}
			else {
				$ra_anal  = $source_ra;     # (deg) User Coordinates
				$dec_anal = $source_dec;    # (deg) User coordinates				
			}
			print "   \#> RA & DEC of source to analyse: \n";
			print "        RA : $ra_anal deg \n";
			print "        DEC: $dec_anal deg \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE "   \#> RA & DEC of source to analyse: \n";
				print MYOUTFILE "        RA : $ra_anal deg \n";
				print MYOUTFILE "        DEC: $dec_anal deg \n";
			}
      #.. Test for existance of GTI files
            &check_GTIs_Files( $instrument, $exposure, $mode, "lcurve",
                              $source_name );

	  #.. Test for existance of PN and MOS event files (both Imaging and Timing)
			&check_Spectra_Files( $inst, $exposure, $mode, "lcurve",
				$source_name );

#.. Get instrument's observing mode and prepare things according to shape of source and background regions.
			$epic_obsmode = $obsid_instruments{$source_obsID}{$instrument}{'mode'};
			$epic_obssubmode =
			  $obsid_instruments{$source_obsID}{$instrument}{'submode'};

			#.. Create filtered event lists.
			if ($epic_lcurve_flag) {
				&produce_cleanfile( $epic_event_list,
					$epic_cleanevents_file, $epic_obsmode,
					$instrument );
			}    # EPIC

			#.. Get some information out of the GTI filtered event lists.
			print "   \#> Information from GTI Filtered Event Files: \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "   \#> Information from GTI Filtered Event Files: \n";
			}
			if ($epic_lcurve_flag) {
				&event_file_inf( $epic_cleanevents_file, $instrument );
			}    # EPIC

			if ( $Log_Evt_Flag == 0 ) {
				close(EVTLOG);
				$Log_Evt_Flag = 1;
			}

#.. Make an image using the filtered event file to get source centroid from eregionanalyse
			if ( ( ($instrument eq "m1") || ($instrument eq "m2") ) &&  ($mode eq "TimingEvts" )  )
			{
				print "   \#> Cannot produce smooth image for exposure: $exposure [instrument: $instrument and mode: $mode] \n";	
			} 
			else
			{
				if ( ($epic_lcurve_flag) || ($gv->epic_instrument eq $inst) )    # && !-e $epic_gti_smoothimage_file)
				{
					&produce_smooth_image(
						$epic_gti_cleanevents_file, $epic_gti_image_file,
						$epic_gti_smoothimage_file, $instrument
					);
				}
			}                         # EPIC

#.. Using the instrument's image to work out the source centroid. Only for Imaging.

			print "   \#> Centroid of source to analyze (Physical Units) \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "   \#> Centroid of source to analyze (Physical Units) \n";
			}

			#.. Run eregionanalyse
			if ( ($epic_lcurve_flag)  || ($gv->epic_instrument eq $inst) ){
				$epic_lcurve_flag =
				  &prepare_spectra( $instrument, $exposure, $mode,
					$epic_obsmode, $epic_obssubmode, "LC" );
			}

			#.. run epatplot
			if ( $gv->ANA_epatplot eq "yes") {
				if ($epic_lcurve_flag) {
					&run_epatplot( $epic_gti_cleanevents_file, $instrument,
						"LC", $epic_obsmode )
				}    # EPIC
				 # Set the flag to false. If run_all has been selected, produce_lightcurve will not run againg epatplot.
				$gv->ANA_epatplot("no");
			}

			#.. Prepare light curve log file and output information
			if ( $Log_Lcur_Flag == 0 ) {
				my $lcurve_log =
				  $results_directory . "lcurve_" . $source_name . "\.log";
				if ( -e $lcurve_log ) {
					unlink("$lcurve_log") or die "Cannot rm file: $!";
				}
				open( LCURVELOG, "> $lcurve_log" )
				  or die
				  "Cant open output file: $! \n";    #open for write, overwrite

				print LCURVELOG
				  "   \#> Centroid of source to analyze (Physical Units) \n";

				&prepare_lcurve_log_file( $instrument, $exposure, $mode );
			}

#	  if ( !$epic_lcurve_flag  ) {
#	    print
#	      "   \#> No information to produce light curve in any camera .... exiting \n";
#	    if ( $Log_Flag == 0 ) {
#	      print MYOUTFILE
#		"   \#> No information to produce light curve in any camera .... exiting \n";
#	    }
#	    if ( $Log_Lcur_Flag == 0 ) {
#	      print LCURVELOG
#		"   \#> No information to produce light curve in any camera .... exiting \n";
#	    }
#	    if ( $Log_Lcur_Flag == 0 ) {
#	      close(LCURVELOG);
#	    }
#
#	    &error_code(0);
#	    return (0);
#	  }
			
			chdir($lcurve_directory);
			
			#.. Make image plots and create region files used to extract the Light curves
			print "\n   \#> Producing plots and region files ..... \n";
			if ( $Log_Flag == 0 ) {
			    print MYOUTFILE
				"\n   \#> Producing plots and region files ..... \n";
			}
			
			if ( -e $epic_lcurve_region_file ) {
			    unlink("$epic_lcurve_region_file")
				or die "Cannot rm file $temp_file: $!";
			}
			
			if ( -e $epic_lcurve_bkgregion_file ) {
			    unlink("$epic_lcurve_bkgregion_file")
				or die "Cannot rm file $temp_file: $!";
			}
			
			if ( -e $epic_lcurve_combinedregion_file ) {
			    unlink("$epic_lcurve_combinedregion_file")
				or die "Cannot rm file $temp_file: $!";
			}
			
			my $src_region = "LC_" . $instrument . "_srcregion";
			my $bkg_region = "LC_" . $instrument . "_bkgregion";
			
			#.. Source Regions...
			my $src_region_X       = "LC_" . $instrument . "_srcregion_X";
			my $src_region_Y       = "LC_" . $instrument . "_srcregion_Y";
			my $src_region_R1      = "LC_" . $instrument . "_srcregion_R1";
			my $src_region_R2      = "LC_" . $instrument . "_srcregion_R2";
			my $src_region_XLength = "LC_" . $instrument . "_srcregion_XLength";
			my $src_region_YLength = "LC_" . $instrument . "_srcregion_YLength";

			#.. Bkg Regions...
			my $bkg_region_X       = "LC_" . $instrument . "_bkgregion_X";
			my $bkg_region_Y       = "LC_" . $instrument . "_bkgregion_Y";
			my $bkg_region_R1      = "LC_" . $instrument . "_bkgregion_R1";
			my $bkg_region_R2      = "LC_" . $instrument . "_bkgregion_R2";
			my $bkg_region_XLength = "LC_" . $instrument . "_bkgregion_XLength";
			my $bkg_region_YLength = "LC_" . $instrument . "_bkgregion_YLength";

			if ( $gv->$src_region eq "circle" || $gv->$src_region eq "annulus" )
			{
				$x  = $gv->$src_region_X;
				$y  = $gv->$src_region_Y;
				$L1 = $gv->$src_region_R1;
				$L2 = $gv->$src_region_R2;
			}
			if ( $gv->$src_region eq "box" ) {
				$x  = $gv->$src_region_X;
				$y  = $gv->$src_region_Y;
				$L1 = $gv->$src_region_XLength;
				$L2 = $gv->$src_region_YLength;
			}
			if ($epic_lcurve_flag) {
				&addsourcetoregionfile( $epic_lcurve_region_file, $x, $y, $L1,
					$L2, "yellow", "", $gv->$src_region, "physical" );
			}    # PN
			if ($epic_lcurve_flag) {
				&addsourcetoregionfile( $epic_lcurve_combinedregion_file,
					$x, $y, $L1, $L2, "yellow", "", $gv->$src_region,
					"physical" );
			}    # PN

			if ( $gv->$bkg_region eq "circle" || $gv->$bkg_region eq "annulus" )
			{
				$x  = $gv->$bkg_region_X;
				$y  = $gv->$bkg_region_Y;
				$L1 = $gv->$bkg_region_R1;
				$L2 = $gv->$bkg_region_R2;
			}
			if ( $gv->$bkg_region eq "box" ) {
				$x  = $gv->$bkg_region_X;
				$y  = $gv->$bkg_region_Y;
				$L1 = $gv->$bkg_region_XLength;
				$L2 = $gv->$bkg_region_YLength;
			}
			if ($epic_lcurve_flag) {
				&addsourcetoregionfile( $epic_lcurve_bkgregion_file, $x, $y,
					$L1, $L2, "red", "", $gv->$bkg_region, "physical" );
			}    # PN
			if ($epic_lcurve_flag) {
				&addsourcetoregionfile( $epic_lcurve_combinedregion_file,
					$x, $y, $L1, $L2, "red", "", $gv->$bkg_region, "physical" );
			}    # PN

			print
			  "\n   \#> Producing Source and BKG Light Curve files ..... \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
"\n   \#> Producing Source and BKG Light Curve files ..... \n";
			}

			if ($epic_lcurve_flag) {
				$SRC_XC = $gv->$src_region_X;
				$SRC_YC = $gv->$src_region_Y;
				if (   $gv->$src_region eq "circle"
					|| $gv->$src_region eq "annulus" )
				{
					$RSRC1 = $gv->$src_region_R1;
				}
				if (   $gv->$src_region eq "circle"
					|| $gv->$src_region eq "annulus" )
				{
					$RSRC2 = $gv->$src_region_R2;
				}
				if ( $gv->$src_region eq "box" ) {
					$RSRC1 = $gv->$src_region_XLength;
				}
				if ( $gv->$src_region eq "box" ) {
					$RSRC2 = $gv->$src_region_YLength;
				}
				$BKG_XC = $gv->$bkg_region_X;
				$BKG_YC = $gv->$bkg_region_Y;
				if (   $gv->$bkg_region eq "circle"
					|| $gv->$bkg_region eq "annulus" )
				{
					$RBKG1 = $gv->$bkg_region_R1;
				}
				if (   $gv->$bkg_region eq "circle"
					|| $gv->$bkg_region eq "annulus" )
				{
					$RBKG2 = $gv->$bkg_region_R2;
				}
				if ( $gv->$bkg_region eq "box" ) {
					$RBKG1 = $gv->$bkg_region_XLength;
				}
				if ( $gv->$bkg_region eq "box" ) {
					$RBKG2 = $gv->$bkg_region_YLength;
				}
				my $binSize = "LC_" . $instrument . "_timebinsize";
				$T      = $gv->$binSize;
				$region = $gv->$src_region;
				my $areaFactor = "LC_" . $instrument . "_areafactor";
				if (   -e $epic_sourcebkgspectrum_file
					&& -e $epic_bkgspectrum_file )
				{
					( $src_livetime, $src_backscale, $src_areascal ) =
					  &event_file_spectralinf( $epic_sourcebkgspectrum_file,
						$instrument );
					( $bkg_livetime, $bkg_backscale, $bkg_areascal ) =
					  &event_file_spectralinf( $epic_bkgspectrum_file,
						$instrument );
					$src_scale = 1.;
					$bkg_scale = ( $src_backscale / $bkg_backscale );

					$gv->$areaFactor($bkg_scale);
				}
				$src_scale = 1.;
				$bkg_scale = $gv->$areaFactor;
				my $enerUp  = "LC_" . $instrument . "_e1_up";
				my $enerLow = "LC_" . $instrument . "_e1_low";
				$E1 = $gv->$enerLow;    # Low energy limit for 1st light curve
				$E2 = $gv->$enerUp;     # High energy limit for 1st light curve

				my $sourceExpression     = "LC_" . $instrument . "_src_expr";
				my $backgroundExpression = "LC_" . $instrument . "_bkg_expr";
				my $currentInteractivity = "LC_".$instrument."_interactivity";	
				my $currentds9Display = "display_ds9_LC_".$instrument."_exposure";	
				#.. Check if we are in Timing and EMOS. In this case, the interactive 
				if (($mode eq "TimingEvts") && ( ($instrument eq "m1") || ($instrument eq "m2") ) )
				{
					print "\n   \#> $instrument in $mode cannot be displayed.\n";
					$gv->$currentInteractivity("no");
				}
				if ( ($gv->$sourceExpression eq "")  && (uc($gv->$currentInteractivity) eq "NO") ) {
					&produce_LC(
						$epic_cleanevents_file,     $region,
						$SRC_XC,                        $SRC_YC,
						$RSRC1,                         $RSRC2,
						$BKG_XC,                        $BKG_YC,
						$RBKG1,                         $RBKG2,
						$E1,                            $E2,
						$T,                             $instrument,
						$epic_sourcelc_e0_file,         $epic_bkglc_e0_file,
						$epic_srcbkgsubtractlc_e0_file, $epic_obsmode
					);
				}
				else {
					 if ((uc($gv->$currentds9Display) eq "YES"))
				  	 {
						#.. Display ds9 and get src and bkg information
						print "file $epic_cleanevents_file\n";
						&show_ds9($epic_cleanevents_file,$instrument,$exposure,$mode,"LC");
					 }
					&produce_user_LC(
						$epic_cleanevents_file,
						$gv->$sourceExpression,
						$gv->$backgroundExpression,
						$E1,
						$E2,
						$T,
						$instrument,
						$epic_sourcelc_e0_file,
						$epic_bkglc_e0_file,
						$epic_srcbkgsubtractlc_e0_file
					);
				}
			}

			#.. Write Summary

			print "\n   \#> Light Curve Analysis Summary :\n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE "\n   \#> Light Curve Analysis Summary :\n";
			}
			if ( $Log_Lcur_Flag == 0 ) {
				print LCURVELOG "\n   \#> Light Curve Analysis Summary :\n";
			}
			if ( !$epic_lcurve_flag ) {
				$epic_summ_ana_msg = "FAILED";
			}
			else {
				$epic_summ_ana_msg = "OK";
			}

			print "      $instrument : lcurve : $epic_summ_ana_msg \n";

			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "      $instrument : lcurve : $epic_summ_ana_msg \n";
			}
			if ( $Log_Lcur_Flag == 0 ) {
				print LCURVELOG
				  "      $instrument : lcurve : $epic_summ_ana_msg \n";
			}

			if ( $Log_Lcur_Flag == 0 ) {
				close(LCURVELOG);
			}

			if ( $temp_Log_Spec_Flag == 0 ) {
				$Log_Spec_Flag = 0;
			}    # Re-activate Spectral Log flag if active
				#.. Workaround to fix the problem of having more than one exposure.
		    #.. In this case, we have to reset the src and bkg variables.
		    #.. Source Regions...
		    
		    $gv->$src_region_X(0);
		    $gv->$src_region_Y(0);
		    $gv->$src_region_XLength(0);
		    $gv->$src_region_YLength(0);
		    $gv->$src_region_R1(0);
		    $gv->$src_region_R2(0);
		    $gv->$bkg_region_X(0);
		    $gv->$bkg_region_Y(0);
		    $gv->$bkg_region_XLength(0);
		    $gv->$bkg_region_YLength(0);
		    $gv->$bkg_region_R1(0);
		    $gv->$bkg_region_R2(0);    
		}				      

	    }    ### END OF EPIC LOOP
	&error_code(0);
	return;
}

## @method void produce_LC($events_file,$region,$X_Source,$Y_Source,$Source_inRad,$Source_outRad,$X_BKG,$Y_BKG,$BKG_inRad,$BKG_outRad,$E1,$E2,$T_Bin,$instr,$LC_SRC,$LC_BKG,$LC_corrected,$subMode)
# Creates LC.
# @param event_file Event File Name
# @param region Type of Region (circle, annulus, box)
# @param X_Source Source X_coordinate (Image Coordinates)
# @param Y_Source Source Y_coordinate (Image Coordinates)
# @param Source_inRad Source Region Inner Radius (circle,annulus), Source Region X_Box Size (box) (Image Coordinates)
# @param Source_outRad Source Outer Radius (circle,annulus), Source Region Y_Box Size (box) (Image Coordinates)
# @param X_BKG Background X_coordinate (Image Coordinates)
# @param Y_BKG Background Y_coordinate (Image Coordinates)
# @param BKG_inRad Background Region Inner Radius (circle,annulus), Background Region X_Box Size (box) (Image Coordinates)
# @param BKG_outRad Background Region Outer Radius (circle,annulus), Background Region Y_Box Size (box) (Image Coordinates)
# @param E1 Lower Bound Energy Range (1st range)
# @param E2 Upper Bound Energy Range (1st range)
# @param T_Bin Time Bin (secs)
# @param instr Instrument
# @param LC_SRC Source Light Curve Output File Name
# @param LC_BKG Background Light Curve Output File Name
# @param LC_corrected Light Curve corrected Output File Name
# @subMode Observation mode (IMAGING, TIMING)
# @return Void

#========================================================================
sub produce_LC() {

	# Creates a file from a time series created by evselect

	my $event_file    = $_[0];
	my @file_t        = split /\//, $event_file;
	my $file          = $gti_directory . $file_t[$#file_t];
	my $region        = $_[1];
	my $X_Source      = $_[2];
	my $Y_Source      = $_[3];
	my $Source_inRad  = $_[4];
	my $Source_outRad = $_[5];
	my $X_BKG         = $_[6];
	my $Y_BKG         = $_[7];
	my $BKG_inRad     = $_[8];
	my $BKG_outRad    = $_[9];
	my $E1            = $_[10];
	my $E2            = $_[11];
	my $T_Bin         = $_[12];
	my $instr         = $_[13];
	my $src_outfile   = $_[14];
	my $bkg_outfile   = $_[15];
	my $LC_corrected  = $_[16];
	my $subMode       = $_[17];

	my $source_name = $gv->SRC_Name;
	my $mode        = $obsid_instruments{ $gv->SRC_OBSID }{$instr}{'mode'};
	my $submode     = $obsid_instruments{ $gv->SRC_OBSID }{$instr}{'submode'};

	&produce_lightcurve_file(
		$E1,           $E2,            $X_Source,    $Y_Source,
		$Source_inRad, $Source_outRad, $T_Bin,       $region,
		$instr,        $event_file,    $src_outfile, $subMode
	);
	&produce_lightcurve_file(
		$E1,        $E2,         $X_BKG,       $Y_BKG,
		$BKG_inRad, $BKG_outRad, $T_Bin,       $region,
		$instr,     $event_file, $bkg_outfile, $subMode
	);

	#.. epiclccorr:
	@args = (
		"epiclccorr",            "srctslist=$src_outfile",
		"eventlist=$event_file", "outset=$LC_corrected",
		"withbkgset=yes",        "bkgtslist=$bkg_outfile",
		"applyabsolutecorrections=yes"
	);

	system(@args) == 0 or &error_code(7);

	return (1);
}

## @method void produce_user_LC($events_file,$src_expr,$bkg_expr,$E1,$E2,$T_Bin,$instr,$LC_SRC,$LC_BKG,$LC_corrected)
# Creates LC.
# @param event_file Event File Name
# @param source expression
# @param background expression
# @param E1 Lower Bound Energy Range (1st range)
# @param E2 Upper Bound Energy Range (1st range)
# @param T_Bin Time Bin (secs)
# @param instr Instrument
# @param LC_SRC Source Light Curve Output File Name
# @param LC_BKG Background Light Curve Output File Name
# @param LC_corrected Light Curve corrected Output File Name
# @return Void

#========================================================================
sub produce_user_LC() {

	# Creates a file from a time series created by evselect

	my $event_file   = $_[0];
	my @file_t       = split /\//, $event_file;
	my $file         = $gti_directory . $file_t[$#file_t];
	my $src_expr     = $_[1];
	my $bkg_expr     = $_[2];
	my $E1           = $_[3];
	my $E2           = $_[4];
	my $T_Bin        = $_[5];
	my $instr        = $_[6];
	my $src_outfile  = $_[7];
	my $bkg_outfile  = $_[8];
	my $LC_corrected = $_[9];

	my $source_name = $gv->SRC_Name;
	my $mode        = $obsid_instruments{ $gv->SRC_OBSID }{$instr}{'mode'};
	my $submode     = $obsid_instruments{ $gv->SRC_OBSID }{$instr}{'submode'};

	&produce_user_lightcurve_file( $E1, $E2, $src_expr, $T_Bin, $instr,
		$event_file, $src_outfile );
	&produce_user_lightcurve_file( $E1, $E2, $bkg_expr, $T_Bin, $instr,
		$event_file, $bkg_outfile );

	#.. epiclccorr:
	@args = (
		"epiclccorr",            "srctslist=$src_outfile",
		"eventlist=$event_file", "outset=$LC_corrected",
		"withbkgset=yes",        "bkgtslist=$bkg_outfile",
		"applyabsolutecorrections=yes"
	);

	system(@args) == 0 or &error_code(7);

	return (1);
}

## @method void produce_lightcurve_file($E1,$E2,$XC,$YC,$R1,$R2,$T,$region,$instr,$event_file,$outfile,$subMode)
# Produce a time series given region (circle, annulus or box) and energy range (E1, E2).
# @param E1 Low Bound Energy Range (eV)
# @param E2 High Bound Energy Range (eV)
# @param XC X-Center of Source Region (Image Pixels)
# @param YC Y-Center of Source Region (Image Pixels)
# @param R1 Radius/Length of Source Region (Image Pixels)
# @param R2 Radius/Length of Source Region (Image Pixels)
# @param T Time Bin (secs)
# @param region	Region Type (circle, annulus, box)
# @param instr Instrument
# @param event_file Event File Name
# @param outfile Output File Name (Rate Set)
# @subMode Observation Mode
# @return Void

#========================================================================
sub produce_lightcurve_file() {

	# Produce an event file for a given region and energy range

	my $E1         = $_[0];
	my $E2         = $_[1];
	my $XC         = $_[2];
	my $YC         = $_[3];
	my $R1         = $_[4];
	my $R2         = $_[5];
	my $T          = $_[6];
	my $region     = $_[7];
	my $instr      = $_[8];
	my $event_file = $_[9];
	my $outfile    = $_[10];
	my $subMode    = $_[11];
	my $xVal;
	my $yVal;

	print "       $instr : $region : Energy Range: $E1 - $E2 eV \n";
	print "       $instr : $region : Time Bining : $T sec\n";
	print "       (Event File: $event_file ; Output File: $outfile) \n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE
		  "       $instr : $region : Energy Range: $E1 - $E2 eV \n";
	}
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "       $instr : $region : Time Bining : $T sec\n";
	}
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE
		  "       (Event File: $event_file ; Output File: $outfile) \n";
	}
	if ( $Log_Lcur_Flag == 0 ) {
		print LCURVELOG
		  "       $instr : $region : Energy Range: $E1 - $E2 eV \n";
	}
	if ( $Log_Lcur_Flag == 0 ) {
		print LCURVELOG "       $instr : $region : Time Bining : $T sec\n";
	}
	if ( $Log_Lcur_Flag == 0 ) {
		print LCURVELOG
		  "       (Event File: $event_file ; Output File: $outfile) \n";
	}

	if ( $instr eq "pn" ) {
		$expr = "(PI>=$E1) && (PI<=$E2)";
	}
	if ( $instr eq "m1" or $instr eq "m2" ) {
		$expr = "(PI>=$E1) && (PI<=$E2)";
	}

	if (   ( $instr eq "m1" or $instr eq "m2" )
		&& ( uc($subMode) eq 'TIMING' ) )
	{

		#$areaexpr="((RAWX,TIME) in box($XC,$YC,$R1,$R2,0))";
		my $X_min = $XC - $R1 / 2.;
		my $X_max = $XC + $R1 / 2.;
		$areaexpr = "(RAWX in [$X_min:$X_max])";
	}
	else {
		if ( $region eq "circle" || $region eq "annulus" ) {
			$areaexpr = "((X,Y) in annulus($XC,$YC,$R1,$R2))";
		}
		if ( $region eq "box" ) {
			if ( $subMode eq 'IMAGING' || $subMode eq 'imaging' ) {
				$xVal = "X";
				$yVal = "Y";
			}
			else {
				$xVal = "RAWX";
				$yVal = "RAWY";
			}

			$areaexpr = "(($xVal,$yVal) in box($XC,$YC,$R1,$R2,0))";
		}
	}
	@args = (
"evselect table=$event_file:EVENTS expression=\"$expr \&\& $areaexpr\" rateset=$outfile timebinsize=$T withrateset=yes maketimecolumn=yes makeratecolumn=yes"
	);

	system(@args) == 0 or &error_code(7);
	return (1);
}

## @method void produce_user_lightcurve_file($E1,$E2,$XC,$YC,$R1,$R2,$T,$region,$instr,$event_file,$outfile,$subMode)
# Produce a time series given region (circle, annulus or box) and energy range (E1, E2).
# @param E1 Low Bound Energy Range (eV)
# @param E2 High Bound Energy Range (eV)
# @param expression
# @param R1 Radius/Length of Source Region (Image Pixels)
# @param R2 Radius/Length of Source Region (Image Pixels)
# @param T Time Bin (secs)
# @param region	Region Type (circle, annulus, box)
# @param instr Instrument
# @param event_file Event File Name
# @param outfile Output File Name (Rate Set)
# @subMode Observation Mode
# @return Void

#========================================================================
sub produce_user_lightcurve_file() {

	# Produce an event file for a given region and energy range

	my $E1         = $_[0];
	my $E2         = $_[1];
	my $expression = $_[2];
	my $T          = $_[3];
	my $instr      = $_[4];
	my $event_file = $_[5];
	my $outfile    = $_[6];

	print "       $instr : $expression : Energy Range: $E1 - $E2 eV \n";
	print "       $instr : Time Bining : $T sec\n";
	print "       (Event File: $event_file ; Output File: $outfile) \n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE
		  "       $instr : $expression : Energy Range: $E1 - $E2 eV \n";
	}
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "       $instr : Time Bining : $T sec\n";
	}
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE
		  "       (Event File: $event_file ; Output File: $outfile) \n";
	}
	if ( $Log_Lcur_Flag == 0 ) {
		print LCURVELOG
		  "       $instr :  $expression : Energy Range: $E1 - $E2 eV \n";
	}
	if ( $Log_Lcur_Flag == 0 ) {
		print LCURVELOG "       $instr :  Time Bining : $T sec\n";
	}
	if ( $Log_Lcur_Flag == 0 ) {
		print LCURVELOG
		  "       (Event File: $event_file ; Output File: $outfile) \n";
	}

	if ( $instr eq "pn" ) {
		$expr = "(PI>=$E1) && (PI<=$E2)";
	}
	if ( $instr eq "m1" or $instr eq "m2" ) {
		$expr = "(PI>=$E1) && (PI<=$E2)";
	}

	@args = (
"evselect table=$event_file:EVENTS expression=\"$expr \&\& ($expression)\" rateset=$outfile timebinsize=$T withrateset=yes maketimecolumn=yes makeratecolumn=yes"
	);

	system(@args) == 0 or &error_code(7);
	return (1);
}


## @method void produce_cleanfile($even_file,$even_cleanfile,$gti_file,$subMode,$instr)
# Given a default expression and event file, produce a gti clean event file.
# @param even_file Event File Name
# @param subMode instrument mode
# @param instr Instrument
# @return Void

#========================================================================
sub produce_cleanfile(){

    my $even_file    = $_[0];
    my $even_cleanfile = $_[1]; 
    my $subMode      = $_[2];
    my $instr        = $_[3];
    

    print "   \#> Producing clean event file from $even_file .... \n";
  	
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Producing clean event file from $even_file .... \n";}
   
    if ($instr eq "pn") {$expr="\#XMMEA_EP && (FLAG==0) && (PATTERN<=4) && (PI>100)";}
    if ($instr =~ m/m/) {
    	if (uc($subMode) eq "TIMING")
    		{$expr="\#XMMEA_EM && (FLAG==0) && (PATTERN==0) && (PI>100)";}
    	else
    		{$expr="\#XMMEA_EM && (FLAG==0) && (PATTERN<=12) && (PI>100)";}
    }	

    @args = ("evselect table=\"$even_file:EVENTS\" withfilteredset=Y filteredset=$even_cleanfile destruct=Y keepfilteroutput=T expression=\"$expr\"");
    system(@args) == 0 or &error_code(6);

    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

    return(1);
}

1;
