## @file
# Produce EPIC point source spectrum

## @method void produce_spectrum()
# Produce EPIC spectrum products
# @return Void

#========================================================================
sub produce_spectrum() {

	use vars qw(
	  $results_directory
	  $epic_event_list
	  $epic_GTI_file

	  $epic_gti_cleanevents_file
	  $epic_gti_image_file
	  $epic_gti_smoothimage_file

	  $epic_eregionanal_file
	  $epic_ecoordconv_file

	  $epic_spectral_flag
	  $epic_spectral_image_psfile
	  $epic_spectral_pileupimage_psfile
	  $epic_spectral_region_file
	  $epic_spectral_bkgregion_file
	  $epic_spectral_combinedregion_file
	  $epic_ancillary_file
	  $epic_redistribution_matrix_file
	  $epic_sourcebkgspectrum_file
	  $epic_bkgspectrum_file
	  $epic_spectrum_psfile
	  $epic_emllist_file
	  $source_epic_region_file

	  $epic_src_livetime
	  $epic_src_backscale
	  $epic_src_areascal

	  $epic_bkg_livetime
	  $epic_bkg_backscale
	  $epic_bkg_areascal

	  %model_name
	);

	#.. Test for ccf file and set ENV variable
	@test_ccf = &test_ccf();
	my $ccffile = $test_ccf[1];
	if ( !$test_ccf[0] ) {
		print "      Run cifbuild first \n";
		&error_code(6);
	}
	&set_ccf($ccffile);

	#.. Test for odfingest file and set ENV variable
	@test_odfingest = &test_odfingest();
	my $odfingestfile = $test_odfingest[1];
	if ( !$test_odfingest[0] ) {
		&run_odfingest();
	}
	@test_odfingest = &test_odfingest();
	if ( !$test_odfingest[0] ) {
		print "      Run odfingest first \n";
		&error_code(6);
	}
	&set_odfingest($odfingestfile);

	#.. Print sas_setup
	&print_sas_setup();

	#.. Get event files
	my @files       = &get_EPIC_eventfiles();
#	my %epicFileMap = get_EPIC_Exposures(@files);
	my $inst;
	my $exposure;
	my $mode;
	if ( $#files == 0 ) { &error_code(4); }

	foreach my $file (@files) {
		( $inst, $exposure, $mode ) = &get_File_Info($file);
		my $isExposureFlag = isExposureSelected($exposure,$inst,"main");	
		my $isExposureLCFlag = isExposureSelected($exposure,$inst,"SPC");
		
		my $instrument = "";
		if ( $inst eq "EPN" ) {
			$instrument = "pn";
		}
		elsif ( $inst eq "EMOS1" ) {
			$instrument = "m1";
		}
		elsif ( $inst eq "EMOS2" ) {
			$instrument = "m2";
		}

		# Check if the user select MOS1, MOS2, PN for analysing data
		$epic_spectral_flag = 0
		  ; # Flag that determines whether the spectra should be worked out (1: True)
		if ( $gv->$inst eq "no" ) {
			print
				"INSTRUMENT $inst [exposure=$exposure]: Not selected for calculating spectra\n";
			print "Skipping to the next exposure...\n";
		}
	  	#.. Check if the current exposure has been selected to be processed
	  	elsif ( ($isExposureFlag eq "no") ||  ($isExposureLCFlag eq "no") ) 
	  	{
	  		print
			"EXPOSURE $exposure [intrument=$inst]: Not selected for producing spectrum\n";
			print "Skipping to the next exposure...\n";
		}			
		else {
			$epic_spectral_flag = 1;

			print "Working out spectra for: $inst, $exposure, $mode\n";

			create_pnProductFileNames( $instrument, $exposure, $mode );
			print "\n----> Working out Source Spectrum ..... \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE "\n----> Working out Source Spectrum ....\n";
			}

			#.. Define Global Variables
			print "   \#> Defining File Names for Source Spectrum .....\n ";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "   \#> Defining File Names for Source Spectrum .....\n";
			}

			#.. Check if the user has defined its own coordinates
			&setcoordinates();

			my $source_obsID = $gv->SRC_OBSID;
			my $source_name  = $gv->SRC_Name;
			my $source_ra    = $gv->SRC_RA;
			my $source_dec   = $gv->SRC_DEC;

			print " OK \n";
			if ( $Log_Flag == 0 ) { print MYOUTFILE " OK \n"; }

			if ( $Log_Lcur_Flag == 0 ) {
				$temp_Log_Lcur_Flag = $Log_Lcur_Flag;
				$Log_Lcur_Flag      = 1;
			}    # Deactivate Spectral Log flag if active

			#.. RA & DEC Coordinates of position to analyze

			#.. If run_edetect chain has been ran, use the closest source to the RA DEC given by the user for analysis
			my ( $ra_epic_closesource, $dec_epic_closesource,
				$epic_mindist_deg ) =
			  &find_closest_source($source_epic_region_file);

			if (   $epic_mindist_deg != 1000.
				&& $epic_mindist_deg <= $gv->EDC_sourcematch )
			{
				print
"   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to $inst).\n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
"   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";
				}
				$ra_anal  = $ra_epic_closesource;
				$dec_anal = $dec_epic_closesource;
			}
			else {
				$ra_anal  = $source_ra;     # (deg) User Coordinates
				$dec_anal = $source_dec;    # (deg) User coordinates
			}
			print "   \#> RA & DEC of source to analyse: \n";
			print "        RA : $ra_anal deg \n";
			print "        DEC: $dec_anal deg \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE "   \#> RA & DEC of source to analyse: \n";
				print MYOUTFILE "        RA : $ra_anal deg \n";
				print MYOUTFILE "        DEC: $dec_anal deg \n";
			}

			#.. Test for existance of GTI files
			&check_GTIs_Files( $instrument, $exposure, $mode, "spectral",
				$source_name );

			#.. Test for existance of PN and MOS event files
			&check_Spectra_Files( $inst, $exposure, $mode, "spectral",
				$source_name );
			
			#.. Get instrument's observing mode and prepare things according to shape of source and background regions.
			$epic_obsmode = $obsid_instruments{$source_obsID}{$instrument}{'mode'};
			$epic_obssubmode =
			  $obsid_instruments{$source_obsID}{$instrument}{'submode'};
	
			#.. Create GTI filtered event lists.
			if ($epic_spectral_flag) {
				&produce_gticleanfile( $epic_event_list,
					$epic_gti_cleanevents_file, $epic_GTI_file, $epic_obsmode,
					$instrument );
			}    # EPIC

			#.. GET some information out of the GTI filtered event lists.
			print "   \#> Information from GTI Filtered Event Files: \n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				  "   \#> Information from GTI Filtered Event Files: \n";
			}
			if ($epic_spectral_flag) {
				&event_file_inf( $epic_gti_cleanevents_file, $instrument );
			}    # EPIC

			if ( $Log_Evt_Flag == 0 ) {
				close(EVTLOG);
				$Log_Evt_Flag = 1;
			}

		  #.. Make an image using the filtered event file to get source centroid

			if ( ( ($instrument eq "m1") || ($instrument eq "m2") ) &&  ($mode eq "TimingEvts" )  )
			{
				print "   \#> Cannot produce smooth image for exposure: $exposure [instrument: $instrument and mode: $mode] \n";	
			} 
			else
			{
				if ($epic_spectral_flag  )    # && !-e $epic_gti_smoothimage_file)
				{
					&produce_smooth_image(
						$epic_gti_cleanevents_file, $epic_gti_image_file,
						$epic_gti_smoothimage_file, $instrument
					);
				}   #EPIC
			}

			#.. Run eregionanalyse
			
			if ($epic_spectral_flag) {
				#.. Using the instrument's image to work out the source centroid. Only for Imaging.
				print
				"   \#> Working out the centroid of source to analyze (Physical Units) .... \n";
				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
					"   \#> Working out the centroid of source to analyze (Physical Units) ....\n";
				}							
				$epic_spectral_flag =
			 	 prepare_spectra( $instrument, $exposure, $mode, $epic_obsmode,
					$epic_obssubmode, "SPC" );
			}

			#.. Prepare spectrum log file and output information
			if ( $Log_Spec_Flag == 0 ) {
				&prepare_spectra_log_file( $instrument, $exposure, $mode );

#					if (!$epic_spectral_flag ) {
#						print "   \#> No information to produce spectra in any camera .... exiting \n";
#						if ($Log_Flag == 0) {print MYOUTFILE "   \#> No information to produce spectra in any camera .... exiting \n";}
#						if ($Log_Spec_Flag == 0) {print SPECTRALOG "   \#> No information to produce spectra in any camera .... exiting \n";}
#						if ($Log_Spec_Flag == 0) {close(SPECTRALOG);}
#						&error_code(0);
#						return(0);
#   				}

				chdir($spectra_directory);

	   #.. Make image plots and create region files used to extract the spectrum
				&create_spec_plots_and_regions( $instrument, $exposure, $mode,"SPC" );

				#.. run epatplot
				if ( $gv->ANA_epatplot eq "yes") {
					if ($epic_spectral_flag) {
						&run_epatplot( $epic_gti_cleanevents_file, $instrument,
							"SPC", $epic_obsmode );
					}    # PN
					 # Set the flag to false. If run_all has been selected, produce_lightcurve will not run againg epatplot.
					$gv->ANA_epatplot("no");
				}

		#.. Obtain spectrum files for PN and MOS: source, bkg, arf and rmf files
				print
"   \#> Producing source/background spectrum files and arf/rmf files (especget) ..... \n";

				if ( $Log_Flag == 0 ) {
					print MYOUTFILE
"   \#> Producing source/background spectrum files and arf/rmf files (especget) ..... \n";
				}
				if ( $Log_Spec_Flag == 0 ) {
					print SPECTRALOG
"   \#> Producing source/background spectrum files and arf/rmf files (especget) ..... \n";
				}

				
				if ($epic_spectral_flag) {
					my $src_region = "SPC_" . $instrument . "_srcregion";
					my $bkg_region = "SPC_" . $instrument . "_bkgregion";

					#.. Source Regions...
					my $src_region_X  = "SPC_" . $instrument . "_srcregion_X";
					my $src_region_Y  = "SPC_" . $instrument . "_srcregion_Y";
					my $src_region_R1 = "SPC_" . $instrument . "_srcregion_R1";
					my $src_region_R2 = "SPC_" . $instrument . "_srcregion_R2";
					my $src_region_XLength =
					  "SPC_" . $instrument . "_srcregion_XLength";
					my $src_region_YLength =
					  "SPC_" . $instrument . "_srcregion_YLength";

					#.. Bkg Regions...
					my $bkg_region_X  = "SPC_" . $instrument . "_bkgregion_X";
					my $bkg_region_Y  = "SPC_" . $instrument . "_bkgregion_Y";
					my $bkg_region_R1 = "SPC_" . $instrument . "_bkgregion_R1";
					my $bkg_region_R2 = "SPC_" . $instrument . "_bkgregion_R2";
					my $bkg_region_XLength =
					  "SPC_" . $instrument . "_bkgregion_XLength";
					my $bkg_region_YLength =
					  "SPC_" . $instrument . "_bkgregion_YLength";

					#.. Area Factor
					my $epic_areafactor = "SPC_" . $instrument . "_areafactor";					

					$SRC_XC = $gv->$src_region_X;
					$SRC_YC = $gv->$src_region_Y;
					if (   $gv->$src_region eq "circle"
						|| $gv->$src_region eq "annulus" )
					{
						$RSRC1 = $gv->$src_region_R1;
					}
					if (   $gv->$src_region eq "circle"
						|| $gv->$src_region eq "annulus" )
					{
						$RSRC2 = $gv->$src_region_R2;
					}
					if ( $gv->$src_region eq "box" ) {
						$RSRC1 = $gv->$src_region_XLength;
					}
					if ( $gv->$src_region eq "box" ) {
						$RSRC2 = $gv->$src_region_YLength;
					}
					$BKG_XC = $gv->$bkg_region_X;
					$BKG_YC = $gv->$bkg_region_Y;
					if (   $gv->$bkg_region eq "circle"
						|| $gv->$bkg_region eq "annulus" )
					{
						$RBKG1 = $gv->$bkg_region_R1;
					}
					if (   $gv->$bkg_region eq "circle"
						|| $gv->$bkg_region eq "annulus" )
					{
						$RBKG2 = $gv->$bkg_region_R2;
					}
					if ( $gv->$bkg_region eq "box" ) {
						$RBKG1 = $gv->$bkg_region_XLength;
					}
					if ( $gv->$bkg_region eq "box" ) {
						$RBKG2 = $gv->$bkg_region_YLength;
					}

					my $epic_src_expr = "SPC_" . $instrument . "_src_expr";
					my $epic_bkg_expr = "SPC_" . $instrument . "_bkg_expr";									
					my $currentInteractivity = "SPC_".$instrument."_interactivity";
					my $currentds9Display = "display_ds9_SPC_".$instrument."_exposure";
					#.. Check if we are in Timing and EMOS. In this case, the interactive 
					if (($mode eq "TimingEvts") && ( ($instrument eq "m1") || ($instrument eq "m2") ) )
					{
						print "\n   \#> $instrument in $mode cannot be displayed\n";
						$gv->$currentInteractivity("no");
					}
					if ( ($gv->$epic_src_expr eq "") && (uc($gv->$currentInteractivity) eq "NO") ) {
						if ( !&test_epic_spetralfiles() ) {
							&run_especget(
								$epic_gti_cleanevents_file, $instrument,
								$SRC_XC,                    $SRC_YC,
								$RSRC1,                     $RSRC2,
								$gv->$src_region,           $BKG_XC,
								$BKG_YC,                    $RBKG1,
								$RBKG2,                     $gv->$bkg_region
							);
						}
						else 
						{
							print "\n   \#> $inst spectral files found ! \n";
							if ( $Log_Flag == 0 ) {
								print MYOUTFILE
							  	"\n   \#> $inst spectral files found ! \n";
							}
						}
					}
					else 
					{					
					    if ((uc($gv->$currentds9Display) eq "YES"))
					    {
							#.. Display ds9 and get src and bkg information
							&show_ds9($epic_gti_cleanevents_file,$instrument,$exposure,$mode,"SPC");
					    }
					    &run_especget_user_expr( $epic_gti_cleanevents_file,
								     $instrument, $gv->$epic_src_expr,
								     $gv->$epic_bkg_expr );
					}

					
								
					($epic_src_livetime, $epic_src_backscale,$epic_src_areascal)
					    = &event_file_spectralinf( $epic_sourcebkgspectrum_file,
								       $instrument );    # EPIC
					($epic_bkg_livetime, $epic_bkg_backscale,$epic_bkg_areascal)
					    = &event_file_spectralinf( $epic_bkgspectrum_file,
								       $instrument );    # EPIC
					if (   $epic_src_livetime eq "unknown"
					       or $epic_src_backscale eq "unknown"
					       or $epic_src_areascal  eq "unknown" )
					{
					    &error_code(6);
					}
					if (   $epic_bkg_livetime eq "unknown"
					       or $epic_bkg_backscale eq "unknown"
					       or $epic_bkg_areascal  eq "unknown" )
					{
					    &error_code(6);
					}
					$gv->$epic_areafactor($epic_src_backscale / $epic_bkg_backscale );

				    }				
			    }
			else 
			{
			    print "\n   \#> No $inst Spectra can be worked out \n";
			    if ( $Log_Flag == 0 ) {
				print MYOUTFILE
				    "\n   \#> No $inst Spectra can be worked out \n";
			    }
			}

			#.. Write Summary
			
			print "\n   \#> Spectral Analysis Summary :\n";
			if ( $Log_Flag == 0 ) {
			    print MYOUTFILE "\n   \#> Spectral Analysis Summary :\n";
			}
			if ( $Log_Spec_Flag == 0 ) {
			    print SPECTRALOG "\n   \#> Spectral Analysis Summary :\n";
			}
			if   ( !$epic_spectral_flag ) { $epic_summ_ana_msg = "FAILED"; }
			else                          { $epic_summ_ana_msg = "OK"; }
			
			print "      epic : analysis : $epic_summ_ana_msg \n";
			if ( $Log_Flag == 0 ) {
			    print MYOUTFILE
				"      epic : analysis : $epic_summ_ana_msg \n";
			}
			if ( $Log_Spec_Flag == 0 ) {
			    print SPECTRALOG
				"      epic : analysis : $epic_summ_ana_msg \n";
			}
			
			if ( $Log_Spec_Flag == 0 ) { close(SPECTRALOG); }
			
			if ( $temp_Log_Lcur_Flag == 0 ) {
			    $Log_Lcur_Flag = 0;
			}    # Re-activate LCurve Log flag if active
			
			&error_code(0);
		    }
		    
		
		#.. Workaround to fix the problem of having more than one exposure.
		#.. In this case, we have to reset the src and bkg variables.
		#.. Source Regions...
		my $src_region_X       = "SPC_" . $instrument . "_srcregion_X";
		my $src_region_Y       = "SPC_" . $instrument . "_srcregion_Y";
		my $src_region_R1      = "SPC_" . $instrument . "_srcregion_R1";
		my $src_region_R2      = "SPC_" . $instrument . "_srcregion_R2";
		my $src_region_XLength = "SPC_" . $instrument . "_srcregion_XLength";
		my $src_region_YLength = "SPC_" . $instrument . "_srcregion_YLength";

		#.. Bkg Regions...
		my $bkg_region_X       = "SPC_" . $instrument . "_bkgregion_X";
		my $bkg_region_Y       = "SPC_" . $instrument . "_bkgregion_Y";
		my $bkg_region_R1      = "SPC_" . $instrument . "_bkgregion_R1";
		my $bkg_region_R2      = "SPC_" . $instrument . "_bkgregion_R2";
		my $bkg_region_XLength = "SPC_" . $instrument . "_bkgregion_XLength";
		my $bkg_region_YLength = "SPC_" . $instrument . "_bkgregion_YLength";
	
		$gv->$src_region_X(0);
		$gv->$src_region_Y(0);
		$gv->$src_region_XLength(0);
		$gv->$src_region_YLength(0);
		$gv->$src_region_R1(0);
		$gv->$src_region_R2(0);
		$gv->$bkg_region_X(0);
		$gv->$bkg_region_Y(0);
		$gv->$bkg_region_XLength(0);
		$gv->$bkg_region_YLength(0);
		$gv->$bkg_region_R1(0);
		$gv->$bkg_region_R2(0);
	} ### END OF EPIC LOOP
	return (0);
}

## @method run_eregionanalyse($image_file,$r,$d,$instr,$submode,$eregionanal_file,$analysis)
# Run the SAS task eregionanalyse (Find optimum source extraction region). The source region is a circle and the background region is an annulus centered on the source region, whose inner radius is a given distance away from the source radius.
# @param image_file Image File Name
# @param r Source RA
# @param d Source DEC
# @param instr Instrument
# @param submode Observation Mode
# @param eregionanal_file File for output depending on instrument
# @param analysis Which analysis routine is calling eregionanalyse (LC,SPC,PG)
# @return Source X_Centroid, Source Y_Centroid, Optimum Source Radius (sky coord), Optimum Source Radius (image coord), Background Inner Radius (Sky coord), Background Inner Radius (Image coord), Background Outer Radius (Sky coord), Background Outer Radius (Image coord)

#========================================================================
sub run_eregionanalyse() {

	my $image_file = $_[0];
	my $r          = $_[1];
	my $d          = $_[2];
	my $instr      = $_[3];
	my $submode    = $_[4];
	my $eregionanal_file =
	  $_[5];    # Select file for output depending on instrument
	my $analysis = $_[6];    # Which analysis routine is calling eregionanalyse

	print "\n   \#> Running eregionanalyse over $image_file .... \n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE
		  "\n   \#> Running eregionanalyse over $image_file .... \n";
	}

	#.. Work out Annulus that will define the BKG Region
	if ( $instr eq "pn" ) {
		if ( $analysis eq "LC" ) {
			$areafactor = $gv->LC_pn_areafactor;
		} # How much bigger the BKG area should be with respect to the on-source Area
		if ( $analysis eq "SPC" ) { $areafactor = $gv->SPC_pn_areafactor; }
		if ( $analysis eq "PG" )  { $areafactor = 1.0; }
	}
	if ( $instr eq "m1" ) {
		if ( $analysis eq "LC" ) {
			$areafactor = $gv->LC_m1_areafactor;
		} # How much bigger the BKG area should be with respect to the on-source Area
		if ( $analysis eq "SPC" ) { $areafactor = $gv->SPC_m1_areafactor; }
		if ( $analysis eq "PG" )  { $areafactor = 1.0; }
	}
	if ( $instr eq "m2" ) {
		if ( $analysis eq "LC" ) {
			$areafactor = $gv->LC_m2_areafactor;
		} # How much bigger the BKG area should be with respect to the on-source Area
		if ( $analysis eq "SPC" ) { $areafactor = $gv->SPC_m2_areafactor; }
		if ( $analysis eq "PG" )  { $areafactor = 1.0; }
	}

#.. NOTE: 0.001666667deg = 6arcsecs (EPIC-pn PSF which the worse case scenario (EPIC-MOS = 5arcsec))
#..       The area covered by the bkg region is chosen to cover the same area as that of the source region


	my $sky2img = 1./0.05;  # 1 sky pixel = 0.05arcsec

#    my $src_radius      = 0.001666667;                             # Initial value (= EPIC-pn PSF)
	my $src_radius = 0.005555555556
	  ;    # (5 June 2008) After talking to Richard, this initial region
	       # is used to work out the centroid. So it should be bigger than 6",
	       # 20" is a good starting point.
	my $bkg_rad_factor =
	  1;    # How far from the src region is the background region
	my $bkg_innerradius = 0.01666667
	  ; # Initial value (Should be around 60arcsec, the values used in the pipeline)

	my $bkg_outerradius =
	  sqrt( $bkg_innerradius**2 + $areafactor * $src_radius**2 )
	  ;    # Initial value

	#.. Return Values
	my $x             = 1;
	my $y             = 1;
	my $xc            = 1;
	my $yc            = 1;
	my $ropt          = 1;
	my $roptsky       = 1;
	my $bkg_innerrsky = 0;
	my $bkg_outerrsky = 0.;
	my $bkg_innerrimg = 0.;
	my $bkg_outerrimg = 0.;

	$src_expr = "((RA,DEC) in circle($r,$d,$src_radius))";
	$bkg_expr =
	  "((RA,DEC) in annulus($r,$d,$bkg_innerradius,$bkg_outerradius))";
	if ( ( $instr eq "m1" || $instr eq "m2" )
		&& ($submode eq "PrimePartialW2" || $submode eq "PrimePartialW3" || $submode eq "PrimePartialW4") )
	{
		print "       WARNING: $instr Entering Small Window Settings.\n";
		if ( $Log_Flag == 0 ) {
			print MYOUTFILE
			  "       WARNING: $instr Entering Small Window Settings. \n";
		}
		
		#call to ecoordconv
		
		($xc,$yc) = &calculate_sky_coords($image_file,$instr,$epic_ecoordconv_file);
		
		$rad1 = 0.;
		$rad2 = sqrt($areafactor) * 3600. * $src_radius * $sky2img;
		$bkg_expr = "((X,Y) in annulus($xc,$yc,$rad1,$rad2))";	
	}
	
	
	@args = (
"eregionanalyse imageset=\"$image_file\" srcexp=\"$src_expr\" backexp=\"$bkg_expr\" -V 4 > $eregionanal_file"
	);
	system(@args) == 0 or &error_code(6);

	if ( -e $eregionanal_file ) {

		open( EREGION, "$eregionanal_file" )
		  or die "Cant open output file: $! \n";
		;    #open for write, overwrite
		print
		  "   \#> INFORMATION OUT OF eregionanalyse ($eregionanal_file) :\n";
		print "\t $instr : Run on location: RA: $r - DEC: $d \n";

		if ( $Log_Flag == 0 ) {
			print MYOUTFILE
"   \#> INFORMATION OUT OF eregionanalyse ($eregionanal_file) :\n";
		}
		while (<EREGION>) {
			print MYOUTFILE "\t $_";
			next if /^eregionanalyse/;
			next if /^(\s)*$/;
			print "\t $instr : $_";
			if (/xcentroid/) {
				@split = split /:/, $_;
				chomp( $x = $split[1] );
				$x =~ s/\s+//;
			}
			if (/ycentroid/) {
				@split = split /:/, $_;
				chomp( $y = $split[1] );
				$y =~ s/\s+//;
			}
			if (/optradius/) {
				( $roptsky, $roptimg ) = ( split /\s+/, $_ )[ 2, 4 ];
				$roptsky =~ s/\s+//;
				$roptimg =~ s/\s+//;
			}
		}
		close(EREGION);
	}

	#.. Background center region
	$xb = $x;
	$yb = $y;
	if ( ( $instr eq "m1" || $instr eq "m2" )
		&& ($submode eq "PrimePartialW2" || $submode eq "PrimePartialW3" || $submode eq "PrimePartialW4") )
	{
		$xb              = $xc;
		$yb              = $yc;
		$bkg_innerradius = 0.;
	}

	#.. Source Region Size
	if ( ( $instr eq "m1" || $instr eq "m2" )
		&& ($submode eq "PrimePartialW2" || $submode eq "PrimePartialW3" || $submode eq "PrimePartialW4") )
	{
		if ( $roptsky > 40.0 ) {
			$roptsky = 40.0;     # arcsec
			$roptimg = 800.0;    # image
			print
"       WARNING: $instr Radius of Source Region greater than 40 arsec. Set to 40 arsec.\n";
			if ( $Log_Flag == 0 ) {
				print MYOUTFILE
"       WARNING: $instr Radius of Source Region greater than 40 arsec. Set to 40 arsec.\n";
			}
		}
	}

	#.. In sky coordinates
	$bkg_innerrsky = 3600. *
	  $bkg_innerradius; # Inner radius of BKG region in sky coordinates (arcsec)
	$bkg_outerrsky = sqrt( $areafactor * $roptsky**2 + $bkg_innerrsky**2 );

	#.. In image coordinates
	$bkg_innerrimg =
	  3600. *
	  $bkg_innerradius *
	  $sky2img;    # Inner radius of BKG region in imag coordinates (arcsec)
	  
	$bkg_outerrimg = sqrt( $areafactor * $roptimg**2 + $bkg_innerrimg**2 );

	return (
		$x,             $y,  $roptsky,       $roptimg,
		$xb,            $yb, $bkg_innerrsky, $bkg_innerrimg,
		$bkg_outerrsky, $bkg_outerrimg
	);
}

## @method void run_especget($event_file,$instr,$src_x,$src_y,$src_L1,$src_L2,$src_rshape,$bkg_x,$bkg_y,$bkg_L1,$bkg_L2,$bkg_rshape)
# Run the SAS task specget (produce EPIC spectral products)
# @param event_file Event Fele Name
# @param instr Instrument
# @param src_x Source X_coordinate (Image Coordinates)
# @param src_y Source Y_coordinate (Image Coordinates)
# @param src_L1 Source Region Inner Radius (circle,annulus), Source Region X_Box Size (box) (Image Coordinates)
# @param src_L2 Source Region Outer Radius (circle,annulus), Source Region Y_Box Size (box) (Image Coordinates)
# @param src_rshape Source Region Shape
# @param bkg_x Background X_coordinate (Image Coordinates)
# @param bkg_y Background Y_coordinate (Image Coordinates)
# @param bkg_L1 Background Region Inner Radius (circle,annulus), Background Region X_Box Size (box) (Image Coordinates)
# @param bkg_L2 Background Region Outer Radius (circle,annulus), Background Region Y_Box Size (box) (Image Coordinates)
# @param bkg_rshape Background Region Shape
# @return Void

#========================================================================
sub run_especget() {

	my $event_file = $_[0];
	my $instr      = $_[1];
	my $src_x      = $_[2];
	my $src_y      = $_[3];
	my $src_L1     = $_[4];
	my $src_L2     = $_[5];
	my $src_rshape = $_[6];
	my $bkg_x      = $_[7];
	my $bkg_y      = $_[8];
	my $bkg_L1     = $_[9];
	my $bkg_L2     = $_[10];
	my $bkg_rshape = $_[11];

	my $src_box_angle = 0.;
	my $bkg_box_angle = 0.;

# Note: especget cant take a '+' in the name of the file, so the '+' coming from the source name has to be replaced by a '_'.

	$epic_ancillary_file =~ s/\+/\_/;
	$srcarfset = $epic_ancillary_file;
	$epic_redistribution_matrix_file =~ s/\+/\_/;
	$srcrmfset = $epic_redistribution_matrix_file;
	$epic_sourcebkgspectrum_file =~ s/\+/\_/;
	$srcspecset = $epic_sourcebkgspectrum_file;
	$epic_bkgspectrum_file =~ s/\+/\_/;
	$bckspecset = $epic_bkgspectrum_file;

	my $mode = $obsid_instruments{ $gv->SRC_OBSID }{$instr}{"mode"};
	if ( $mode eq "IMAGING" || $mode eq "imaging" ) {
		$xval = "X";
		$yval = "Y";
	}
	elsif (( $mode eq "TIMING" || $mode eq "timing" )
		&& ( $instr eq "pn" ) )
	{
		$xval = "RAWX";
		$yval = "RAWY";
	}
	elsif (( $mode eq "TIMING" || $mode eq "timing" )
		&& ( $instr eq "m1" || $instr eq "m2" ) )
	{
		$xval = "RAWX";
		$yval = "TIME";
	}
	else {
		print "ERROR \n";
		if ( $Log_Flag == 0 ) { print MYOUTFILE "ERROR \n"; }
		return (0);
	}
	chdir($spectra_directory);

	if ( $src_rshape eq "circle" ) {
		$src_rshape = "annulus";
	}    # treat circle regions as annulus with R1=0
	if ( $bkg_rshape eq "circle" ) {
		$bkg_rshape = "annulus";
	}    # treat circle regions as annulus with R1=0

	if ( ( $src_rshape eq "annulus" ) && ( $bkg_rshape eq "annulus" ) ) {
		@args = (
"especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset"
		);
	}
	elsif ( ( $src_rshape eq "box" ) && ( $bkg_rshape eq "annulus" ) ) {
		$src_L1 = $src_L1 / 2.;
		$src_L2 = $src_L2 / 2.;
		@args   = (
"especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2,$src_box_angle))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset"
		);
	}
	elsif ( ( $src_rshape eq "annulus" ) && ( $bkg_rshape eq "box" ) ) {
		$bkg_L1 = $bkg_L1 / 2.;
		$bkg_L2 = $bkg_L2 / 2.;
		@args   = (
"especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2,$bkg_box_angle))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset"
		);
	}
	elsif (( $src_rshape eq "box" )
		&& ( $bkg_rshape eq "box" )
		&& ( ( $instr eq "m1" ) || ( $instr eq "m2" ) )
		&& ( uc($mode) eq "TIMING" ) )
	{
		$src_L1 = $src_L1 / 2.;
		$src_L2 = $src_L2 / 2.;
		$bkg_L1 = $bkg_L1 / 2.;
		$bkg_L2 = $bkg_L2 / 2.;
		my $SX_min = $src_x - 10;
		my $SX_max = $src_x + 10;
		my $BX_min = $bkg_x - 10;
		my $BX_max = $bkg_x + 10;
		@args = (
"especget -t table=\"$event_file\" srcexp=\"(($xval) in [$SX_min:$SX_max])\" backexp=\"(($xval) in [$BX_min:$BX_max])\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset"
		);
	}
	else {
		$src_L1 = $src_L1 / 2.;
		$src_L2 = $src_L2 / 2.;
		$bkg_L1 = $bkg_L1 / 2.;
		$bkg_L2 = $bkg_L2 / 2.;
		@args   = (
"especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2,$src_box_angle))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2,$bkg_box_angle))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset"
		);
	}

	system(@args) == 0 or &error_code(6);

	print "   \#> $instr : Specget Summary:  \n";
	print "        SRC FILE  : $srcspecset \n";
	print "        BKG FILE  : $bckspecset \n";
	print "        RESP FILE : $srcrmfset \n";
	print "        ANCR FILE : $srcarfset \n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "   \#> $instr : Specget Summary:  \n";
		print MYOUTFILE "        SRC FILE  : $srcspecset \n";
		print MYOUTFILE "        BKG FILE  : $bckspecset \n";
		print MYOUTFILE "        RESP FILE : $srcrmfset \n";
		print MYOUTFILE "        ANCR FILE : $srcarfset \n";
	}
	if ( $Log_Spec_Flag == 0 ) {
		print SPECTRALOG "   \#> $instr : Specget Summary:  \n";
		print SPECTRALOG "        SRC FILE  : $srcspecset \n";
		print SPECTRALOG "        BKG FILE  : $bckspecset \n";
		print SPECTRALOG "        RESP FILE : $srcrmfset \n";
		print SPECTRALOG "        ANCR FILE : $srcarfset \n";
	}
	return (1);
}

## @method void run_especget_user_expr($event_file,$instr,$src_expr,$bkg_expr)
# Run the SAS task specget (produce EPIC spectral products) using the user defined expression
# @param event_file Event Fele Name
# @param instr Instrument
# @param source expression
# @param background expression
# @return Void

#========================================================================
sub run_especget_user_expr() {
	print "Runnig run_especget_user_expr\n";

	my $event_file = $_[0];
	my $instr      = $_[1];
	my $src_expr   = $_[2];
	my $bkg_expr   = $_[3];

# Note: especget cant take a '+' in the name of the file, so the '+' coming from the source name has to be replaced by a '_'.

	$epic_ancillary_file =~ s/\+/\_/;
	$srcarfset = $epic_ancillary_file;
	$epic_redistribution_matrix_file =~ s/\+/\_/;
	$srcrmfset = $epic_redistribution_matrix_file;
	$epic_sourcebkgspectrum_file =~ s/\+/\_/;
	$srcspecset = $epic_sourcebkgspectrum_file;
	$epic_bkgspectrum_file =~ s/\+/\_/;
	$bckspecset = $epic_bkgspectrum_file;

	chdir($spectra_directory);

	@args = (
"especget -t table=\"$event_file\" srcexp=\"$src_expr\" backexp=\"$bkg_expr\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset"
	);

	system(@args) == 0 or &error_code(6);

	print "   \#> $instr : Specget Summary:  \n";
	print "        SRC FILE  : $srcspecset \n";
	print "        BKG FILE  : $bckspecset \n";
	print "        RESP FILE : $srcrmfset \n";
	print "        ANCR FILE : $srcarfset \n";
	if ( $Log_Flag == 0 ) {
		print MYOUTFILE "   \#> $instr : Specget Summary:  \n";
		print MYOUTFILE "        SRC FILE  : $srcspecset \n";
		print MYOUTFILE "        BKG FILE  : $bckspecset \n";
		print MYOUTFILE "        RESP FILE : $srcrmfset \n";
		print MYOUTFILE "        ANCR FILE : $srcarfset \n";
	}
	if ( $Log_Spec_Flag == 0 ) {
		print SPECTRALOG "   \#> $instr : Specget Summary:  \n";
		print SPECTRALOG "        SRC FILE  : $srcspecset \n";
		print SPECTRALOG "        BKG FILE  : $bckspecset \n";
		print SPECTRALOG "        RESP FILE : $srcrmfset \n";
		print SPECTRALOG "        ANCR FILE : $srcarfset \n";
	}
	exit(1);
	return (1);

}

## @method void test_epic_spetralfiles()
# Test for the existance of pn spectral files
# @return Void

#========================================================================
sub test_epic_spetralfiles {

   # Specget does not take file names with '+' on them. They are changed by '_'.
	chdir($spectra_directory);
	$epic_ancillary_file =~ s/\+/\_/;
	if ( !-e $epic_ancillary_file ) {
		print "PN Ancillery file does not exist \n";
		return (0);
	}
	$epic_redistribution_matrix_file =~ s/\+/\_/;
	if ( !-e $epic_redistribution_matrix_file ) {
		print "PN Redistribution Matrix file does not exist \n";
		return (0);
	}
	$epic_sourcebkgspectrum_file =~ s/\+/\_/;
	if ( !-e $epic_sourcebkgspectrum_file ) {
		print "PN Source Spectrum file does not exist \n";
		return (0);
	}
	$epic_bkgspectrum_file =~ s/\+/\_/;
	if ( !-e $epic_bkgspectrum_file ) {
		print "PN Background Spectrum file does not exist \n";
		return (0);
	}
	return (1);
}

## @method void calculate_sky_coords()
# Converts DETX and DETY coordinates into sky images
# @return sky images

sub calculate_sky_coords {
	$imageFile = $_[0];
	$instr = $_[1];
	$output_File = $_[2];
	$DETX = 3640; #CCD2
	$DETY = -10290 ; #CCD2
	$coordtype = "DET";
	 	
	
	@args = ("ecoordconv imageset=\"$imageFile\" x=$DETX y=$DETY coordtype=$coordtype -V 4 > $output_File");
	system(@args) == 0 or &error_code(9);
	
	#Return values
	my $Xima = 0;
	my $Yima = 0;
	
	
	if ( -e $output_File ) {

		open( EREGION, "$output_File" )
		  or die "Cant open output file: $! \n";
		;    #open for write, overwrite
		print
		  "   \#> INFORMATION OUT OF ecoordconv ($output_File) :\n";
		print "\t $instr : Run on location: DETX: $DETX - DETY: $DETY \n";

		if ( $Log_Flag == 0 ) {
			print MYOUTFILE
"   \#> INFORMATION OUT OF ecoordconv ($output_File) :\n";
		}
		while (<EREGION>) {
			print MYOUTFILE "\t $_";
			next if /^ecoordconv/;
			next if /^(\s)*$/;
			if (/X: Y:/) {			
				my @values = split(/ /,$_);
				chomp( $Xima = $values[3] );
				$Xima =~ s/\s+//;
				chomp( $Yima = $values[4] );
				$Yima =~ s/\s+//;
				
			}
		}
		close(EREGION);
	}
	print "\t Image coordinates X: $Xima - Y: $Yima \n";
	if (($Xima == 0) || ($Yima == 0))
	{
		print "   \#> ERROR converting to Image coordinates\n";
		&error_code(9);
	} 
	return ($Xima,$Yima);
	
}

1;
