## @file
# Initialize analysis parameters

#========================================================================
sub pse_parameters_init(){

#.. Read Script Options
    my $name="psechain";
    my $line="@ARGV"; 
    my @save;
    my $source_name; my $source_obsID; my $source_ra; my $source_dec;
    if ( @save = grep /^\-p/ , @ARGV) {
       &pse_usage();
    };

    if ( @save = grep /^-d/ , @ARGV) {
	     system("sasdialog $name");
		 my $x = $? >> 8;
    	 exit($x);      
    }

    if ( ( @save = grep /^-v/ , @ARGV) || ( @save = grep /^--version/ , @ARGV) ) {
		&getVersion();
    }


#.. Default parameters (Start)
    
#.. Store source OBSID, Name, RA and DEC
    if ( @save = grep /^obsid=/ , @ARGV) {
      $source_obsID  ="$save[-1]";
      $source_obsID  =~ s/^.*=//; 
    }else { &error_code(101); }
	$gv->SRC_OBSID($source_obsID);

   #... If sourcename was not defined, set the variable to the OBSID	 
    if ( @save = grep /^sourcename=/ , @ARGV) {
       $source_name="$save[-1]";
       $source_name =~ s/^.*=//; 
       $source_name =~ s/\+/\_/;   
       my $valid_src_name =	&check_sourceName($source_name);   
	   $gv->SRC_Name($valid_src_name);           
    }else {$gv->SRC_Name($source_obsID);}
   
   
   if ( @save = grep /^epicsrc=/ , @ARGV) {
       $epicsrc ="$save[-1]";
       $epicsrc =~ s/^.*=//; 
    } else { $epicsrc="no"; }     
	$gv->WITH_EPIC_SRC_COORDS($epicsrc);
	
    if ( @save = grep /^ra=/ , @ARGV) {
       $source_ra ="$save[-1]";
       $source_ra  =~ s/^.*=//; 
    } else { $source_ra=-999; }
    $gv->SRC_RA($source_ra);
    
    if ( @save = grep /^dec=/ , @ARGV) {
       $source_dec ="$save[-1]";
       $source_dec  =~ s/^.*=//; 
    } else { $source_dec=-999; }
    $gv->SRC_DEC($source_dec);
    
    if ( @save = grep /^analysisoption=/ , @ARGV) {
       $analysis_action ="$save[-1]";
       $analysis_action =~ s/^.*=//; 
    } else { $analysis_action="0:all"; }

    if ( @save = grep /^EPN=/ , @ARGV) {
       $EPN ="$save[-1]";
       $EPN =~ s/^.*=//; 
    } else { $EPN="yes"; }
	$gv->EPN($EPN);

	if ( @save = grep /^EPN_exposures=/ , @ARGV) {
       $temp ="$save[-1]";       
       $temp =~ s/^.*=//; 
       @pn_exposures = split(' ',$temp);
       $gv->EPN_exposures(@pn_exposures);     # EPN exposures to be analysed
    } else {$gv->EPN_exposures("all"); }

    if ( @save = grep /^EMOS1=/ , @ARGV) {
       $EMOS1 ="$save[-1]";
       $EMOS1 =~ s/^.*=//; 
    } else { $EMOS1="yes"; }
	$gv->EMOS1($EMOS1);

	if ( @save = grep /^EMOS1_exposures=/ , @ARGV) {
       $temp ="$save[-1]";       
       $temp =~ s/^.*=//; 
       @m1_exposures = split(' ',$temp);
       $gv->EMOS1_exposures(@m1_exposures);     # EPN exposures to be analysed
    } else {$gv->EMOS1_exposures("all"); }

    if ( @save = grep /^EMOS2=/ , @ARGV) {
       $EMOS2 ="$save[-1]";
       $EMOS2 =~ s/^.*=//; 
    } else { $EMOS2="yes"; }
	$gv->EMOS2($EMOS2);

	if ( @save = grep /^EMOS2_exposures=/ , @ARGV) {
       $temp ="$save[-1]";       
       $temp =~ s/^.*=//; 
       @m2_exposures = split(' ',$temp);
       $gv->EMOS2_exposures(@m2_exposures);     # EPN exposures to be analysed
    } else {$gv->EMOS2_exposures("all"); }

    if ( @save = grep /^RGS=/ , @ARGV) {
       $RGS ="$save[-1]";
       $RGS =~ s/^.*=//; 
    } else { $RGS="yes"; }
	$gv->RGS($RGS);

    if ( @save = grep /^OM=/ , @ARGV) {
       $OM ="$save[-1]";
       $OM =~ s/^.*=//; 
    } else { $OM="yes"; }
	$gv->OM($OM);
	
    if ( @save = grep /^epic_instrument=/ , @ARGV) {
       $epic_instrument ="$save[-1]";
       $epic_instrument =~ s/^.*=//; 
    } else { $epic_instrument="NONE"; }

   if ( @save = grep /^runepatplot=/ , @ARGV) {
       $runepatplot ="$save[-1]";
       $runepatplot =~ s/^.*=//; 
    } else { $runepatplot="yes"; }
    $gv->ANA_epatplot($runepatplot);
    

#.. Default parameters (End)

#.. Compute GTI user option (Start)
#.. Store Compute GTI user options or initialize default values. 
       
   # eV, of energy cut to use for GTI	 
   if ( @save = grep /^m1energy_cut=/ , @ARGV) {
       $m1energy_cut ="$save[-1]";
       $m1energy_cut =~ s/^.*=//;
       $gv->GTI_m1energy_cut($m1energy_cut); 
    } else {$gv->GTI_m1energy_cut(10000.);  }

   if ( @save = grep /^m2energy_cut=/ , @ARGV) {
       $m2energy_cut ="$save[-1]";
       $m2energy_cut =~ s/^.*=//;
	   $gv->GTI_m2energy_cut($m2energy_cut);
    } else { $gv->GTI_m2energy_cut(10000.); } 

   if ( @save = grep /^pnenergy_cut=/ , @ARGV) {
       $pnenergy_cut ="$save[-1]";
       $pnenergy_cut =~ s/^.*=//;
       $gv->GTI_pnenergy_cut($pnenergy_cut);  
    } else { $gv->GTI_pnenergy_cut(10000.); }
   
   if ( @save = grep /^m1_Rate=/ , @ARGV) {
       $m1_Rate ="$save[-1]";
       $m1_Rate =~ s/^.*=//;
       $gv->GTI_m1_Rate($m1_Rate);        # Rate, in c/s, to cut on for GTI 
    } else { $gv->GTI_m1_Rate(0.35); }

   if ( @save = grep /^m2_Rate=/ , @ARGV) {
       $m2_Rate ="$save[-1]";
       $m2_Rate =~ s/^.*=//;
       $gv->GTI_m2_Rate($m2_Rate); 
    } else { $gv->GTI_m2_Rate(0.35);}

   if ( @save = grep /^pn_Rate=/ , @ARGV) {
       $pn_Rate ="$save[-1]";
       $pn_Rate =~ s/^.*=//;
       $gv->GTI_pn_Rate($pn_Rate);           # Rate, in c/s, to cut on for GTI 
    } else {$gv->GTI_pn_Rate(0.4);  }

   if ( @save = grep /^timebin=/ , @ARGV) {
       $timebin ="$save[-1]";
       $timebin =~ s/^.*=//;
        $gv->GTI_timebin($timebin);          # Time bin to use for GTI light curve 
    } else {  $gv->GTI_timebin(10.);  }
     
   if ( @save = grep /^PG_optimize_SN=/ , @ARGV) {
       $PG_optimize_SN ="$save[-1]";
       $PG_optimize_SN =~ s/^.*=//; 
    } else { $PG_optimize_SN="no"; }
	$gv->GTI_optimize_SN_flag($PG_optimize_SN);    
# Use best S/N to work out GTI. If set to T, use PG_gti_filtering parameters
#.. Parameters for PG_gti_filtering (if values are not equal to 0, they will be used. 
#   If values are 0, defaults will be used)
    
#.. Compute GTI user options (End)

#.. edetect chain user options (Start)

  if ( @save = grep /^EDC_pn_ene_low=/ , @ARGV) {
       $temp ="$save[-1]";
       $temp =~ s/^.*=//; 
       @EDC_pn_ene_low = split(',',$temp);
       $gv->EDC_pn_ene_low(@EDC_pn_ene_low);
    } else {$gv->EDC_pn_ene_low((300,500,2000,4500,7500));  }# energy in eV 
    
  if ( @save = grep /^EDC_pn_ene_high=/ , @ARGV) {
       $temp ="$save[-1]";
       $temp =~ s/^.*=//;
       @EDC_pn_ene_high  = split(',',$temp); 
       $gv->EDC_pn_ene_high(@EDC_pn_ene_high);
    } else { $gv->EDC_pn_ene_high(500,2000,4500,7500,12000);} # energy in eV

  if ( @save = grep /^EDC_m1_ene_high=/ , @ARGV) {
       $temp ="$save[-1]";
       $temp =~ s/^.*=//; 
       @EDC_m1_ene_high = split(',',$temp);
       $gv->EDC_m1_ene_high(@EDC_m1_ene_high);
    } else { $gv->EDC_m1_ene_high(500,2000,4500,7500,12000); } # energy in eV

  if ( @save = grep /^EDC_m1_ene_low=/ , @ARGV) {
       $temp ="$save[-1]";
       $temp =~ s/^.*=//; 
       @EDC_m1_ene_low = split(',',$temp);
       $gv->EDC_m1_ene_low(@EDC_m1_ene_low);
    } else {  $gv->EDC_m1_ene_low(200,500,2000,4500,7500);  } # energy in eV

  if ( @save = grep /^EDC_m2_ene_high=/ , @ARGV) {
       $temp ="$save[-1]";
       $temp =~ s/^.*=//;
       @EDC_m2_ene_high = split(',',$temp); 
       $gv->EDC_m2_ene_high(@EDC_m2_ene_high);
    } else { $gv->EDC_m2_ene_high(500,2000,4500,7500,12000);  } # energy in eV

  if ( @save = grep /^EDC_m2_ene_low=/ , @ARGV) {
       $temp ="$save[-1]";
       $temp =~ s/^.*=//;
       @EDC_m2_ene_low = split(',',$temp);
       $gv->EDC_m2_ene_low(@EDC_m2_ene_low);
    } else {  $gv->EDC_m2_ene_low(200,500,2000,4500,7500);  } # energy in eV
 
  if ( @save = grep /^pn_likelihood_detection=/ , @ARGV) {
       $pn_likelihood_detection ="$save[-1]";
       $pn_likelihood_detection =~ s/^.*=//; 
       $gv->EDC_pn_likelihood_detection($pn_likelihood_detection);
    } else {$gv->EDC_pn_likelihood_detection(10); }

  if ( @save = grep /^mos_likelihood_detection=/ , @ARGV) {
       $mos_likelihood_detection="$save[-1]";
       $mos_likelihood_detection=~ s/^.*=//; 
       $gv->EDC_mos_likelihood_detection($mos_likelihood_detection);
    } else {$gv->EDC_mos_likelihood_detection(10); }

  if ( @save = grep /^sourcematch=/ , @ARGV) {
       $sourcematch ="$save[-1]";
       $sourcematch =~ s/^.*=//; 
       $gv->EDC_sourcematch($sourcematch); 
    } else { $gv->EDC_sourcematch(0.001689656258); } # For source crosscorrelation (deg) (XMM-Newton pos error: 6arcsec)

  if ( @save = grep /^cameramatch=/ , @ARGV) {
       $cameramatch="$save[-1]";
       $cameramatch=~ s/^.*=//; 
       $gv->EDC_cameramatch($cameramatch);
    } else { $gv->EDC_cameramatch(0.0005555555); } # For camera crosscorrelation (deg) (Relative EPIC Camera error: 2arcsec)
  
  #.. Parameters for OM Source Detection
  if ( @save = grep /^om_sourcematch=/ , @ARGV) {
       $om_sourcematch ="$save[-1]";
       $om_sourcematch =~ s/^.*=//; 
       $gv->OM_sourcematch($om_sourcematch); # For source crosscorrelation (deg)
    } else { $gv->OM_sourcematch(0.000277777); }
    
#####################################
##### SPECTRA PARAMETERS ###########
####################################  

    if ( @save = grep /^SPC_pn_interactivity=/ , @ARGV) {
       $SPC_pn_interactivity ="$save[-1]";
       $SPC_pn_interactivity =~ s/^.*=//;
       $gv->SPC_pn_interactivity($SPC_pn_interactivity); 
    } else { $gv->SPC_pn_interactivity("no"); }
  
    if ( @save = grep /^SPC_m1_interactivity=/ , @ARGV) {
       $SPC_m1_interactivity ="$save[-1]";
       $SPC_m1_interactivity =~ s/^.*=//;
       $gv->SPC_m1_interactivity($SPC_m1_interactivity); 
    } else { $gv->SPC_m1_interactivity("no"); }
    
    if ( @save = grep /^SPC_m2_interactivity=/ , @ARGV) {
       $SPC_m2_interactivity ="$save[-1]";
       $SPC_m2_interactivity =~ s/^.*=//;
       $gv->SPC_m2_interactivity($SPC_m2_interactivity); 
    } else { $gv->SPC_m2_interactivity("no"); }

    if ( @save = grep /^SPC_pn_exposure=/ , @ARGV) {
       $SPC_pn_exposure ="$save[-1]";
       $SPC_pn_exposure =~ s/^.*=//;
       $gv->SPC_pn_exposure($SPC_pn_exposure); 
    } else { $gv->SPC_pn_exposure("no"); }

    if ( @save = grep /^SPC_m1_exposure=/ , @ARGV) {
       $SPC_m1_exposure ="$save[-1]";
       $SPC_m1_exposure =~ s/^.*=//;
       $gv->SPC_m1_exposure($SPC_m1_exposure); 
    } else { $gv->SPC_m1_exposure("no"); }

    if ( @save = grep /^SPC_m2_exposure=/ , @ARGV) {
       $SPC_m2_exposure ="$save[-1]";
       $SPC_m2_exposure =~ s/^.*=//;
       $gv->SPC_m2_exposure($SPC_m2_exposure); 
    } else { $gv->SPC_m2_exposure("no"); }

    if ( @save = grep /^display_ds9_SPC_pn_exposure=/ , @ARGV) {
       $display_ds9_SPC_pn_exposure ="$save[-1]";
       $display_ds9_SPC_pn_exposure =~ s/^.*=//;
       $gv->display_ds9_SPC_pn_exposure($display_ds9_SPC_pn_exposure); 
    } else { $gv->display_ds9_SPC_pn_exposure("no"); }

    if ( @save = grep /^display_ds9_SPC_m1_exposure=/ , @ARGV) {
       $display_ds9_SPC_m1_exposure ="$save[-1]";
       $display_ds9_SPC_m1_exposure =~ s/^.*=//;
       $gv->display_ds9_SPC_m1_exposure($display_ds9_SPC_m1_exposure); 
    } else { $gv->display_ds9_SPC_m1_exposure("no"); }

    if ( @save = grep /^display_ds9_SPC_m2_exposure=/ , @ARGV) {
       $display_ds9_SPC_m2_exposure ="$save[-1]";
       $display_ds9_SPC_m2_exposure =~ s/^.*=//;
       $gv->display_ds9_SPC_m2_exposure($display_ds9_SPC_m2_exposure); 
    } else { $gv->display_ds9_SPC_m2_exposure("no"); }
    
    # eV : Low energy Range for PN spectral fit	    
    if ( @save = grep /^SPC_pn_e1_low=/ , @ARGV) {
       $SPC_pn_e1_low ="$save[-1]";
       $SPC_pn_e1_low =~ s/^.*=//;
       $gv->SPC_pn_e1_low($SPC_pn_e1_low); 
    } else { $gv->SPC_pn_e1_low(0.3); }

	if ( @save = grep /^SPC_pn_e1_up=/ , @ARGV) {
       $SPC_pn_e1_up ="$save[-1]";
       $SPC_pn_e1_up =~ s/^.*=//;
       $gv->SPC_pn_e1_up($SPC_pn_e1_up); 
    } else { $gv->SPC_pn_e1_up(9.99999999); }
    
    if ( @save = grep /^SPC_m1_e1_low=/ , @ARGV) {
       $SPC_m1_e1_low ="$save[-1]";
       $SPC_m1_e1_low =~ s/^.*=//;
       $gv->SPC_m1_e1_low($SPC_m1_e1_low); 
    } else { $gv->SPC_m1_e1_low(0.3); }

 	if ( @save = grep /^SPC_m1_e1_up=/ , @ARGV) {
       $SPC_m1_e1_up ="$save[-1]";
       $SPC_m1_e1_up =~ s/^.*=//;
       $gv->SPC_m1_e1_up($SPC_m1_e1_up); 
    } else { $gv->SPC_m1_e1_up(9.999999); }
    
    if ( @save = grep /^SPC_m2_e1_low=/ , @ARGV) {
       $SPC_m2_e1_low ="$save[-1]";
       $SPC_m2_e1_low =~ s/^.*=//; 
       $gv->SPC_m2_e1_low($SPC_m2_e1_low);
    } else { $gv->SPC_m2_e1_low(0.3); }

 	if ( @save = grep /^SPC_m2_e1_up=/ , @ARGV) {
       $SPC_m2_e1_up ="$save[-1]";
       $SPC_m2_e1_up =~ s/^.*=//;
       $gv->SPC_m2_e1_up($SPC_m2_e1_up); 
    } else {$gv->SPC_m2_e1_up(9.9999999); }       
    
  if ( @save = grep /^SPC_pn_src_expr=/ , @ARGV) {
       $SPC_pn_src_expr ="$save[-1]";
       $SPC_pn_src_expr =~ s/^.*=//; 
       $gv->SPC_pn_src_expr($SPC_pn_src_expr);
    } else {$gv->SPC_pn_src_expr(""); }

  if ( @save = grep /^SPC_pn_bkg_expr=/ , @ARGV) {
       $SPC_pn_bkg_expr ="$save[-1]";
       $SPC_pn_bkg_expr =~ s/^.*=//; 
       	$gv->SPC_pn_bkg_expr($SPC_pn_bkg_expr);
    } else { $gv->SPC_pn_bkg_expr(""); }
    
 if ( @save = grep /^SPC_m1_src_expr=/ , @ARGV) {
       $SPC_m1_src_expr ="$save[-1]";
       $SPC_m1_src_expr =~ s/^.*=//; 
       $gv->SPC_m1_src_expr($SPC_m1_src_expr);
    } else { $gv->SPC_m1_src_expr(""); }

  if ( @save = grep /^SPC_m1_bkg_expr=/ , @ARGV) {
       $SPC_m1_bkg_expr ="$save[-1]";
       $SPC_m1_bkg_expr =~ s/^.*=//;
       $gv->SPC_m1_bkg_expr($SPC_m1_bkg_expr); 
    } else { $gv->SPC_m1_bkg_expr(""); }    

 if ( @save = grep /^SPC_m2_src_expr=/ , @ARGV) {
       $SPC_m2_src_expr ="$save[-1]";
       $SPC_m2_src_expr =~ s/^.*=//;
       $gv->SPC_m2_src_expr($SPC_m2_src_expr); 
    } else { $gv->SPC_m2_src_expr(""); }

  if ( @save = grep /^SPC_m2_bkg_expr=/ , @ARGV) {
       $SPC_m2_bkg_expr ="$save[-1]";
       $SPC_m2_bkg_expr =~ s/^.*=//; 
       $gv->SPC_m2_bkg_expr($SPC_m2_bkg_expr);
    } else { $gv->SPC_m2_bkg_expr(""); }
        


#########################################
######## LIGHTCURVE PARAMETERS ##########
#########################################

   if ( @save = grep /^LC_pn_interactivity=/ , @ARGV) {
       $LC_pn_interactivity ="$save[-1]";
       $LC_pn_interactivity =~ s/^.*=//;
       $gv->LC_pn_interactivity($LC_pn_interactivity); 
    } else { $gv->LC_pn_interactivity("no"); }
  
    if ( @save = grep /^LC_m1_interactivity=/ , @ARGV) {
       $LC_m1_interactivity ="$save[-1]";
       $LC_m1_interactivity =~ s/^.*=//;
       $gv->LC_m1_interactivity($LC_m1_interactivity); 
    } else { $gv->LC_m1_interactivity("no"); }
    
    if ( @save = grep /^LC_m2_interactivity=/ , @ARGV) {
       $LC_m2_interactivity ="$save[-1]";
       $LC_m2_interactivity =~ s/^.*=//;
       $gv->LC_m2_interactivity($LC_m2_interactivity); 
    } else { $gv->LC_m2_interactivity("no"); }

    if ( @save = grep /^LC_pn_exposure=/ , @ARGV) {
       $LC_pn_exposure ="$save[-1]";
       $LC_pn_exposure =~ s/^.*=//;
       $gv->LC_pn_exposure($LC_pn_exposure); 
    } else { $gv->LC_pn_exposure("no"); }

    if ( @save = grep /^LC_m1_exposure=/ , @ARGV) {
       $LC_m1_exposure ="$save[-1]";
       $LC_m1_exposure =~ s/^.*=//;
       $gv->LC_m1_exposure($LC_m1_exposure);
    } else { $gv->LC_m1_exposure("no"); }

    if ( @save = grep /^LC_m2_exposure=/ , @ARGV) {
       $LC_m2_exposure ="$save[-1]";
       $LC_m2_exposure =~ s/^.*=//;
       $gv->LC_m2_exposure($LC_m2_exposure); 
    } else { $gv->LC_m2_exposure("no"); }

    if ( @save = grep /^display_ds9_LC_pn_exposure=/ , @ARGV) {
       $display_ds9_LC_pn_exposure ="$save[-1]";
       $display_ds9_LC_pn_exposure =~ s/^.*=//;
       $gv->display_ds9_LC_pn_exposure($display_ds9_LC_pn_exposure); 
    } else { $gv->display_ds9_LC_pn_exposure("no"); }

    if ( @save = grep /^display_ds9_LC_m1_exposure=/ , @ARGV) {
       $display_ds9_LC_m1_exposure ="$save[-1]";
       $display_ds9_LC_m1_exposure =~ s/^.*=//;
       $gv->display_ds9_LC_m1_exposure($display_ds9_LC_m1_exposure); 
    } else { $gv->display_ds9_LC_m1_exposure("no"); }

    if ( @save = grep /^display_ds9_LC_m2_exposure=/ , @ARGV) {
       $display_ds9_LC_m2_exposure ="$save[-1]";
       $display_ds9_LC_m2_exposure =~ s/^.*=//;
       $gv->display_ds9_LC_m2_exposure($display_ds9_LC_m2_exposure); 
    } else { $gv->display_ds9_LC_m2_exposure("no"); }

    
################################
##### LIGHTCURVE PARAMETERS ####
################################

#.. Parameters for Light Curve
    
  if ( @save = grep /^LC_pn_src_expr=/ , @ARGV) {
       $LC_pn_src_expr ="$save[-1]";
       $LC_pn_src_expr =~ s/^.*=//;
       $gv->LC_pn_src_expr($LC_pn_src_expr); 
    } else { $gv->LC_pn_src_expr(""); }

  if ( @save = grep /^LC_pn_bkg_expr=/ , @ARGV) {
       $LC_pn_bkg_expr ="$save[-1]";
       $LC_pn_bkg_expr =~ s/^.*=//;
       $gv->LC_pn_bkg_expr($LC_pn_bkg_expr); 
    } else { $gv->LC_pn_bkg_expr(""); }
    
 if ( @save = grep /^LC_m1_src_expr=/ , @ARGV) {
       $LC_m1_src_expr ="$save[-1]";
       $LC_m1_src_expr =~ s/^.*=//; 
       $gv->LC_m1_src_expr($LC_m1_src_expr);
    } else {$gv->LC_m1_src_expr("");}

  if ( @save = grep /^LC_m1_bkg_expr=/ , @ARGV) {
       $LC_m1_bkg_expr ="$save[-1]";
       $LC_m1_bkg_expr =~ s/^.*=//;
       $gv->LC_m1_bkg_expr($LC_m1_bkg_expr); 
    } else { $gv->LC_m1_bkg_expr(""); }    

 if ( @save = grep /^LC_m2_src_expr=/ , @ARGV) {
       $LC_m2_src_expr ="$save[-1]";
       $LC_m2_src_expr =~ s/^.*=//;
       $gv->LC_m2_src_expr($LC_m2_src_expr); 
    } else { $gv->LC_m2_src_expr(""); }

  if ( @save = grep /^LC_m2_bkg_expr=/ , @ARGV) {
       $LC_m2_bkg_expr ="$save[-1]";
       $LC_m2_bkg_expr =~ s/^.*=//;
       $gv->LC_m2_bkg_expr($LC_m2_bkg_expr); 
    } else { $gv->LC_m2_bkg_expr(""); }
    
  if ( @save = grep /^LC_pn_timebinsize=/ , @ARGV) {
       $LC_pn_timebinsize ="$save[-1]";
       $LC_pn_timebinsize =~ s/^.*=//;
       $gv->LC_pn_timebinsize($LC_pn_timebinsize); 
    } else { $gv->LC_pn_timebinsize(100); }
 
 if ( @save = grep /^LC_pn_e1_low=/ , @ARGV) {
       $LC_pn_e1_low ="$save[-1]";
       $LC_pn_e1_low =~ s/^.*=//;
       $gv->LC_pn_e1_low($LC_pn_e1_low); 
    } else { $gv->LC_pn_e1_low(200.); }

 if ( @save = grep /^LC_pn_e1_up=/ , @ARGV) {
       $LC_pn_e1_up ="$save[-1]";
       $LC_pn_e1_up =~ s/^.*=//;
       	$gv->LC_pn_e1_up($LC_pn_e1_up); 
    } else { $gv->LC_pn_e1_up(10000.); }

 if ( @save = grep /^LC_m1_timebinsize=/ , @ARGV) {
       $LC_m1_timebinsize ="$save[-1]";
       $LC_m1_timebinsize =~ s/^.*=//;
       $gv->LC_m1_timebinsize($LC_m1_timebinsize); 
    } else { $gv->LC_m1_timebinsize(100); }
    

if ( @save = grep /^LC_m1_e1_low=/ , @ARGV) {
       $LC_m1_e1_low ="$save[-1]";
       $LC_m1_e1_low =~ s/^.*=//;
       $gv->LC_m1_e1_low($LC_m1_e1_low); 
    } else { $gv->LC_m1_e1_low(200.); }

 if ( @save = grep /^LC_m1_e1_up=/ , @ARGV) {
       $LC_m1_e1_up ="$save[-1]";
       $LC_m1_e1_up =~ s/^.*=//;
       $gv->LC_m1_e1_up($LC_m1_e1_up); 
    } else { $gv->LC_m1_e1_up(10000.); }
 
  if ( @save = grep /^LC_m2_timebinsize=/ , @ARGV) {
       $LC_m2_timebinsize ="$save[-1]";
       $LC_m2_timebinsize =~ s/^.*=//;
       $gv->LC_m2_timebinsize($LC_m2_timebinsize); 
    } else { $gv->LC_m2_timebinsize(100); }
     
    if ( @save = grep /^LC_m2_e1_low=/ , @ARGV) {
       $LC_m2_e1_low ="$save[-1]";
       $LC_m2_e1_low =~ s/^.*=//;
       $gv->LC_m2_e1_low($LC_m2_e1_low); 
    } else { $gv->LC_m2_e1_low(200.); }

 if ( @save = grep /^LC_m2_e1_up=/ , @ARGV) {
       $LC_m2_e1_up ="$save[-1]";
       $LC_m2_e1_up =~ s/^.*=//;
       $gv->LC_m2_e1_up($LC_m2_e1_up); 
    } else { $gv->LC_m2_e1_up(10000.); }
    

#... RGS parameters	
            
   if ( @save = grep /^withsrc=/ , @ARGV) {
       $withsrc ="$save[-1]";
       $withsrc =~ s/^.*=//; 
    } else { $withsrc="no"; }
    $gv->RGS_withsrc($withsrc);
    
   if ( @save = grep /^srcstyle=/ , @ARGV) {
       $srcstyle ="$save[-1]";
       $srcstyle =~ s/^.*=//; 
    } else { $srcstyle="radec"; }
    $gv->RGS_srcstyle($srcstyle);
    
   if ( @save = grep /^rgs_srcra=/ , @ARGV) {
       $rgs_srcra ="$save[-1]";
       $rgs_srcra=~ s/^.*=//; 
    } else { $rgs_srcra="-999"; }
    $gv->RGS_srcra($rgs_srcra);
    
   if ( @save = grep /^rgs_srcdec=/ , @ARGV) {
       $rgs_srcdec ="$save[-1]";
       $rgs_srcdec=~ s/^.*=//; 
    } else { $rgs_srcdec="-999"; }
	$gv->RGS_srcdec($rgs_srcdec);
	
   if ( @save = grep /^srcdisp=/ , @ARGV) {
       $srcdisp ="$save[-1]";
       $srcdisp=~ s/^.*=//; 
    } else { $srcdisp="0"; }
	$gv->RGS_srcdisp($srcdisp);
	
   if ( @save = grep /^srcxdsp=/ , @ARGV) {
       $srcxdsp ="$save[-1]";
       $srcxdsp=~ s/^.*=//; 
    } else { $srcxdsp="0"; }
	$gv->RGS_srcxdsp($srcxdsp);
	
  if ( @save = grep /^srclabel=/ , @ARGV) {
       $srclabel ="$save[-1]";
       $srclabel=~ s/^.*=//; 
    } else { $srclabel="USER"; }
	$gv->RGS_srclabel($srclabel);
	
   if ( @save = grep /^srcrate=/ , @ARGV) {
       $srcrate ="$save[-1]";
       $srcrate=~ s/^.*=//; 
    } else { $srcrate=0; }
	$gv->RGS_srcrate($srcrate);
	
   if ( @save = grep /^keepcool=/ , @ARGV) {
       $keepcool ="$save[-1]";
       $keepcool=~ s/^.*=//; 
    } else { $keepcool=1; }
	$gv->RGS_keepcool($keepcool);
	    
   if ( @save = grep /^rejflags=/ , @ARGV) {
       $rejflags ="$save[-1]";
       $rejflags=~ s/^.*=//; 
    } else { $rejflags="BAD_SHAPE ON_NODE_INTERFACE ON_BADPIX NEXT_TO_BADPIX ON_WINDOW_BORDER BELOW_ACCEPTANCE"; }    
	$gv->RGS_rejflags($rejflags);
	
   if ( @save = grep /^spectrumbinning=/ , @ARGV) {
       $spectrumbinning ="$save[-1]";
       $spectrumbinning=~ s/^.*=//; 
    } else { $spectrumbinning="lambda"; }    
	$gv->RGS_spectrumbinnig($spectrumbinning); 
	    
    if ( @save = grep /^LC_rgs1_timebinsize=/ , @ARGV) {
       $LC_rgs1_timebinsize ="$save[-1]";
       $LC_rgs1_timebinsize =~ s/^.*=//; 
    } else { $LC_rgs1_timebinsize=200; }
	$gv->LC_rgs1_timebinsize($LC_rgs1_timebinsize);
	
    if ( @save = grep /^LC_rgs2_timebinsize=/ , @ARGV) {
       $LC_rgs2_timebinsize ="$save[-1]";
       $LC_rgs2_timebinsize =~ s/^.*=//; 
    } else { $LC_rgs2_timebinsize=200; }
 	$gv->LC_rgs2_timebinsize($LC_rgs2_timebinsize);   
    
    
#.. edetect chain user options (End)


	
	$gv->epic_instrument($epic_instrument);

    $gv->PG_timebinning(100); # secs; Bin size for src and bkg light curves
    $gv->PG_timestart(0);     # secs; Time of the first event
    $gv->PG_timeend(0);       # secs; Time of the last event
    $gv->PG_minpi(0);         # eV; Minimum event energy to use
    $gv->PG_maxpi(0);         # eV; Maximum event energy to use
    
    $gv->PG_pn_srcregion("circle");         # Shape of the bkg region
    $gv->PG_pn_srcregion_X(0.);             # SOURCE REGION. X_center in PHYSICAL PIXELS
    $gv->PG_pn_srcregion_Y(0.);             # SOURCE REGION. Y_center in PHYSICAL PIXELS
    $gv->PG_pn_srcregion_R1(400);           # SOURCE REGION. radius in PHYSICAL PIXELS (inner)
    $gv->PG_pn_srcregion_R2(0.);            # SOURCE REGION. radius in PHYSICAL PIXELS (outer)
    $gv->PG_pn_srcregion_XLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_pn_srcregion_YLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_pn_bkgregion("annulus");        # Shape of the bkg region
    $gv->PG_pn_bkgregion_X(0.);             # BACKGROUND ANNULAR REGION. X_center in PHYSICAL PIXELS
    $gv->PG_pn_bkgregion_Y(0.);             # BACKGROUND ANNULAR REGION. Y_center in PHYSICAL PIXELS
    $gv->PG_pn_bkgregion_R1(0.);            # BACKGROUND ANNULAR REGION. inner_radius in PHYSICAL PIXELS
    $gv->PG_pn_bkgregion_R2(13200);         # BACKGROUND ANNULAR REGION. outer_radius in PHYSICAL PIXELS
    $gv->PG_pn_bkgregion_XLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_pn_bkgregion_YLength(0.); 	    # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_pn_areafactor(2.);              # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
    $gv->PG_m1_srcregion("circle");         # Shape of the bkg region
    $gv->PG_m1_srcregion_X(0.);             # SOURCE REGION. X_center in PHYSICAL PIXELS
    $gv->PG_m1_srcregion_Y(0.);             # SOURCE REGION. Y_center in PHYSICAL PIXELS
    $gv->PG_m1_srcregion_R1(400);           # SOURCE REGION. radius in PHYSICAL PIXELS (inner)
    $gv->PG_m1_srcregion_R2(0.);            # SOURCE REGION. radius in PHYSICAL PIXELS (outer)
    $gv->PG_m1_srcregion_XLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m1_srcregion_YLength(0.); 	    # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m1_bkgregion("annulus");        # Shape of the bkg region
    $gv->PG_m1_bkgregion_X(0.);             # BACKGROUND ANNULAR REGION. X_center in PHYSICAL PIXELS
    $gv->PG_m1_bkgregion_Y(0.);             # BACKGROUND ANNULAR REGION. Y_center in PHYSICAL PIXELS
    $gv->PG_m1_bkgregion_R1(0.);            # BACKGROUND ANNULAR REGION. inner_radius in PHYSICAL PIXELS
    $gv->PG_m1_bkgregion_R2(13200);         # BACKGROUND ANNULAR REGION. outer_radius in PHYSICAL PIXELS 
    $gv->PG_m1_bkgregion_XLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m1_bkgregion_YLength(0.); 	    # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m1_areafactor(2.);              # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
    $gv->PG_m2_srcregion("circle");         # Shape of the bkg region
    $gv->PG_m2_srcregion_X(0.);             # SOURCE REGION. X_center in PHYSICAL PIXELS
    $gv->PG_m2_srcregion_Y(0.);             # SOURCE REGION. Y_center in PHYSICAL PIXELS
    $gv->PG_m2_srcregion_R1(400);           # SOURCE REGION. radius in PHYSICAL PIXELS (inner)
    $gv->PG_m2_srcregion_R2(0.);            # SOURCE REGION. radius in PHYSICAL PIXELS (outer)
    $gv->PG_m2_srcregion_XLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m2_srcregion_YLength(0.); 	    # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m2_bkgregion("annulus");        # Shape of the bkg region
    $gv->PG_m2_bkgregion_X(0.);             # BACKGROUND ANNULAR REGION. X_center in PHYSICAL PIXELS
    $gv->PG_m2_bkgregion_Y(0.);             # BACKGROUND ANNULAR REGION. Y_center in PHYSICAL PIXELS
    $gv->PG_m2_bkgregion_R1(0.);            # BACKGROUND ANNULAR REGION. inner_radius in PHYSICAL PIXELS
    $gv->PG_m2_bkgregion_R2(13200);         # BACKGROUND ANNULAR REGION. outer_radius in PHYSICAL PIXELS
    $gv->PG_m2_bkgregion_XLength(0.);       # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m2_bkgregion_YLength(0.); 	    # SOURCE REGION. For box region centered in X,Y. 0: default
    $gv->PG_m2_areafactor(2.);              # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
 	$gv->PG_pn_src_expr("");
 	$gv->PG_m1_src_expr("");
 	$gv->PG_m2_src_expr("");
 	$gv->PG_pn_bkg_expr("");
 	$gv->PG_m1_bkg_expr("");
 	$gv->PG_m2_bkg_expr("");



    $gv->LC_pn_srcregion("circle");    # Shape of source extraction region (circle,annulus,box). Default: circle
    $gv->LC_pn_srcregion_X(0.);        # For circle region. 0: default
    $gv->LC_pn_srcregion_Y(0.);        # For circle region. 0: default
    $gv->LC_pn_srcregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_pn_srcregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_pn_srcregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_pn_srcregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_pn_bkgregion("annulus");   # Shape of background extraction region (circle,annulus,box). Default: Annulus
    $gv->LC_pn_bkgregion_X(0.);        # For circle region. 0: default
    $gv->LC_pn_bkgregion_Y(0.);        # For circle region. 0: default
    $gv->LC_pn_bkgregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_pn_bkgregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: defaultt
    $gv->LC_pn_bkgregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_pn_bkgregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_pn_areafactor(2.);         # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
    $gv->LC_m1_srcregion("circle");    # Shape of source extraction region (circle,annulus,box). Default: circle
    $gv->LC_m1_srcregion_X(0.);        # For circle region. 0: default
    $gv->LC_m1_srcregion_Y(0.);        # For circle region. 0: default
    $gv->LC_m1_srcregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: defualt
    $gv->LC_m1_srcregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m1_srcregion_XLength(0.);  # For box region centered in X,Y. 0: automatic box region
    $gv->LC_m1_srcregion_YLength(0.);  # For box region centered in X,Y. 0: automatic box region
    $gv->LC_m1_bkgregion("annulus");   # Shape of background extraction region (circle,annulus,box). Default: Annulus
    $gv->LC_m1_bkgregion_X(0.);        # For circle region. 0: default
    $gv->LC_m1_bkgregion_Y(0.);        # For circle region. 0: default
    $gv->LC_m1_bkgregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m1_bkgregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m1_bkgregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_m1_bkgregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_m1_areafactor(2.);         # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
    $gv->LC_m2_srcregion("circle");    # Shape of source extraction region (circle,annulus,box). Default: circle
    $gv->LC_m2_srcregion_X(0.);        # For circle region. 0: default
    $gv->LC_m2_srcregion_Y(0.);        # For circle region. 0: default
    $gv->LC_m2_srcregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m2_srcregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m2_srcregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_m2_srcregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_m2_bkgregion("annulus");   # Shape of background extraction region (circle,annulus,box). Default: Annulus
    $gv->LC_m2_bkgregion_X(0.);        # For circle region. 0: default
    $gv->LC_m2_bkgregion_Y(0.);        # For circle region. 0: default
    $gv->LC_m2_bkgregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m2_bkgregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->LC_m2_bkgregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_m2_bkgregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->LC_m2_areafactor(2.);         # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area




    $gv->SPC_pn_srcregion("circle");    # Shape of source extraction region (circle,annulus,box). Default: circle
    $gv->SPC_pn_srcregion_X(0.);        # For circle region. 0: default
    $gv->SPC_pn_srcregion_Y(0.);        # For circle region. 0: default
    $gv->SPC_pn_srcregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_pn_srcregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_pn_srcregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_pn_srcregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_pn_bkgregion("annulus");   # Shape of background extraction region (circle,annulus,box). Default: Annulus
    $gv->SPC_pn_bkgregion_X(0.);        # For circle region. 0: default
    $gv->SPC_pn_bkgregion_Y(0.);        # For circle region. 0: default
    $gv->SPC_pn_bkgregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_pn_bkgregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: defaultt
    $gv->SPC_pn_bkgregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_pn_bkgregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_pn_areafactor(2.);         # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
    $gv->SPC_m1_srcregion("circle");    # Shape of source extraction region (circle,annulus,box). Default: circle
    $gv->SPC_m1_srcregion_X(0.);        # For circle region. 0: default
    $gv->SPC_m1_srcregion_Y(0.);        # For circle region. 0: default
    $gv->SPC_m1_srcregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: defualt
    $gv->SPC_m1_srcregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m1_srcregion_XLength(0.);  # For box region centered in X,Y. 0: automatic box region
    $gv->SPC_m1_srcregion_YLength(0.);  # For box region centered in X,Y. 0: automatic box region
    $gv->SPC_m1_bkgregion("annulus");   # Shape of background extraction region (circle,annulus,box). Default: Annulus
    $gv->SPC_m1_bkgregion_X(0.);        # For circle region. 0: default
    $gv->SPC_m1_bkgregion_Y(0.);        # For circle region. 0: default
    $gv->SPC_m1_bkgregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m1_bkgregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m1_bkgregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_m1_bkgregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_m1_areafactor(2.);         # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area
    $gv->SPC_m2_srcregion("circle");    # Shape of source extraction region (circle,annulus,box). Default: circle
    $gv->SPC_m2_srcregion_X(0.);        # For circle region. 0: default
    $gv->SPC_m2_srcregion_Y(0.);        # For circle region. 0: default
    $gv->SPC_m2_srcregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m2_srcregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m2_srcregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_m2_srcregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_m2_bkgregion("annulus");   # Shape of background extraction region (circle,annulus,box). Default: Annulus
    $gv->SPC_m2_bkgregion_X(0.);        # For circle region. 0: default
    $gv->SPC_m2_bkgregion_Y(0.);        # For circle region. 0: default
    $gv->SPC_m2_bkgregion_R1(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m2_bkgregion_R2(0.);       # For circle/annulus region centered in X,Y. 0: default
    $gv->SPC_m2_bkgregion_XLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_m2_bkgregion_YLength(0.);  # For box region centered in X,Y. 0: default
    $gv->SPC_m2_areafactor(2.);         # For default circle regions: How much bigger the BKG area should be with respect to the on-source Area



    return(0);
}
1;
