## @file
# Run the SAS task edetect_chain on EPIC event files

## @method void run_edetectchain()
# Runs the SAS task edetect_chain and cross-correlates the user source position with the EPIC source list.
# @return Void

#======================================================================== 
sub run_edetectchain(){ print "\n----> Running edetect_chain ..... \n"; if
($Log_Flag == 0) {print MYOUTFILE "\n----> Running edetect_chain ....\n";}

#.. Test for ccf file and set ENV variable
	@test_ccf=&test_ccf();
	my $ccffile=$test_ccf[1];
	if (!$test_ccf[0]) {print "      Run cifbuild first \n"; &error_code(5);}
	&set_ccf($ccffile);
	
#.. Test for odfingest file and set ENV variable     
	@test_odfingest=&test_odfingest();
	my $odfingestfile=$test_odfingest[1];
	if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(5);}
	&set_odfingest($odfingestfile);
	    
#.. Print sas_setup    
	&print_sas_setup();

#.. Define Global Variables
    print "   \#> Defining File Names for edetect chain ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for edetect chain  ..... ";}
 
    use vars qw(
    	$epic_event_list
		$epic_edetect_flag
		$epic_GTI_file

		$epic_fullimage 
		$epic_emllist_file
		$epic_eboxllist_file 
		$epic_eboxmlist_file
		$source_epicregion_file	       
		);

#.. Cross match variables
	my %pnMap = ();
	my %m1Map = ();
	my %m2Map = ();
#.. Get event files
   	my @files = &get_EPIC_eventfiles();
	
 	my %epicFileMap = get_EPIC_Exposures(@files);
	my $inst;
	my $exposure;
	my $mode;
   	if ( $#files == 0 ) {&error_code(4);} 
	
	my $source_ra = "";
	my $source_dec = "";
	foreach my $file (@files) 
	{

		($inst,$exposure,$mode) = &get_File_Info($file );
		my $isExposureFlag = isExposureSelected($exposure,$inst,"main");	
			

	    $epic_edetect_flag                = 0; # Flag that determines whether the spectra should be worked out (1: True)
	
		# Check if the user select MOS1, MOS2, PN for analysing data
		if ($gv->$inst eq "no") 
		{ 
			print
				"INSTRUMENT $inst [exposure=$exposure]: Not selected for running edetect_chain\n";
			print "Skipping to the next exposure...\n";
		}
	  	#.. Check if the current exposure has been selected to be processed
	  	elsif ( $isExposureFlag eq "no") 
	  	{
	  		print
			"EXPOSURE $exposure [intrument=$inst]: Not selected for running edetect_chain\n";
			print "Skipping to the next exposure...\n";
		}		
		else
		{
		  $epic_edetect_flag = 1;
		  if ($mode =~ m/Imaging/)
		  {			  			  	
			#.. Check if the user has defined its own coordinates
			&setcoordinates();	
			
		    my $source_obsID = $gv->SRC_OBSID;
		    my $source_name  = $gv->SRC_Name;
		    $source_ra    = $gv->SRC_RA;
		    $source_dec   = $gv->SRC_DEC;
		
			my $instrument = "";
			if ($inst eq "EPN") {		 
				$instrument = "pn";
			} elsif ($inst eq "EMOS1") {
				$instrument = "m1";
			} elsif ($inst eq "EMOS2") {
				$instrument = "m2";
			}
			print "   \#> Current Exposure: [$inst,$exposure,$mode] ..... \n";
	    	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Current Exposure: [$inst,$exposure,$mode]  ..... ";}
	 
	 
		    print "OK \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}
		
		#.. Definition of file names
			create_pnProductFileNames( $instrument, $exposure, $mode );		    
		    	
		#.. Energy Ranges and detection likelihoods for PN and MOS
			my $currentEnerLow ="EDC_".$instrument."_ene_low";
			my $currentEnerHigh ="EDC_".$instrument."_ene_high";		
		    @epic_ene_low  = $gv->$currentEnerLow; 
		    @pn_ene_high = $gv->$currentEnerHigh; 
		 	my $currentInst = "pn";
		 	if (($instrument eq "m1") || ($instrument eq "m2")) {
	    		$currentInst = "mos";
	    	}
	    	
		    my $currentLikelihood = "EDC_".$currentInst."_likelihood_detection";
		    $epic_likelihood_detection  = $gv->$currentLikelihood; 
				
		
		#.. Test for existance of GTI files
		    if ($Log_GTIEvt_Flag == 0) {
				my $gtievent_log=$results_directory."gtievent_".$source_name."\.log";
				if (-e $gtievent_log) {unlink("$gtievent_log") or die "Cannot rm file: $!";}    
				open(GTILOG, "> $gtievent_log") or die "Cant open output file: $! \n";    #open for write, overwrite
		    }
		
			
		    @epicGTI=&test_epic_gtifile($instrument,$exposure,$mode);
		    if ($epicGTI[0]){
				@epicinfoGTI=&gti_file_inf($epic_GTI_file,$instrument);
				if ($epicinfoGTI[0] != 0.0){$epic_GTI_file=$epicGTI[1];} else {$epic_edetect_flag = 0;}
		    } else {$epic_edetect_flag = 0;}   			
		    
		    if ($Log_GTIEvt_Flag == 0) {
			close(GTILOG);
			$Log_GTIEvt_Flag = 1;	
		    }
		
		#.. Test for existance of event files    
		    if ($Log_Evt_Flag == 0) {
				my $event_log=$results_directory."event_".$source_name."\.log";
				if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}    
				open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";    #open for write, overwrite
		    }
		
		    @test_epicevt=&test_epiceventfiles($inst,$exposure,$mode);
		    $epic_event_list=$test_epicevt[1]; 
		    if ($test_epicevt[0]) {$epic_event_list=$test_epicevt[1];} else {$epic_edetect_flag = 0;}		   
		    if ($epic_edetect_flag eq 1) {@epicinfo=&event_file_inf($epic_event_list,$instrument);}
		 	
		    if ($Log_Evt_Flag == 0) {
				close(EVTLOG);
				$Log_Evt_Flag = 1;	
		    }
		
		#.. Get instrument's observing mode and prepare things according to shape of source and background regions.  
		    $epic_obsmode = $obsid_instruments{$source_obsID}{$instrument}{'mode'};
	
		    
		#.. Create EPIC GTI filtered event lists.
		
		    if ($epic_edetect_flag ) 
		    {
		    	&produce_gticleanfile($epic_event_list,$epic_gti_cleanevents_file,$epic_GTI_file,$epic_obsmode,$instrument);
		    }   
		
		#.. Prepare edetect chain log file    
		    chdir($images_directory);
		
		    if ($Log_Edet_Flag == 0) {
				my $edetect_log=$results_directory."edetect_chain_".$source_name."\.log";
				if (-e $edetect_log) {unlink("$edetect_log") or die "Cannot rm file: $!";}    
				open(EDETECTLOG, "> $edetect_log") or die "Cant open output file: $! \n";    #open for write, overwrite
		    }
		
		#.. Test if I have to run edetect chain or simply produce images.    
		    
		    print "\n----> Checking if edetect_chain needs to be run ..... \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Checking if edetect_chain needs to be run ....\n";}	  
		     
		    if (!-e $epic_fullimage && $epic_edetect_flag eq 1) {&epic_create_energy_bands($instrument,$exposure,$mode);}	    	  
		    if (!-e $epic_emllist_file && $epic_edetect_flag eq 1) {&epic_edetect_chain($instrument,$exposure,$mode);}
	
		#..
		#.. Produce Images 
		#..
		    print "\n----> Producing Instruments Images ..... \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Producing Instruments Images ....\n";}
		    
		#.. For EPIC
		    @epic_closesource = (0.,0.,1000.);
		    	
		    if ($epic_edetect_flag eq 1)
		    {	     		    		
				if ($epicinfo[2] eq "IMAGING") 
				{			 	
			    	if (-e $epic_emllist_file) {   
			    		#.. Has our the source been detected in EPIC ?
						@epic_closesource=&epic_image($instrument,$exposure,$mode);
						if ($instrument eq "pn")
						{					
							push( @{$pnMap{$exposure}},@epic_closesource );		
						}
						if ($instrument eq "m1")
						{
							push (@{$m1Map{$exposure}}, @epic_closesource);
						}
						if ($instrument eq "m2")
						{
							push (@{$m2Map{$exposure}}, @epic_closesource);
						}
	
						if ($epic_closesource[2] <= $gv->EDC_sourcematch){$epic_edetect_flag = 1;}else{$epic_edetect_flag = 0;}   
			    	} else {
						print "   \#> No Sources detected in $instrument within ",join(", ", $gv->EDC_sourcematch)," deg \n";
						if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in $instrument within",join(", ", $gv->EDC_sourcematch)," deg \n";}
						$epic_edetect_flag = 0;
			    	}
				} else 
				{
			    	print "   \#> Data is not IMAGING mode. Ignoring $instrument \n";
			    	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Data is not IMAGING mode. Ignoring $instrument \n"};
			    	$epic_edetect_flag = 0;
				}
		    } else 
		    {
				print "   \#> No Sources detected in $instrument \n";
				if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in $instrument \n";}
				$epic_edetect_flag = 0;
		    }		
		
		#.. Get count rate information if source has been detected
		
		    if (-e $epic_emllist_file && $epic_edetect_flag eq 1) 
			    {@epicsrcrate=&get_source_count_rate($epic_emllist_file,$instrument,0,$epic_closesource[0],$epic_closesource[1]);} 
		    if (-e $epic_emllist_file && $epic_edetect_flag eq 1) 
			    {@pnbkgrate=&get_bkg_count_rate($epic_emllist_file,$instrument,$epic_closesource[0],$epic_closesource[1]);} 
		
		#.. Get Source count rate information if source has been detected in different energy bands
		    print "\n   \#> Count Rate Information: \n";
		    if ($Log_Edet_Flag == 0){print EDETECTLOG "\n   \#> Count Rate Information: \n";}
		    if ($Log_Flag == 0){print MYOUTFILE "\n   \#> Count Rate Information: \n";}
		
		    if (-e $epic_emllist_file && $epic_edetect_flag eq 1){
			@epicsrcrate_E1=&get_source_count_rate($epic_emllist_file,$instrument,1,$epic_closesource[0],$epic_closesource[1]);
			@epicsrcrate_E2=&get_source_count_rate($epic_emllist_file,$instrument,2,$epic_closesource[0],$epic_closesource[1]);
			@epicsrcrate_E3=&get_source_count_rate($epic_emllist_file,$instrument,3,$epic_closesource[0],$epic_closesource[1]);
			@epicsrcrate_E4=&get_source_count_rate($epic_emllist_file,$instrument,4,$epic_closesource[0],$epic_closesource[1]);
			@epicsrcrate_E5=&get_source_count_rate($epic_emllist_file,$instrument,5,$epic_closesource[0],$epic_closesource[1]);
		    } 
		
		#.. Output rate information onto different log files  
		    print "       Source IDs (from emllist.fits table): \n";
		    if ($epic_edetect_flag eq 1) {print "        $instrument SRC ID  : $epicsrcrate[0] \n";} else {print "        No positive $instrument identification\n";}
	
		    print "       CCD IDs (from emllist.fits table): \n";
		    if ($epic_edetect_flag eq 1) {print "        $instrument CCD ID  : $epicsrcrate[3] \n";} else {print "        No positive $instrument identification\n";}
	
		    print "       Source and BKG Count Rates in entire energy range considered: \n";
		    if ($epic_edetect_flag eq 1) {print "        $instrument  : \n";} else {print "        No positive $instrument identification\n";}
		    if ($epic_edetect_flag eq 1) {print "          SRC Rate (cts/s): $epicsrcrate[1] +/- $epicsrcrate[2] \n";}
		    if ($epic_edetect_flag eq 1) {print "          BKG Rate (cts/s): $pnbkgrate[1] +/- $pnbkgrate[2] \n";}
	
		    print "       Source Count Rates in different energy ranges considered: \n";
		    if ($epic_edetect_flag eq 1) {print "        $instrument  : \n";} else {print "        No positive $instrument identification\n";}
		    if ($epic_edetect_flag eq 1) {print "          SRC Rate (cts/s): $epicsrcrate_E1[1] +/- $epicsrcrate_E1[2] ($epic_ene_low[0] - $pn_ene_high[0]) \n";}
		    if ($epic_edetect_flag eq 1) {print "          SRC Rate (cts/s): $epicsrcrate_E2[1] +/- $epicsrcrate_E2[2] ($epic_ene_low[1] - $pn_ene_high[1]) \n";}
		    if ($epic_edetect_flag eq 1) {print "          SRC Rate (cts/s): $epicsrcrate_E3[1] +/- $epicsrcrate_E3[2] ($epic_ene_low[2] - $pn_ene_high[2]) \n";}
		    if ($epic_edetect_flag eq 1) {print "          SRC Rate (cts/s): $epicsrcrate_E4[1] +/- $epicsrcrate_E4[2] ($epic_ene_low[3] - $pn_ene_high[3]) \n";}
		    if ($epic_edetect_flag eq 1) {print "          SRC Rate (cts/s): $epicsrcrate_E5[1] +/- $epicsrcrate_E5[2] ($epic_ene_low[4] - $pn_ene_high[4]) \n";}
	
		    if ($Log_Edet_Flag == 0) {
				print EDETECTLOG "       Source IDs (from emllist.fits table): \n";
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "        $instrument SRC ID  : $epicsrcrate[0] \n";}
			
				print EDETECTLOG "       CCD IDs (from emllist.fits table): \n";
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "        $instrument CCD ID   : $epicsrcrate[3] \n";}
			
				print EDETECTLOG "       Source and BKG Count Rates in entire energy range considered: \n";
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "        $instrument  : \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          $instrument SRC Rate (cts/s): $epicsrcrate[1] +/- $epicsrcrate[2] \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          $instrument BKG Rate (cts/s): $pnbkgrate[1] +/- $pnbkgrate[2] \n";}
				print EDETECTLOG "       Source Count Rates in different energy ranges considered: \n";
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "        $instrument  : \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $epicsrcrate_E1[1] +/- $epicsrcrate_E1[2] ($epic_ene_low[0] - $pn_ene_high[0]) \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $epicsrcrate_E2[1] +/- $epicsrcrate_E2[2] ($epic_ene_low[1] - $pn_ene_high[1]) \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $epicsrcrate_E3[1] +/- $epicsrcrate_E3[2] ($epic_ene_low[2] - $pn_ene_high[2]) \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $epicsrcrate_E4[1] +/- $epicsrcrate_E4[2] ($epic_ene_low[3] - $pn_ene_high[3]) \n";}
				if ($epic_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $epicsrcrate_E5[1] +/- $epicsrcrate_E5[2] ($epic_ene_low[4] - $pn_ene_high[4]) \n";}
	
		    }	
		    
		    if ($Log_Flag == 0) {
				print MYOUTFILE "       Source IDs (from emllist.fits table): \n";
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "        $instrument SRC ID  : $epicsrcrate[0] \n";}
				print MYOUTFILE "       CCD IDs (from emllist.fits table): \n";
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "        $instrument CCD ID  : $epicsrcrate[3] \n";}
				print MYOUTFILE "       Source and BKG Count Rates in entire energy range considered: \n";
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "        $instrument  : \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          $instrument SRC Rate (cts/s): $epicsrcrate[1] +/- $epicsrcrate[2] \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          $instrument BKG Rate (cts/s): $pnbkgrate[1] +/- $pnbkgrate[2] \n";}
				print MYOUTFILE "       Source Count Rates in different energy ranges considered: \n";
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "        $instrument  : \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $epicsrcrate_E1[1] +/- $epicsrcrate_E1[2] ($epic_ene_low[0] - $pn_ene_high[0]) \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $epicsrcrate_E2[1] +/- $epicsrcrate_E2[2] ($epic_ene_low[1] - $pn_ene_high[1]) \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $epicsrcrate_E3[1] +/- $epicsrcrate_E3[2] ($epic_ene_low[2] - $pn_ene_high[2]) \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $epicsrcrate_E4[1] +/- $epicsrcrate_E4[2] ($epic_ene_low[3] - $pn_ene_high[3]) \n";}
				if ($epic_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $epicsrcrate_E5[1] +/- $epicsrcrate_E5[2] ($epic_ene_low[4] - $pn_ene_high[4]) \n";}
		    }
		  } # Only Imaging events
		}
	}# Loop over all event files...
	
	#.. Is the source detected in the different instruments the same ?

	crossDetection(\%pnMap,\%m1Map,\%m2Map,\$source_ra,\$source_dec);
	
    if ($Log_Edet_Flag == 0) {close(EDETECTLOG);}
    &error_code(0); 

    if ($epic_edetect_flag eq 0 ) {return(1);}
    else{return(0);} 
}

## end of run_edetectchain

## @method void crossDetection()
# Check if the source has been detected in the different instruments.
#
# @return void

#====================================================================== 
sub crossDetection()
{
	#.. Is the source detected in the different instruments the same ?
 	my($pnMap,$m1Map,$m2Map,$source_ra,$source_dec)=@_;
	
    my $pnmos1det     = "NO";
    my $pnmos1_dist   = 1000.;
    my $pnmos2det     = "NO";
    my $pnmos2_dist   = 1000.;
    my $mos1mos2det   = "NO";
    my $mos1mos2_dist   = 1000.;
	my $pn_edetect_flag = "0";
	my $m1_edetect_flag = "0";
	my $m2_edetect_flag = "0";

	my @pn_closesource = ();
	my @m1_closesource = ();
	my @m2_closesource = ();
	
	my $pn_exposure;
	my $m1_exposure;
	my $m2_exposure;
	foreach $pn_exposure ( keys %$pnMap) {		
		@pn_closesource = @{$$pnMap{$pn_exposure}};		
	}
	foreach $m1_exposure ( keys %$m1Map) {
		@m1_closesource = @{$$m1Map{$m1_exposure}};
	
	}
	foreach $m2_exposure ( keys %$m2Map) {
		@m2_closesource = @{$$m2Map{$m2_exposure}};
	}
   	
   	#.. initial value
   	$Log_Edet_Flag = 1;
   	$Log_Flag = 1;
	if ( (keys( %$pnMap ) != 0)  && (keys( %$m1Map ) != 0) ){			
		$pnmos1_dist = &Angular_separation($pn_closesource[0],$m1_closesource[0],$pn_closesource[1],$m1_closesource[1]);
 			if ($pnmos1_dist <= $gv->EDC_cameramatch()) {$pnmos1det = "YES";}
 		$Log_Edet_Flag = 0;
 		$Log_Flag = 0;	
	} 
    
    if ( (keys( %$pnMap ) != 0)  && (keys( %$m2Map ) != 0) ){	
		$pnmos2_dist = &Angular_separation($pn_closesource[0],$m2_closesource[0],$pn_closesource[1],$m2_closesource[1]);
		if ($pnmos2_dist <= $gv->EDC_cameramatch()) {$pnmos2det = "YES";}
		$Log_Edet_Flag = 0;
		$Log_Flag = 0;	
    } 
    
    if ( (keys( %$m1Map ) != 0)  && (keys( %$m1Map ) != 0) ){
		$mos1mos2_dist = &Angular_separation($m1_closesource[0],$m2_closesource[0],$m1_closesource[1],$m2_closesource[1]);
		if ($mos1mos2_dist <= $gv->EDC_cameramatch()) {$mos1mos2det = "YES";}
		$Log_Edet_Flag = 0;
		$Log_Flag = 0;	 
    }
    
    
    if ($Log_Edet_Flag == 0) {
	    print "\n   \#> Cross Correlation Information for EPIC: EXPOSURES: PN[$pn_exposure] EMOS1[$m1_exposure] EMOS2[$m2_exposure] \n";
	    print "       EXPOSURE SOURCE           PN               MOS1             MOS2   \n";
	    print "  RA       DEC     RA       DEC      RA       DEC    RA        DEC \n";
	    print " $$source_ra $$source_dec $pn_closesource[0] $pn_closesource[1] $m1_closesource[0] $m1_closesource[1] $m2_closesource[0] $m2_closesource[1] \n";
	    print "  Distance to closest source in each instrument: \n";
	    print "  (If RA,DEC of the source is within ", join(", ", $gv->EDC_sourcematch)," deg of RA,DEC of detected \n";
	    print "  camera source, it is assumed the source is the same) \n";
	    print "    SOURCE-PN : $pn_closesource[2] deg. Same source ?: $pn_edetect_flag \n";
	    print "    SOURCE-M1 : $m1_closesource[2] deg. Same source ?: $m1_edetect_flag \n";
	    print "    SOURCE-M2 : $m2_closesource[2] deg. Same source ?: $m2_edetect_flag \n";
	    print "  Comparing Source detection on different cameras: \n";
	    print "  (If RA,DEC of one camera is within ", join(", ", $gv->EDC_cameramatch)," deg of RA,DEC of another camera \n";
	    print "  it is assumed the source in both cameras is the same) \n";
	    print "    Distance PN-MOS1    : $pnmos1_dist deg. Same source ?: $pnmos1det \n";
	    print "    Distance PN-MOS2    : $pnmos2_dist deg. Same source ?: $pnmos2det \n";
	    print "    Distance MOS1-MOS2  : $mos1mos2_dist deg. Same source ?: $mos1mos2det \n";
	    
	    print EDETECTLOG "\n   \#> Cross Correlation Information for EPIC: \n";
		print EDETECTLOG "       SOURCE           PN               MOS1             MOS2   \n";
		print EDETECTLOG "  RA       DEC     RA       DEC      RA       DEC    RA        DEC \n";
		print EDETECTLOG " $$source_ra $$source_dec $pn_closesource[0] $pn_closesource[1] $m1_closesource[0] $m1_closesource[1] $m2_closesource[0] $m2_closesource[1] \n";
		print EDETECTLOG "  Distance to closest source in each instrument: \n";
		print EDETECTLOG "  (If RA,DEC of the source is within ", join(", ", $gv->EDC_sourcematch)," deg of RA,DEC of detected \n";
		print EDETECTLOG "  camera source, it is assumed the source is the same) \n";
		print EDETECTLOG "    SOURCE-PN : $pn_closesource[2] deg. Same source ?: $pn_edetect_flag \n";
		print EDETECTLOG "    SOURCE-M1 : $m1_closesource[2] deg. Same source ?: $m1_edetect_flag \n";
		print EDETECTLOG "    SOURCE-M2 : $m2_closesource[2] deg. Same source ?: $m2_edetect_flag \n";
		print EDETECTLOG "  Comparing Source detection on different cameras: \n";
		print EDETECTLOG "  (If RA,DEC of one camera is within ", join(", ", $gv->EDC_cameramatch)," deg of RA,DEC of another camera \n";
		print EDETECTLOG "  it is assumed the source in both cameras is the same) \n";
		print EDETECTLOG "    Distance PN-MOS1    : $pnmos1_dist deg. Same source ?: $pnmos1det \n";
		print EDETECTLOG "    Distance PN-MOS2    : $pnmos2_dist deg. Same source ?: $pnmos2det \n";
		print EDETECTLOG "    Distance MOS1-MOS2  : $mos1mos2_dist deg. Same source ?: $mos1mos2det \n";
    }

	if ($Log_Flag == 0) {
		print MYOUTFILE "\n   \#> Cross Correlation Information for EPIC: \n";
		print MYOUTFILE "       SOURCE           PN               MOS1             MOS2   \n";
		print MYOUTFILE "  RA       DEC     RA       DEC      RA       DEC    RA        DEC \n";
		print MYOUTFILE " $$source_ra $$source_dec $pn_closesource[0] $pn_closesource[1] $m1_closesource[0] $m1_closesource[1] $m2_closesource[0] $m2_closesource[1] \n";
		print MYOUTFILE "  Distance to closest source in each instrument: \n";
		print MYOUTFILE "  (If RA,DEC of the source is within ", join(", ", $gv->EDC_sourcematch)," deg of RA,DEC of detected \n";
		print MYOUTFILE "  camera source, it is assumed the source is the same) \n";
		print MYOUTFILE "    SOURCE-PN : $pn_closesource[2] deg. Same source ?: $pn_edetect_flag \n";
		print MYOUTFILE "    SOURCE-M1 : $m1_closesource[2] deg. Same source ?: $m1_edetect_flag \n";
		print MYOUTFILE "    SOURCE-M2 : $m2_closesource[2] deg. Same source ?: $m2_edetect_flag \n";
		print MYOUTFILE "  Comparing Source detection on different cameras: \n";
		print MYOUTFILE "  (If RA,DEC of one camera is within ", join(", ", $gv->EDC_cameramatch)," deg of RA,DEC of another camera \n";
		print MYOUTFILE "  it is assumed the source in both cameras is the same) \n";
		print MYOUTFILE "    Distance PN-MOS1    : $pnmos1_dist deg. Same source ?: $pnmos1det \n";
		print MYOUTFILE "    Distance PN-MOS2    : $pnmos2_dist deg. Same source ?: $pnmos2det \n";
		print MYOUTFILE "    Distance MOS1-MOS2  : $mos1mos2_dist deg. Same source ?: $mos1mos2det \n";
	}

}

## @method void epic_create_energy_bands($instrument,$exposure,$mode)
# Produce EPIC images in 5 user define energy bands.
# @param $instrument
# @param $exposure
# @param $mode
# @return Void

#========================================================================
sub epic_create_energy_bands(){
    my $instrument = $_[0];
    my $exposure = $_[1];
    my $mode = $_[2];

    print "   \#> $instrument: \n";
    print "      Events File Used: $epic_gti_cleanevents_file \n";
    print "      GTI File Used   : $epic_GTI_file \n";

    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> $instrument: \n";
		print MYOUTFILE "      Events File Used: $epic_gti_cleanevents_file \n";
		print MYOUTFILE "      GTI File Used   : $epic_GTI_file \n";
    }
	
    my $number_of_bands = $#epic_ene_low;
    my $xbinsize        = 80;
    my $ybinsize        = 80;

    chdir($images_directory);

    print "      Energy Band: $epic_ene_low[0] - $pn_ene_high[$number_of_bands] \n";
    if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $epic_ene_low[0] - $pn_ene_high[$number_of_bands] \n";}

    @args = ("evselect"
	     ,"table=$epic_gti_cleanevents_file:EVENTS"
	     ,"imagebinning=binSize"
	     ,"imageset=$epic_fullimage"
	     ,"withimageset=yes"
	     ,"xcolumn=X","ycolumn=Y"
	     ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
	     ,"expression=(PI in \[$epic_ene_low[0]:$pn_ene_high[$number_of_bands]\])");
    system(@args)== 0 or &error_code(5);
    
    my $ene_band = 0;
    while ($ene_band <= $number_of_bands){
	
		print "      Energy Band($ene_band) : $epic_ene_low[$ene_band] - $pn_ene_high[$ene_band] \n";
		if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $ene_band : $epic_ene_low[$ene_band] - $pn_ene_high[$ene_band] \n";}
		
		my $image = $instrument."_".$exposure."_".$mode."_image_b".$ene_band.".fits";
	
		@args = ("evselect"
		 ,"table=$epic_gti_cleanevents_file:EVENTS"
		 ,"imagebinning=binSize"
		 ,"imageset=$image"
		 ,"withimageset=yes"
		 ,"xcolumn=X","ycolumn=Y"
		 ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
		 ,"expression=(PI in \[$epic_ene_low[$ene_band]:$pn_ene_high[$ene_band]\])");
		system(@args)== 0 or &error_code(5);	
		$ene_band++;
		

    }
    
    return (0);
}

## @method void epic_edetect_chain()
# Runs the SAS task edetect_chain for EPIC
# @param $instrument
# @param $exposure
# @param $mode
# @return Void

#========================================================================
sub epic_edetect_chain(){
    my $instrument = $_[0];
    my $exposure = $_[1];
    my $mode = $_[2];
    
    my $inst= $instrument;
    if (($instrument eq "m1") || ($instrument eq "m2")) {
    	$inst = "mos";
    }
    my $currentInstDir = $inst."_directory";

    chdir($$currentInstDir);
	
    my @epicAttitudelist = glob('*AttHk.ds');
    if ($#epicAttitudelist < 0) {&error_code(5);}
    my $epicAttitudefile = $$currentInstDir.$epicAttitudelist[0]; 

    my @filter=&event_file_inf($epic_gti_cleanevents_file,$instrument);
    my $epic_filter=$filter[1];

    chdir($images_directory);
    
    my @epic_ecf      = &epic_ecfvalues($epic_filter,$instrument);
    my $number_of_bands = $#epic_ene_low; 
    
    print "\n   \#> Running edetect chain for $instrument .... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Running edetect chain for $instrument .... \n";}
 
    my $img0=$images_directory.$instrument."_".$exposure."_".$mode."_image_b0\.fits";
    my $img1=$images_directory.$instrument."_".$exposure."_".$mode."_image_b1\.fits";
    my $img2=$images_directory.$instrument."_".$exposure."_".$mode."_image_b2\.fits";
    my $img3=$images_directory.$instrument."_".$exposure."_".$mode."_image_b3\.fits";
    my $img4=$images_directory.$instrument."_".$exposure."_".$mode."_image_b4\.fits";
    @args = ("edetect_chain","imagesets=$img0 $img1 $img2 $img3 $img4" 
	     ,"eventsets=$epic_gti_cleanevents_file"
	     ,"attitudeset=$epicAttitudefile" 
	     ,"pimin=$epic_ene_low[0] $epic_ene_low[1] $epic_ene_low[2] $epic_ene_low[3] $epic_ene_low[4]"
	     ,"pimax=$pn_ene_high[0] $pn_ene_high[1] $pn_ene_high[2] $pn_ene_high[3] $pn_ene_high[4]" 
	     ,"ecf=$epic_ecf[0] $epic_ecf[1] $epic_ecf[2] $epic_ecf[3] $epic_ecf[4]" 
	     ,"eboxl_list=$epic_eboxllist_file"
	     ,"eboxm_list=$epic_eboxmlist_file" 
	     ,"esp_nsplinenodes=16"
	     ,"eml_list=$epic_emllist_file"
	     ,"esen_mlmin=$epic_likelihood_detection"
	     ,"eml_fitextent=\"yes\"");
    system(@args)== 0 or &error_code(5);

    print "      $instrument ECF Values  : "; foreach (@epic_ecf) {print "$_ ";}; print "\n\n";
    if ($Log_Flag == 0) {print MYOUTFILE "      $instrument ECF Values  : "; foreach (@epic_ecf) {print MYOUTFILE "$_ ";}; print MYOUTFILE "\n\n";}
			 
    return(0);
}


## @method epic_image()
# Produce an EPIC Image and region file from Source List. Produces an EPIC Image with regions from region file overlaid.
# @param instrument
# @param exposure
# @param mode
# @return RA (deg) and DEC (deg) of closest EPIC-pn detected source to the source of interes. Angular distance (deg) between EPIC-pn and source of interest.

#========================================================================
sub epic_image(){
	my $instrument = $_[0];
	my $exposure = $_[1];
	my $mode = $_[2];
	
    chdir ($images_directory);
     
    my $epicregion_file = $images_directory.$instrument."_".$exposure."_".$mode."region\.txt"; 
    my $source_ra     = $gv->SRC_RA;
    my $source_dec    = $gv->SRC_DEC;
    my $source_name   = $gv->SRC_Name;

#.. Run the SAS commands srcdisplay to produce region file
	
    @args=("srcdisplay"
	   ,"boxlistset=$epic_emllist_file" 
	   ,"imageset=$epic_fullimage"
	   ,"sourceradius=0.01"
	   ,"withregionfile=yes"
	   ,"regionfile=$epicregion_file"); 
    system(@args);                         # == 0 or &error_code(5); NOTE: If I put this bit, it wont run on the grid

#.. From the list of detected sources in the region file find the closest source to yours
#.. and add your source to the region file to be plotted. 
    if (-e $epicregion_file){
		@epicclose_source=&find_closest_source($epicregion_file);
		&addsourcetoregionfile($epicregion_file,$epicclose_source[0],$epicclose_source[1],0.,0.008,"red","","circle","fk5");
		&addsourcetoregionfile($epicregion_file,$source_ra,$source_dec,0.,0.009,"green",$source_name,"circle","fk5");	
    } else {
		print "   \#> $instrument Region file $epicregion_file does not exist \n";
		if ($Log_Flag == 0) {print MYOUTFILE "   \#> $instrument Region file $epicregion_file does not exist \n";}	
		return(100.,100.,100.);
    }

    rename($epicregion_file,$source_epicregion_file);
    print "   \#> $instrument Region file produced: $source_epicregion_file \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> $instrument Region file produced: $source_epicregion_file \n";}

#.. Run DS9. Arguments: image file, region file, title, output name
    &produce_ps_file($epic_fullimage,$source_epicregion_file,$instrument,$images_directory.$source_name."_".$instrument."_".$exposure."_".$mode."_image.ps");

    return($epicclose_source[0],$epicclose_source[1],$epicclose_source[2]);
}


## @method epic_ecfvalues($filter, $instrument)
# Definition of Energy Conversion Factors (ECF) for EPIC cameras
# @param filter EPIC Filter
# @param instrument
# @return Array with ECF for 5 user defined energy bins

#========================================================================
sub epic_ecfvalues(){

    my $filter = $_[0];
    my $instrument = $_[1];

    print "    $instrument  Filter ECF Set: $filter \n";
    $filter =~ s/\s+//;
    if ($instrument eq "pn")
    {
	    if ($filter eq "Open") {return(15.711,7.5030,1.9837,0.9415,0.2399);} 
    	if ($filter eq "Thick") {return(4.990,5.315,1.847,0.927,0.2390);} 
    	if ($filter eq "Medium") {return(8.010,6.442,1.930,0.9394,0.2400);} 
    	if ($filter eq "Thin" or $filter eq "Thin1" or $filter eq "Thin2") {return(9.041,6.596,1.953,0.941,0.2401);}
    }
    if ($instrument eq "m1")
    {
    	if ($filter eq "Open") {return(3.0377,2.2011,0.75244,0.26492,0.02774);}
    	if ($filter eq "Thick") {return(0.984,1.622,0.7024,0.260,0.02760);} 
    	if ($filter eq "Medium") {return(1.564,1.983,0.7325,0.2639,0.02772);}
    	if ($filter eq "Thin" or $filter eq "Thin1" or $filter eq "Thin2") {return(1.756,1.980,0.741,0.2644,0.02773);}   	 
    }

    if ($instrument eq "m2")
    {
    	if ($filter eq "Open") {return(3.0618,2.1963,0.75597,0.27772,0.029915);} 
    	if ($filter eq "Thick") {return(0.994,1.620,0.7059,0.2727,0.02976);} 
    	if ($filter eq "Medium") {return(1.579,1.935,0.7361,0.2767,0.02988);} 
    	if ($filter eq "Thin" or $filter eq "Thin1" or $filter eq "Thin2") {return(1.772,1.977,0.7445,0.2771,0.0299);} 	 
    }

    return(1,1,1,1,1);
}


## @method find_closest_source($region_file)
# Given a Region File, find the closest source in the region file to the source of interest.
# @param region_file Name of Region File
# @return RA (deg), DEC (deg) Angular distance (deg) of closest detected source to the source of interest.

#========================================================================
sub find_closest_source(){

    my $source_ra=$gv->SRC_RA;
    my $source_dec=$gv->SRC_DEC;
    
    my $region_file = $_[0];

    my @ra_list;
    my @dec_list;

    if (-e $region_file) {
	
		open (REGIONFILE, $region_file); # or die "couldn't open file $region_file: $! \n";
		
		while (<REGIONFILE>) {
		    my $line = $_;
		    next if /^global/;  # skip comment lines
	    	next if /^[\#*]/;   # skip comment lines
	    	next if /^(\s)*$/;  # skip blank lines
	    	next if !/(color\=white)$/;  # use only lines that come from edetect_chain
	    	chomp($line);
	    	$line =~ s/fk5\;circle\(//;
	    	$line =~ s/\,0.01\) \# color\=white//;
	    	my($ra, $dec) = split(/\,/, $line);
	    	push @ra_list, $ra;
	    	push @dec_list, $dec;
		}
		close(REGIONFILE);
		$counter = 0;
		$mindist = 100.;
		foreach(@ra_list){
	    	$angsep = &Angular_separation($ra_list[$counter],$source_ra,$dec_list[$counter],$source_dec);
	    	if ($angsep < $mindist){
				$mindist    = $angsep; 
				$mincounter = $counter;
	    	}
	    $counter ++;
		}
    } else {
		$mincounter            = 0;
		$ra_list[$mincounter]  = 0.;
		$dec_list[$mincounter] = 0.;
		$mindist = 1000.;
    }
    
    return($ra_list[$mincounter],$dec_list[$mincounter],$mindist);
}

## @method get_source_count_rate($src_file,$instr,$energyband,$ra,$dec)
# Get source count rate from the Source List
# @param src_file Source List
# @param instr Instrument
# @param energyband Energy Band of interest
# @param ra Source RA
# @param dec Source DEC
# @return Source ID in the Source List, Source Count Rate and Error, CCD Number where the source is.

#========================================================================
sub get_source_count_rate(){

    my $src_file    = $_[0];
    my $instr       = $_[1]; 
    my $energyband  = $_[2];
    my $ra          = $_[3];
    my $dec         = $_[4];

    my $RALOW       = $ra-0.0005;
    my $RAHIG       = $ra+0.0005;
    my $DELOW       = $dec-0.0005;
    my $DEHIG       = $dec+0.0005;
    my $temp_file   = $images_directory."temp";
    my $out_file    = $src_file.".temp";
    my $src_ml_id   = 0.;
    my $src_ccd_id  = 0.;
    my $tot_srcrate = 0.;
    my $tot_esrcrate= 0.;

    if ($instr eq "pn") {$Instrument_ID = 1;}
    if ($instr eq "m1") {$Instrument_ID = 2;}
    if ($instr eq "m2") {$Instrument_ID = 3;}

#.. Create a reduced list from the input table of detected instrument sources with the one closest to our source
    @args = ("fselect $src_file $out_file \"ID_BAND==$energyband&&ID_INST==$Instrument_ID&&RA>$RALOW&&RA<$RAHIG&&DEC>$DELOW&&DEC<$DEHIG\" clobber=yes");
    system(@args)== 0 or &error_code(5);

    chomp(my $source   = `fkeyprint infile=$out_file+1  keynam=NAXIS2 outfil=STDOUT exact=yes | grep NAXIS2 | tail -1 | cut -c12-30`);
    $source     =~ s/\s+//;
    $source     =~ s/\'+//;

    if ($source == 1){
	
	#.. Get Source ID from reduced list
		@args = ("fdump $out_file $temp_file \"ML_ID_SRC\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
		system(@args)== 0 or &error_code(5);
		chomp($src_ml_id = `cat $temp_file`);
		$src_ml_id =~ s/\s+//;
	
	#.. Get CCD ID in which source falls from reduced list
		@args = ("fdump $out_file $temp_file \"CCDNR\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
		system(@args)== 0 or &error_code(5);
		chomp($src_ccd_id = `cat $temp_file`);
		$src_ccd_id =~ s/\s+//;
	
	#.. Get Source Rate from reduced list
		@args = ("fdump $out_file $temp_file \"RATE\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
		system(@args)== 0 or &error_code(5);
		chomp($tot_srcrate = `cat $temp_file`);
		$tot_srcrate =~ s/\s+//;
		
	#.. Get Source Rate Error from reduced list
		@args = ("fdump $out_file $temp_file \"RATE_ERR\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
		system(@args)== 0 or &error_code(5);
		chomp($tot_esrcrate = `cat $temp_file`);
		$tot_esrcrate =~ s/\s+//;
	
	#.. Delete Temporary Files
		unlink("$temp_file") or die "Cannot rm file $temp_file: $!";
		unlink("$out_file") or die "Cannot rm file $out_file: $!";
		
    } else {
		$src_ml_id    = 10000.;
		$tot_srcrate  = 0.;
		$tot_esrcrate = 0.;
		if ($source > 1) {
	    	print "      More than one source identified: $source \n"; 	    
	    	if ($Log_Flag == 0) {print  MYOUTFILE "      More than one source identified: $source \n";}
		} elsif ($source == 0) {
	    	print "      No Sources identified in $_[0] \n";
	    	if ($Log_Flag == 0) {print MYOUTFILE "      No Sources identified in $_[0] \n";}
		}
    }
 
    return($src_ml_id,$tot_srcrate,$tot_esrcrate,$src_ccd_id);

}

## @method get_bkg_count_rate($src_file,$instr,$ra,$dec)
# Get background count rate from the Source List
# @param src_file Source List
# @param instr Instrument
# @param ra Source RA
# @param dec Source DEC
# @return Source ID in the Source List, Background Count Rate and Error.

#========================================================================
sub get_bkg_count_rate{
    
    my $src_file     = $_[0];
    my $instr        = $_[1];
    my $ra           = $_[2];
    my $dec          = $_[3];

    my $RALOW        = $ra-0.0005;
    my $RAHIG        = $ra+0.0005;
    my $DELOW        = $dec-0.0005;
    my $DEHIG        = $dec+0.0005;
    my $energyband   = 1;
    my $number_of_bands = 5;
    my $temp_file    = $images_directory."temp";
    my $out_file     = $src_file.".temp";
    my $src_ml_id    = 0.;
    my $tot_bkgrate  = 0.;
    my $tot_ebkgrate = 0.;
    my $BG_MAP       = 0.;
    my $EXP_MAP      = 1.;
    my $CUTRAD       = 0.;

    if ($instr eq "pn") {$Instrument_ID = 1;}
    if ($instr eq "m1") {$Instrument_ID = 2;}
    if ($instr eq "m2") {$Instrument_ID = 3;}

    while ($energyband <= $number_of_bands) {
#.. Create a reduced list from the input table of detected instrument sources with the one closest to our source
	@args = ("fselect $src_file $out_file \"ID_BAND==$energyband&&ID_INST==$Instrument_ID&&RA>$RALOW&&RA<$RAHIG&&DEC>$DELOW&&DEC<$DEHIG\" clobber=yes");
	system(@args)== 0 or &error_code(5);

	chomp(my $source   = `fkeyprint infile=$out_file+1 keynam=NAXIS2 outfil=STDOUT exact=yes | grep NAXIS2 | tail -1 | cut -c12-30`);
	$source     =~ s/\s+//;
	$source     =~ s/\'+//;

	if ($source == 1){

#.. Get Source ID from reduced list
	    @args = ("fdump $out_file $temp_file \"ML_ID_SRC\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($src_ml_id = `cat $temp_file`);
	    $src_ml_id =~ s/\s+//;
	    
#.. Get Source BG_MAP from reduced list
	    @args = ("fdump $out_file $temp_file \"BG_MAP\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($BG_MAP = `cat $temp_file`);
	    $BG_MAP =~ s/\s+//;
	    
#.. Get Source EXP_MAP from reduced list	
	    @args = ("fdump $out_file $temp_file \"EXP_MAP\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($EXP_MAP = `cat $temp_file`);
	    $EXP_MAP =~ s/\s+//;
	    
#.. Get Source CUTRAD from reduced list	
	    @args = ("fdump $out_file $temp_file \"CUTRAD\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($CUTRAD = `cat $temp_file`);
	    $CUTRAD =~ s/\s+//;
	    
#.. Workout bkg rate for a given energy band and accumulate 	
	    print "   \#> Values used for Background count rate (Energy $energyband): \n";
	    print "      $_[1] Exposure map  : $EXP_MAP \n";
	    print "      $_[1] Background map: $BG_MAP \n";
	    print "      $_[1] Cut Radius    : $CUTRAD \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> Values used for Background count rate (Energy $energyband): \n";
		print MYOUTFILE "      $_[1] Exposure map  : $EXP_MAP \n";
		print MYOUTFILE "      $_[1] Background map: $BG_MAP \n";
		print MYOUTFILE "      $_[1] Cut Radius    : $CUTRAD \n";
	    }
	    if ($EXP_MAP != 0 || $EXP_MAP eq "" || $BG_MAP != 0 || $BG_MAP eq "" || $CUTRAD != 0 || $CUTRAD eq ""){ 
		$tot_bkgrate    = $tot_bkgrate+(($BG_MAP*(3.14159265*$CUTRAD**2))/$EXP_MAP);
		$tot_ebkgrate   = $tot_ebkgrate+($BG_MAP*((3.14159265*$CUTRAD**2)/$EXP_MAP)**2);
	    } else {
		print "      Could not workout bakground rate. \n";
		if ($Log_Flag == 0) {print MYOUTFILE "      Could not workout bakground rate. \n";}
		$tot_bkgrate  = 0;
		$tot_ebkgrate = 0;
	    }	
	    
#.. Delete Temporary Files
	    unlink("$temp_file") or die "Cannot rm file $temp_file: $!";
	    unlink("$out_file") or die "Cannot rm file $out_file: $!";
	    
	} else {
	    $src_ml_id    = 10000.;
	    $tot_bkgrate  = 0.;
	    $tot_ebkgrate = 0.;
	    if ($source > 1) {
		print "      More than one source identified: $source \n"; 	    
		if ($Log_Flag == 0) {print  MYOUTFILE "      More than one source identified: $source \n";}
	    } elsif ($source == 0) {
		print "      No Sources identified in $_[0] \n";
		if ($Log_Flag == 0) {print MYOUTFILE "      No Sources identified in $_[0] \n";}
	    }
	}

	$energyband++;
	
    }
    
    $tot_ebkgrate   = sqrt($tot_ebkgrate);
    
    return($src_ml_id,$tot_bkgrate,$tot_ebkgrate);
}
1;



