## @file
# Produce pile-up plots

## @method run_epatplot()
# run SAS task epatplot
# @return Flag of error/success.

#========================================================================

sub run_epatplot()
{
	my $event_file = $_[0];
	my $instr      = $_[1];
	my $analysis   = $_[2];	
	my $subMode    = $_[3];
	

	my $instrX = $analysis."_".$instr."_srcregion_X";
	my $instrY = $analysis."_".$instr."_srcregion_Y";
	my $instrR1 = $analysis."_".$instr."_srcregion_R1";
	my $instrR2 = $analysis."_".$instr."_srcregion_R2";
	my $instrXLength = $analysis."_".$instr."_srcregion_XLength";
	my $instrYLength = $analysis."_".$instr."_srcregion_YLength";
	my $outfile = $instr."_epatplot.ps";
	my $instrRegion = $analysis."_".$instr."_srcregion";
	my $region = $gv->$instrRegion;

	my $SRC_XC    = $gv->$instrX;
	my $SRC_YC    = $gv->$instrY;
	if ($gv->$instrRegion eq "circle" || $gv->$instrRegion eq "annulus") {$RSRC1     = $gv->$instrR1};
	if ($gv->$instrRegion eq "circle" || $gv->$instrRegion eq "annulus") {$RSRC2     = $gv->$instrR2};
	if ($gv->$instrRegion eq "box") {$RSRC1     = $gv->$instrXLength};
	if ($gv->$instrRegion eq "box") {$RSRC2     = $gv->$instrYLength};

	print "SRC_XC: $SRC_XC\n";
	print "SRC_YC: $SRC_YC\n";
	print "RSRC1: $RSRC1\n";
	print "RSRC2: $RSRC2\n";
	print "region: $region\n";
	print "instr: $instr\n";
	print "event_file: $event_file\n";
	print "subMode: $subMode\n";
	my $filter_event_file = &produce_source_event_file($SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$region,$instr,$event_file,$subMode);
	print "filter_event_file: $filter_event_file\n";
	print "outfile: $outfile\n";
	@args = ("epatplot set=$filter_event_file  device=\"\/VCPS\" outdir=$epatplot_directory plotfile=$outfile");
 		
    system(@args)== 0 or &error_code(8);		
	
	return(1);
}

## @method void produce_lightcurve_file($E1,$E2,$XC,$YC,$R1,$R2,$T,$region,$instr,$event_file,$outfile,$subMode)
# Produce a time series given region (circle, annulus or box) and energy range (E1, E2).
# @param E1 Low Bound Energy Range (eV)	   
# @param E2 High Bound Energy Range (eV)	   
# @param XC X-Center of Source Region (Image Pixels)	   
# @param YC Y-Center of Source Region (Image Pixels)	   
# @param R1 Radius/Length of Source Region (Image Pixels)
# @param R2 Radius/Length of Source Region (Image Pixels)
# @param T Time Bin (secs)	   
# @param region	Region Type (circle, annulus, box)   
# @param instr Instrument	   
# @param event_file Event File Name
# @param outfile Output File Name (Rate Set)	    
# @subMode Observation Mode 
# @return Void

#========================================================================
sub produce_source_event_file(){
    
# Produce an event file for a given region and energy range

    my $XC      = $_[0];
    my $YC      = $_[1];
    my $R1      = $_[2];
    my $R2      = $_[3];
    my $region  = $_[4];
    my $instr   = $_[5];
    my $event_file = $_[6];
    my $subMode = $_[7];
	my $xVal;
    my $yVal;
    my $areaexpr;
    my $outputname = $epatplot_directory.$instr."_epatplot.fits";

    if (($instr eq "m1" or $instr eq "m2") &&  (uc($subMode) eq 'TIMING') )
    {    	       		
    	#$areaexpr="((RAWX,TIME) in box($XC,$YC,$R1,$R2,0))";
    	my $X_min = $XC - $R1/2.;
    	my $X_max = $XC + $R1/2.; 
    	$areaexpr="(RAWX in [$X_min:$X_max])";
    } 
    else
    {
	    if ($region eq "circle" || $region eq "annulus") {$areaexpr="((X,Y) in annulus($XC,$YC,$R1,$R2))";}
   		if ($region eq "box") 
   		{
    		if ($subMode eq 'IMAGING' || $subMode eq 'imaging') {
    			$xVal = "X";
    			$yVal = "Y";
    		}
    		else 
    		{
    			$xVal = "RAWX";
    			$yVal = "RAWY";
    		}
    	
    		$areaexpr="(($xVal,$yVal) in box($XC,$YC,$R1,$R2,0))";
   		}
    }


    print "areaexpr: $areaexpr\n";
    @args = ("evselect table=$event_file:EVENTS expression=\"$areaexpr\" keepfilteroutput=yes withfilteredset=yes filteredset=$outputname ");

    system(@args)== 0 or &error_code(7);
    
    return($outputname);
}

1;


